/*
  QtCurve (C) Craig Drummond, 2004 Craig.Drummond@lycos.co.uk


  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qtabwidget.h>
#include <qheader.h>
#include <kglobal.h>
#include <klocale.h>
#include <kcolorbutton.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>
#include <kdialog.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kinputdialog.h>
#include <knuminput.h>
#include <unistd.h>
#include "qtcurveconfig.h"
#include "../../config.h"
#define CONFIG_READ
#define CONFIG_WRITE
#include "../config_file.c"

extern "C"
{
    QWidget * allocate_kstyle_config(QWidget *parent)
    {
        KGlobal::locale()->insertCatalogue("kstyle_qtcurve_config");
        return new QtCurveConfig(parent);
    }
}

static void insertShadeEntries(QComboBox *combo, bool withDarken, bool noneIsText=false)
{
    combo->insertItem(noneIsText ? i18n("Text") : i18n("None"));
    combo->insertItem(i18n("Selected background"));
    combo->insertItem(i18n("Custom:"));
    if(withDarken)
        combo->insertItem(i18n("Darken"));
}

static void insertAppearanceEntries(QComboBox *combo, bool bev, bool raised)
{
    combo->insertItem(i18n("Flat"));
    combo->insertItem(i18n("Dull glass"));
    combo->insertItem(i18n("Shiny glass"));
    combo->insertItem(i18n("Gradient"));
    combo->insertItem(i18n("Inverted gradient"));
    if(bev)
    {
        combo->insertItem(i18n("Bevelled"));
        if(raised)
            combo->insertItem(i18n("Raised"));
    }
}

static void insertHeaderColorEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Background"));
    combo->insertItem(i18n("Button"));
    combo->insertItem(i18n("Custom:"));
}

static void insertLineEntries(QComboBox *combo, bool none)
{
    combo->insertItem(i18n("Raised lines"));
    combo->insertItem(i18n("Sunken lines"));
    combo->insertItem(i18n("Small dots"));
    combo->insertItem(i18n("Large dots"));
    combo->insertItem(none ? i18n("None") : i18n("Dashes"));
}

static void insertFocusEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Dots"));
    combo->insertItem(i18n("Colored"));
}

static void insertDefBtnEntries(QComboBox *combo)
{
    combo->insertItem(i18n("Corner indicator"));
    combo->insertItem(i18n("Font color thin border"));
    combo->insertItem(i18n("Selected background thick border"));
    combo->insertItem(i18n("None"));
}

static void insertRoundEntries(QComboBox *combo)
{
    combo->insertItem(i18n("None"));
    combo->insertItem(i18n("Slight"));
    combo->insertItem(i18n("Full"));
}

static void insertScrollbarEntries(QComboBox *combo)
{
    combo->insertItem(i18n("KDE (3 Buttons - 1 top/left, 2 bottom/right)"));
    combo->insertItem(i18n("Windows (1 button at each end)"));
    combo->insertItem(i18n("Platinum (2 Buttons at bottom/right)"));
    combo->insertItem(i18n("Next (2 Buttons at top/left)"));
    combo->insertItem(i18n("No buttons"));
}

QtCurveConfig::ListViewItem::ListViewItem(const Options &o, KListView *parent, bool t)
                           : KListViewItem(parent),
                             opts(o),
                             top(t)
{
    setText(0, opts.name);
}

QString QtCurveConfig::ListViewItem::key(int column, bool ascending) const
{
    return QString((top ? ascending ? "0" : "2" : "1") + text(column));
}

QtCurveConfig::QtCurveConfig(QWidget *parent)
             : QtCurveConfigBase(parent),
               currentStyle(0),
               defaultStyle(0)
{
    titleLabel->setText("QtCurve " VERSION " - (C) Craig Drummond, 2003-2006");
    insertShadeEntries(shadeSliders, false);
    insertShadeEntries(shadeSbSliders, false);
    insertShadeEntries(shadeMenubars, true);
    insertShadeEntries(shadeCheckRadio, false, true);
    insertAppearanceEntries(appearance, true, true);
    insertAppearanceEntries(menubarAppearance, false, false);
    insertAppearanceEntries(toolbarAppearance, false, false);
    insertAppearanceEntries(lvHeaderAppearance, true, false);
    insertAppearanceEntries(selectedTabAppearance, false, false);
    insertAppearanceEntries(normalTabAppearance, false, false);
    insertHeaderColorEntries(lvHeaderColor);
    insertLineEntries(handles, false);
    insertLineEntries(sliderThumbs, true);
    insertLineEntries(sbSliderThumbs, true);
    insertLineEntries(toolbarSeparators, true);
    insertLineEntries(splitters, false);
    insertFocusEntries(focus);
    insertDefBtnEntries(defaultIndicator);
    insertRoundEntries(round);
    insertScrollbarEntries(scrollbarType);
    highlightFactor->setMinValue(MIN_HIGHLIGHT_FACTOR);
    highlightFactor->setMaxValue(MAX_HIGHLIGHT_FACTOR);
    highlightFactor->setValue(((int)(DEFAULT_HIGHLIGHT_FACTOR*100))-100);

    version->header()->hide();

    KPushButton *save=new KPushButton(KGuiItem(i18n("Save"), "filesave", i18n("Save style")),
                                      controlsFrame),
                *imp=new KPushButton(KGuiItem(i18n("Import..."), "fileimport", i18n("Import style")),
                                     controlsFrame),
                *exp=new KPushButton(KGuiItem(i18n("Export..."), "fileexport", i18n("Export style")),
                                     controlsFrame);
    QGridLayout *controlsLayout = new QGridLayout(controlsFrame, 2, 2, 0, KDialog::spacingHint());

    delBtn=new KPushButton(KStdGuiItem::del(), controlsFrame),
    controlsLayout->addWidget(save, 0, 0);
    controlsLayout->addWidget(delBtn, 0, 1);
    controlsLayout->addWidget(imp, 1, 0);
    controlsLayout->addWidget(exp, 1, 1);

    connect(save, SIGNAL(clicked()), this, SLOT(saveStyle()));
    connect(delBtn, SIGNAL(clicked()), this, SLOT(deleteStyle()));
    connect(imp, SIGNAL(clicked()), this, SLOT(importStyle()));
    connect(exp, SIGNAL(clicked()), this, SLOT(exportStyle()));

    loadStandardStyles();

    connect(menubarRoundTopOnly, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lighterPopupMenuBgnd, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(round, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(toolbarBorders, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(sliderThumbs, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(sbSliderThumbs, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvExpander, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvHeaderColor, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvDark, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lvLines, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(lvHeaderAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(drawStatusBarFrames, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(handles, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(appearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(vArrow, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(fillSlider, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lightSliderBorder, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(lightSbSliderBorder, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(stripedProgress, SIGNAL(toggled(bool)), SLOT(stripedProgressChanged()));
    connect(customMenuTextColor, SIGNAL(toggled(bool)), SLOT(customMenuTextColorChanged()));
    connect(animatedProgress, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(embolden, SIGNAL(toggled(bool)), SLOT(emboldenToggled()));
    connect(crLabelHighlight, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(defaultIndicator, SIGNAL(activated(int)), SLOT(dbiChanged()));
    connect(highlightTab, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(menubarAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(toolbarAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(toolbarSeparators, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(splitters, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(highlightEdits, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(fixParentlessDialogs, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(drawDockWindowTitles, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(coloredSidebarButtons, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(coloredMouseOver, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(menubarMouseOver, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(shadeMenubarOnlyWhenActive, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(inactiveHighlight, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(thinnerMenuItems, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(roundMenuItems, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    connect(borderMenuItems, SIGNAL(toggled(bool)), SLOT(borderMenuItemsChanged()));
    connect(roundMenubarItems, SIGNAL(toggled(bool)), SLOT(roundMenubarItemsChanged()));
    connect(borderMenubarItems, SIGNAL(toggled(bool)), SLOT(borderMenubarItemsChanged()));
    connect(selectedTabAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(normalTabAppearance, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(customSlidersColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customSbSlidersColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customMenubarsColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customCheckRadioColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customLvHeaderColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customMenuSelTextColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(customMenuNormTextColor, SIGNAL(changed(const QColor &)), SLOT(updateChanged()));
    connect(shadeSliders, SIGNAL(activated(int)), SLOT(shadeSlidersChanged()));
    connect(shadeSbSliders, SIGNAL(activated(int)), SLOT(shadeSbSlidersChanged()));
    connect(shadeMenubars, SIGNAL(activated(int)), SLOT(shadeMenubarsChanged()));
    connect(shadeCheckRadio, SIGNAL(activated(int)), SLOT(shadeCheckRadioChanged()));
    connect(lvHeaderColor, SIGNAL(activated(int)), SLOT(lvHeaderColorChanged()));
    connect(focus, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(highlightFactor, SIGNAL(valueChanged(int)), SLOT(updateChanged()));
    connect(scrollbarType, SIGNAL(activated(int)), SLOT(updateChanged()));
    connect(version, SIGNAL(selectionChanged()), SLOT(setVersion()));
    version->setSelected(currentStyle, true);
}

QtCurveConfig::~QtCurveConfig()
{
}

void QtCurveConfig::save()
{
    Options opts;

    setOptions(opts);
    opts.name=QString::null;
    writeConfig(NULL, opts);
}

void QtCurveConfig::defaults()
{
    version->setSelected(defaultStyle, true);
}

void QtCurveConfig::emboldenToggled()
{
    if(!embolden->isChecked() && IND_NONE==defaultIndicator->currentItem())
        defaultIndicator->setCurrentItem(IND_CORNER);
    updateChanged();
}

void QtCurveConfig::dbiChanged()
{
    if(IND_NONE==defaultIndicator->currentItem() && !embolden->isChecked())
        embolden->setChecked(true);
    updateChanged();
}

void QtCurveConfig::lvChanged()
{
    lvDark->setEnabled(LV_LINES_NONE!=lvLines->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeSlidersChanged()
{
    customSlidersColor->setEnabled(SHADE_CUSTOM==shadeSliders->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeSbSlidersChanged()
{
    customSbSlidersColor->setEnabled(SHADE_CUSTOM==shadeSbSliders->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeMenubarsChanged()
{
    customMenubarsColor->setEnabled(SHADE_CUSTOM==shadeMenubars->currentItem());
    updateChanged();
}

void QtCurveConfig::shadeCheckRadioChanged()
{
    customCheckRadioColor->setEnabled(SHADE_CUSTOM==shadeCheckRadio->currentItem());
    updateChanged();
}

void QtCurveConfig::lvHeaderColorChanged()
{
    customLvHeaderColor->setEnabled(HEADER_COLOR_CUSTOM==lvHeaderColor->currentItem());
    updateChanged();
}

void QtCurveConfig::stripedProgressChanged()
{
    animatedProgress->setEnabled(stripedProgress->isChecked());
    if(animatedProgress->isChecked() && !stripedProgress->isChecked())
        animatedProgress->setChecked(false);
    updateChanged();
}

void QtCurveConfig::customMenuTextColorChanged()
{
    customMenuNormTextColor->setEnabled(customMenuTextColor->isChecked());
    customMenuSelTextColor->setEnabled(customMenuTextColor->isChecked());
    updateChanged();
}

void QtCurveConfig::borderMenuItemsChanged()
{
    roundMenuItems->setEnabled(borderMenuItems->isChecked());

    if(!borderMenuItems->isChecked())
        roundMenuItems->setChecked(false);
}

void QtCurveConfig::borderMenubarItemsChanged()
{
    roundMenubarItems->setEnabled(borderMenubarItems->isChecked());

    if(!borderMenubarItems->isChecked())
        roundMenubarItems->setChecked(false);

    roundMenubarItemsChanged();
}

void QtCurveConfig::roundMenubarItemsChanged()
{
    menubarRoundTopOnly->setEnabled(roundMenubarItems->isChecked() &&
                                          borderMenubarItems->isChecked());
    if(!roundMenubarItems->isChecked() || !borderMenubarItems->isChecked())
        menubarRoundTopOnly->setChecked(false);

    updateChanged();
}

void QtCurveConfig::updateChanged()
{
    if (settingsChanged())
        emit changed(true);
}

#define QTC_EXTENSION   ".qtcurve"

void QtCurveConfig::saveStyle()
{
    Options opts;

    setOptions(opts);
    saveStyle(KInputDialog::getText(i18n("Style Name"), i18n("Please enter a name for this style:"),
                                    0==version->selectedItem() || currentStyle==version->selectedItem()
                                        ? i18n("New Style")
                                        : version->selectedItem()->text(0), NULL, this).simplifyWhiteSpace(), opts);
}

void QtCurveConfig::deleteStyle()
{
    ListViewItem *item=(ListViewItem *)(version->selectedItem());

    if(item && defaultStyle!=item &&
       !item->opts.file.isEmpty() &&
       KMessageBox::Yes==KMessageBox::warningYesNo(this, i18n("Delete the selected style?")))
        if(0==::unlink(item->opts.file.local8Bit().data()))
        {
            delete item;
            version->setSelected(currentStyle, true);
        }
        else
            KMessageBox::error(this, i18n("Could not delete style"));
}

void QtCurveConfig::importStyle()
{
    QString file(KFileDialog::getOpenFileName(QString::null, "*"QTC_EXTENSION, this));

    if(!file.isEmpty())
    {
        Options opts;

        if(loadOptions(file, opts) && !opts.name.isEmpty())
            saveStyle(opts.name, opts);
        else
            KMessageBox::error(this, i18n("This is not a QtCurve file!"));
    }
}

void QtCurveConfig::exportStyle()
{
    ListViewItem *item=(ListViewItem *)(version->selectedItem());

    if(!item)
        return;

    QString name(item==currentStyle || item==currentStyle
                    ? KInputDialog::getText(i18n("Style Name"), i18n("Please enter a name for this style:"),
                                            i18n("New Style"))
                    : item->text(0));

    if(name.isEmpty())
        return;

    if(name==currentStyle->opts.name || name==defaultStyle->opts.name)
    {
        KMessageBox::error(this, i18n("Sorry, you can't create a style named \"%1\", this is reseved")
                                     .arg(name));
        return;
    }

    QString file(KFileDialog::getSaveFileName(0==version->selectedItem() ||
                                              currentStyle==item || item==defaultStyle
                                                  ? QString::null
                                                  : version->selectedItem()->text(0),
                                              "*"QTC_EXTENSION, this));

    if(!file.isEmpty())
    {
        KConfig cfg(file, false, false);

        if(cfg.isReadOnly())
            KMessageBox::error(this, i18n("Could not write to file:\n%1").arg(file));
        else
        {
            Options opts;

            setOptions(opts);
            opts.name=name;
            saveStyle(cfg, opts);
        }
    }
}

void QtCurveConfig::setVersion()
{
    setVersion(version->selectedItem());
}

void QtCurveConfig::setVersion(QListViewItem *item)
{
    ListViewItem *i=(ListViewItem *)item;

    if(!i)
        i=currentStyle;

    setWidgetOptions(i->opts);
    lvDark->setEnabled(LV_LINES_NONE!=lvLines->currentItem());
    delBtn->setEnabled(item && !i->opts.readOnly);

    emboldenToggled();
    dbiChanged();
    lvChanged();
    shadeSlidersChanged();
    shadeSbSlidersChanged();
    shadeMenubarsChanged();
    shadeCheckRadioChanged();
    lvHeaderColorChanged();
    stripedProgressChanged();
    customMenuTextColorChanged();
    borderMenuItemsChanged();
    borderMenubarItemsChanged();
    roundMenubarItemsChanged();
}

void QtCurveConfig::saveStyle(const QString &title, Options &opts)
{
    if(!title.isEmpty())
    {
        if(currentStyle->opts.name==title || defaultStyle->opts.name==title)
        {
            KMessageBox::error(this,
                               i18n("Sorry, you can't create a style named \"%1\", this is reseved")
                               .arg(title));
            return;
        }

        QListViewItem *item=version->firstChild();

        while(item)
        {
            if (item->text(0)==title)
                if(((ListViewItem *)item)->opts.readOnly)
                {
                    KMessageBox::error(this,
                                       i18n("Sorry, a style named \"%1\" already exists, "
                                            "and is read-only")
                                       .arg(title));
                    return;
                }
                else
                    if(KMessageBox::No==KMessageBox::warningYesNo(this, i18n("Overwrite the "
                                                                             "existing style?")))
                        return;
                    else
                        break;
            item=item->nextSibling();
        }

        QString resource("qtcurve/"+title+QTC_EXTENSION);
        KConfig cfg(resource, false, false, "data");

        opts.name=title;
        if(saveStyle(cfg, opts))
        {
            QString file(::locate("data", resource));

            if(!file.isEmpty())
            {
                Options opts;

                if(loadOptions(file, opts) && !opts.name.isEmpty())
                    if(item)
                        ((ListViewItem *)item)->opts=opts;
                    else
                        new ListViewItem(opts, version);
            }
        }
    }
}

bool QtCurveConfig::saveStyle(KConfig &cfg, const Options &opts)
{
    if(cfg.isReadOnly())
        KMessageBox::error(this, i18n("Could not save style"));
    else
        return writeConfig(&cfg, opts);

    return false;
}

void QtCurveConfig::loadStandardStyles()
{
    QStringList           files=KGlobal::dirs()->findAllResources("data",
                                                                  "qtcurve/*"QTC_EXTENSION, false, true);
    QStringList::Iterator it(files.begin()),
                          end(files.end());
    Options               opts;

    defaultSettings(&opts);
    opts.name=i18n("Default");
    opts.readOnly=true;
    defaultStyle=new ListViewItem(opts, version, true);

    readConfig(QString::null, &opts, &defaultStyle->opts);
    setWidgetOptions(opts);
    opts.readOnly=true;
    opts.name=i18n("Current Scheme");
    currentStyle=new ListViewItem(opts, version, true);

    for(; it!=end; ++it)
    {
        Options stdOpts;

        if(loadOptions(*it, stdOpts) && !stdOpts.name.isEmpty())
        {
            QListViewItem *i=version->firstChild();

            while(i)
            {
                if (i->text(0)==stdOpts.name)
                    break;
                else
                    i=i->nextSibling();
            }

            if(!i)
                new ListViewItem(stdOpts, version);
        }
    }

    if(!defaultStyle)
        defaultStyle=currentStyle;
}

void QtCurveConfig::setOptions(Options &opts)
{
    opts.round=(ERound)round->currentItem();
    opts.toolbarBorders=(ETBarBorder)toolbarBorders->currentItem();
    opts.appearance=(EAppearance)appearance->currentItem();
    opts.vArrow=vArrow->isChecked();
    opts.fillSlider=fillSlider->isChecked();
    opts.drawDockWindowTitles=drawDockWindowTitles->isChecked();
    opts.coloredSidebarButtons=coloredSidebarButtons->isChecked();
    opts.coloredMouseOver=coloredMouseOver->isChecked();
    opts.menubarMouseOver=menubarMouseOver->isChecked();
    opts.shadeMenubarOnlyWhenActive=shadeMenubarOnlyWhenActive->isChecked();
    opts.inactiveHighlight=inactiveHighlight->isChecked();
    opts.thinnerMenuItems=thinnerMenuItems->isChecked();
    opts.roundMenuItems=roundMenuItems->isChecked();
    opts.borderMenuItems=borderMenuItems->isChecked();
    opts.roundMenubarItems=roundMenubarItems->isChecked();
    opts.borderMenubarItems=borderMenubarItems->isChecked();
    opts.fixParentlessDialogs=fixParentlessDialogs->isChecked();
    opts.lightSliderBorder=lightSliderBorder->isChecked();
    opts.lightSbSliderBorder=lightSbSliderBorder->isChecked();
    opts.stripedProgress=stripedProgress->isChecked();
    opts.animatedProgress=animatedProgress->isChecked() &&
                         stripedProgress->isChecked();
    opts.lighterPopupMenuBgnd=lighterPopupMenuBgnd->isChecked();
    opts.embolden=embolden->isChecked();
    opts.focus=(EFocus)focus->currentItem();
    opts.scrollbarType=(EScrollbar)scrollbarType->currentItem();
    opts.crLabelHighlight=crLabelHighlight->isChecked();
    opts.defBtnIndicator=(EDefBtnIndicator)defaultIndicator->currentItem();
    opts.sliderThumbs=(ELine)sliderThumbs->currentItem();
    opts.sbSliderThumbs=(ELine)sbSliderThumbs->currentItem();
    opts.handles=(ELine)handles->currentItem();
    opts.lvExpander=(ELvExpander)lvExpander->currentItem();
    opts.lvLines=(ELvLines)lvLines->currentItem();
    opts.lvDark=lvDark->isChecked();
    opts.drawStatusBarFrames=drawStatusBarFrames->isChecked();
    opts.lvHeaderAppearance=(EAppearance)lvHeaderAppearance->currentItem();
    opts.lvHeaderColor=(EHeaderColor)lvHeaderColor->currentItem();
    opts.highlightTab=highlightTab->isChecked();
    opts.selectedTabAppearance=(EAppearance)selectedTabAppearance->currentItem();
    opts.normalTabAppearance=(EAppearance)normalTabAppearance->currentItem();
    opts.shadeSliders=(EShade)shadeSliders->currentItem();
    opts.shadeSbSliders=(EShade)shadeSbSliders->currentItem();
    opts.shadeMenubars=(EShade)shadeMenubars->currentItem();
    opts.shadeCheckRadio=(EShade)shadeCheckRadio->currentItem();
    opts.menubarAppearance=(EAppearance)menubarAppearance->currentItem();
    opts.menubarRoundTopOnly=menubarRoundTopOnly->isChecked();
    opts.toolbarAppearance=(EAppearance)toolbarAppearance->currentItem();
    opts.toolbarSeparators=(ELine)toolbarSeparators->currentItem();
    opts.splitters=(ELine)splitters->currentItem();
    opts.highlightEdits=highlightEdits->isChecked();
    opts.customSlidersColor=customSlidersColor->color();
    opts.customSbSlidersColor=customSbSlidersColor->color();
    opts.customMenubarsColor=customMenubarsColor->color();
    opts.customCheckRadioColor=customCheckRadioColor->color();
    opts.customLvHeaderColor=customLvHeaderColor->color();
    opts.highlightFactor=((double)(highlightFactor->value()+100))/100.0;
    opts.customMenuNormTextColor=customMenuNormTextColor->color();
    opts.customMenuSelTextColor=customMenuSelTextColor->color();
    opts.customMenuTextColor=customMenuTextColor->isChecked();
}

void QtCurveConfig::setWidgetOptions(const Options &opts)
{
    focus->setCurrentItem(opts.focus);
    scrollbarType->setCurrentItem(opts.scrollbarType);
    lighterPopupMenuBgnd->setChecked(opts.lighterPopupMenuBgnd);
    round->setCurrentItem(opts.round);
    toolbarBorders->setCurrentItem(opts.toolbarBorders);
    sliderThumbs->setCurrentItem(opts.sliderThumbs);
    sbSliderThumbs->setCurrentItem(opts.sbSliderThumbs);
    lvExpander->setCurrentItem(opts.lvExpander);
    lvLines->setCurrentItem(opts.lvLines);
    lvDark->setChecked(opts.lvDark);
    drawStatusBarFrames->setChecked(opts.drawStatusBarFrames);
    lvHeaderAppearance->setCurrentItem(opts.lvHeaderAppearance);
    lvHeaderColor->setCurrentItem(opts.lvHeaderColor);
    handles->setCurrentItem(opts.handles);
    appearance->setCurrentItem(opts.appearance);
    vArrow->setChecked(opts.vArrow);
    fillSlider->setChecked(opts.fillSlider);
    drawDockWindowTitles->setChecked(opts.drawDockWindowTitles);
    coloredSidebarButtons->setChecked(opts.coloredSidebarButtons);
    coloredMouseOver->setChecked(opts.coloredMouseOver);
    menubarMouseOver->setChecked(opts.menubarMouseOver);
    shadeMenubarOnlyWhenActive->setChecked(opts.shadeMenubarOnlyWhenActive);
    inactiveHighlight->setChecked(opts.inactiveHighlight);
    thinnerMenuItems->setChecked(opts.thinnerMenuItems);
    roundMenuItems->setChecked(opts.roundMenuItems);
    borderMenuItems->setChecked(opts.borderMenuItems);
    roundMenubarItems->setChecked(opts.roundMenubarItems);
    borderMenubarItems->setChecked(opts.borderMenubarItems);
    fixParentlessDialogs->setChecked(opts.fixParentlessDialogs);
    lightSliderBorder->setChecked(opts.lightSliderBorder);
    lightSbSliderBorder->setChecked(opts.lightSbSliderBorder);
    stripedProgress->setChecked(opts.stripedProgress);
    animatedProgress->setChecked(opts.stripedProgress && opts.animatedProgress);
    embolden->setChecked(opts.embolden);
    crLabelHighlight->setChecked(opts.crLabelHighlight);
    defaultIndicator->setCurrentItem(opts.defBtnIndicator);
    highlightTab->setChecked(opts.highlightTab);
    selectedTabAppearance->setCurrentItem(opts.selectedTabAppearance);
    normalTabAppearance->setCurrentItem(opts.normalTabAppearance);
    menubarRoundTopOnly->setChecked(opts.menubarRoundTopOnly);
    menubarAppearance->setCurrentItem(opts.menubarAppearance);
    toolbarAppearance->setCurrentItem(opts.toolbarAppearance);
    toolbarSeparators->setCurrentItem(opts.toolbarSeparators);
    splitters->setCurrentItem(opts.splitters);
    highlightEdits->setChecked(opts.highlightEdits);
    shadeSliders->setCurrentItem(opts.shadeSliders);
    shadeSbSliders->setCurrentItem(opts.shadeSbSliders);
    shadeMenubars->setCurrentItem(opts.shadeMenubars);
    shadeCheckRadio->setCurrentItem(opts.shadeCheckRadio);
    highlightFactor->setValue((int)(opts.highlightFactor*100)-100);
    customSlidersColor->setColor(opts.customSlidersColor);
    customSbSlidersColor->setColor(opts.customSbSlidersColor);
    customMenubarsColor->setColor(opts.customMenubarsColor);
    customCheckRadioColor->setColor(opts.customCheckRadioColor);
    customLvHeaderColor->setColor(opts.customLvHeaderColor);
    customMenuNormTextColor->setColor(opts.customMenuNormTextColor);
    customMenuSelTextColor->setColor(opts.customMenuSelTextColor);
    customMenuTextColor->setChecked(opts.customMenuTextColor);

    customSlidersColor->setEnabled(SHADE_CUSTOM==opts.shadeSliders);
    customSbSlidersColor->setEnabled(SHADE_CUSTOM==opts.shadeSbSliders);
    customMenubarsColor->setEnabled(SHADE_CUSTOM==opts.shadeMenubars);
    customCheckRadioColor->setEnabled(SHADE_CUSTOM==opts.shadeCheckRadio);
    customLvHeaderColor->setEnabled(HEADER_COLOR_CUSTOM==opts.lvHeaderColor);
    customMenuNormTextColor->setEnabled(customMenuTextColor->isChecked());
    customMenuSelTextColor->setEnabled(customMenuTextColor->isChecked());
}

bool QtCurveConfig::loadOptions(const QString &file, Options &opts)
{
    Options fileOpts;

    if(readConfig(file, &fileOpts, &defaultStyle->opts) && !fileOpts.name.isEmpty())
    {
        opts=fileOpts;
        opts.file=file;
        return true;
    }
    return false;
}

bool QtCurveConfig::settingsChanged()
{
    return round->currentItem()!=currentStyle->opts.round ||
         toolbarBorders->currentItem()!=currentStyle->opts.toolbarBorders ||
         appearance->currentItem()!=(int)currentStyle->opts.appearance ||
         vArrow->isChecked()!=currentStyle->opts.vArrow ||
         fillSlider->isChecked()!=currentStyle->opts.fillSlider ||
         drawDockWindowTitles->isChecked()!=currentStyle->opts.drawDockWindowTitles ||
         coloredSidebarButtons->isChecked()!=currentStyle->opts.coloredSidebarButtons ||
         coloredMouseOver->isChecked()!=currentStyle->opts.coloredMouseOver ||
         menubarMouseOver->isChecked()!=currentStyle->opts.menubarMouseOver ||
         shadeMenubarOnlyWhenActive->isChecked()!=currentStyle->opts.shadeMenubarOnlyWhenActive ||
         inactiveHighlight->isChecked()!=currentStyle->opts.inactiveHighlight ||
         thinnerMenuItems->isChecked()!=currentStyle->opts.thinnerMenuItems ||
         roundMenuItems->isChecked()!=currentStyle->opts.roundMenuItems ||
         borderMenuItems->isChecked()!=currentStyle->opts.borderMenuItems ||
         roundMenubarItems->isChecked()!=currentStyle->opts.roundMenubarItems ||
         borderMenubarItems->isChecked()!=currentStyle->opts.borderMenubarItems ||
         fixParentlessDialogs->isChecked()!=currentStyle->opts.fixParentlessDialogs ||
         lightSliderBorder->isChecked()!=currentStyle->opts.lightSliderBorder ||
         lightSbSliderBorder->isChecked()!=currentStyle->opts.lightSbSliderBorder ||
         stripedProgress->isChecked()!=currentStyle->opts.stripedProgress ||
         animatedProgress->isChecked()!=currentStyle->opts.animatedProgress ||
         lighterPopupMenuBgnd->isChecked()!=currentStyle->opts.lighterPopupMenuBgnd ||
         embolden->isChecked()!=currentStyle->opts.embolden ||
         crLabelHighlight->isChecked()!=currentStyle->opts.crLabelHighlight ||
         defaultIndicator->currentItem()!=(int)currentStyle->opts.defBtnIndicator ||
         sliderThumbs->currentItem()!=(int)currentStyle->opts.sliderThumbs ||
         sbSliderThumbs->currentItem()!=(int)currentStyle->opts.sbSliderThumbs ||
         handles->currentItem()!=(int)currentStyle->opts.handles ||
         lvExpander->currentItem()!=(int)currentStyle->opts.lvExpander ||
         lvLines->currentItem()!=(int)currentStyle->opts.lvLines ||
         focus->currentItem()!=(int)currentStyle->opts.focus ||
         scrollbarType->currentItem()!=(int)currentStyle->opts.scrollbarType ||
         lvDark->isChecked()!=currentStyle->opts.lvDark ||
         drawStatusBarFrames->isChecked()!=currentStyle->opts.drawStatusBarFrames ||
         lvHeaderAppearance->currentItem()!=(int)currentStyle->opts.lvHeaderAppearance ||
         highlightTab->isChecked()!=currentStyle->opts.highlightTab ||
         shadeSliders->currentItem()!=(int)currentStyle->opts.shadeSliders ||
         shadeSbSliders->currentItem()!=(int)currentStyle->opts.shadeSbSliders ||
         shadeMenubars->currentItem()!=(int)currentStyle->opts.shadeMenubars ||
         shadeCheckRadio->currentItem()!=(int)currentStyle->opts.shadeCheckRadio ||
         menubarRoundTopOnly->isChecked()!=currentStyle->opts.menubarRoundTopOnly ||
         menubarAppearance->currentItem()!=currentStyle->opts.menubarAppearance ||
         toolbarAppearance->currentItem()!=currentStyle->opts.toolbarAppearance ||
         toolbarSeparators->currentItem()!=currentStyle->opts.toolbarSeparators ||
         splitters->currentItem()!=currentStyle->opts.splitters ||
         highlightEdits->isChecked()!=currentStyle->opts.highlightEdits ||
         lvHeaderColor->currentItem()!=(int)currentStyle->opts.lvHeaderColor ||
         (highlightFactor->value()+100)!=(int)(currentStyle->opts.highlightFactor*100) ||
         selectedTabAppearance->currentItem()!=currentStyle->opts.selectedTabAppearance ||
         normalTabAppearance->currentItem()!=currentStyle->opts.normalTabAppearance ||
         customMenuTextColor->isChecked()!=currentStyle->opts.customMenuTextColor ||
         (HEADER_COLOR_CUSTOM==currentStyle->opts.lvHeaderColor &&
               customLvHeaderColor->color()!=currentStyle->opts.customLvHeaderColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeSliders &&
               customSlidersColor->color()!=currentStyle->opts.customSlidersColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeSbSliders &&
               customSbSlidersColor->color()!=currentStyle->opts.customSbSlidersColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeMenubars &&
               customMenubarsColor->color()!=currentStyle->opts.customMenubarsColor) ||
         (SHADE_CUSTOM==currentStyle->opts.shadeCheckRadio &&
               customCheckRadioColor->color()!=currentStyle->opts.customCheckRadioColor) ||
         (customMenuTextColor->isChecked() &&
               customMenuNormTextColor->color()!=currentStyle->opts.customMenuNormTextColor) ||
         (customMenuTextColor->isChecked() &&
               customMenuSelTextColor->color()!=currentStyle->opts.customMenuSelTextColor);
}

#include "qtcurveconfig.moc"
