/*
 * QtCurve GTK engine
 *
 * This is a hacked/modified version of the B***Curve GTK engine. Modified to use
 * ~/.qt/qtrc to get color information to recolor widgets.
 *
 * Changes are (C) Craig Drummond 2003 - Craig.Drummond@lycos.co.uk
 *
 */

#include "common.h"
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <sys/types.h>

#define QTC_MAX_FILENAME_LEN   1024
#define QTC_MAX_INPUT_LINE_LEN 256
#define QTC_FILE               "qtcurvestylerc"

#ifdef CONFIG_READ
static int c2h(char ch)
{
    return (ch>='0' && ch<='9') ? ch-'0' :
           (ch>='a' && ch<='f') ? 10+(ch-'a') :
           (ch>='A' && ch<='F') ? 10+(ch-'A') :
           0;
}

#define ATOH(str) ((c2h(*str)<<4)+c2h(*(str+1)))

static void setRgb(color *col, const char *str)
{
    if(str && strlen(str)>6)
    {
        int offset='#'==str[0] ? 1 : 0;
#ifdef __cplusplus
        col->setRgb(ATOH(&str[offset]), ATOH(&str[offset+2]), ATOH(&str[offset+4]));
#else
        col->red=ATOH(&str[offset])<<8;
        col->green=ATOH(&str[offset+2])<<8;
        col->blue=ATOH(&str[offset+4])<<8;
        col->pixel=0;
#endif
    }
    else
#ifdef __cplusplus
        col->setRgb(0, 0, 0);
#else
        col->red=col->green=col->blue=col->pixel=0;
#endif
}

static EDefBtnIndicator toInd(const char *str, EDefBtnIndicator def)
{
    if(str)
    {
        if(0==memcmp(str, "fontcolor", 9) || 0==memcmp(str, "border", 6))
            return IND_FONT_COLOR;
        if(0==memcmp(str, "none", 4))
            return IND_NONE;
        if(0==memcmp(str, "corner", 6))
            return IND_CORNER;
        if(0==memcmp(str, "colored", 7))
            return IND_COLORED;
    }

    return def;
}

static ELine toLine(const char *str, ELine def)
{
    if(str)
    {
        if(0==memcmp(str, "raised", 6))
            return LINE_RAISED;
        if(0==memcmp(str, "dashes", 6))
            return LINE_DASHES;
        if(0==memcmp(str, "none", 4))
            return LINE_NONE;
        if(0==memcmp(str, "sunken", 6))
            return LINE_SUNKEN;
        if(0==memcmp(str, "largedots", 9))
            return LINE_LARGE_DOTS;
        if(0==memcmp(str, "dots", 4))
            return LINE_DOTS;
    }
    return def;
}

static ETBarBorder toTBarBorder(const char *str, ETBarBorder def)
{
    if(str)
    {
        if(0==memcmp(str, "dark", 4))
            return TB_DARK;
        if(0==memcmp(str, "none", 4))
            return TB_NONE;
        if(0==memcmp(str, "light", 5))
            return TB_LIGHT;
    }
    return def;
}

static ELvExpander toLvExpander(const char *str, ELvExpander def)
{
    return 0==memcmp(str, "arrow", 5) ? LV_EXP_ARR : LV_EXP_PM;

    if(str)
    {
        if(0==memcmp(str, "arrow", 5))
            return LV_EXP_ARR;
        if(0==memcmp(str, "plusminus", 9))
            return LV_EXP_PM;
    }
    return def;
}

static ELvLines toLvLines(const char *str, ELvLines def)
{
    if(str)
    {
        if(0==memcmp(str, "solid", 5))
            return LV_LINES_SOLID;
        if(0==memcmp(str, "dotted", 6))
            return LV_LINES_DOTTED;
        if(0==memcmp(str, "none", 4))
            return LV_LINES_NONE;
    }
    return def;
}

static EAppearance toAppearance(const char *str, EAppearance def)
{
    if(str)
    {
        if(0==memcmp(str, "flat", 4))
            return APPEARANCE_FLAT;
        if(0==memcmp(str, "raised", 6))
            return APPEARANCE_RAISED;
        if(0==memcmp(str, "gradient", 8) || 0==memcmp(str, "lightgradient", 13))
            return APPEARANCE_GRADIENT;
        if(0==memcmp(str, "glass", 5) || 0==memcmp(str, "shinyglass", 10))
            return APPEARANCE_SHINY_GLASS;
        if(0==memcmp(str, "dullglass", 9))
            return APPEARANCE_DULL_GLASS;
        if(0==memcmp(str, "inverted", 8))
            return APPEARANCE_INVERTED;
        if(0==memcmp(str, "bevelled", 8))
            return APPEARANCE_BEVELLED;
    }
    return def;
}

static EShade toShade(const char *str, bool allowDarken, EShade def)
{
    if(str)
    {
        /* true/false is from 0.25... */
        if(0==memcmp(str, "true", 4) || 0==memcmp(str, "selected", 8))
            return SHADE_SELECTED;
        if(allowDarken && 0==memcmp(str, "darken", 6))
            return SHADE_DARKEN;
        if(0==memcmp(str, "custom", 6))
            return SHADE_CUSTOM;
        if(0==memcmp(str, "none", 4))
            return SHADE_NONE;
    }

    return def;
}

static EHeaderColor toHeaderColor(const char *str, EHeaderColor def)
{
    if(str)
    {
        if(0==memcmp(str, "background", 10))
            return HEADER_COLOR_BGND;
        if(0==memcmp(str, "custom", 6))
            return HEADER_COLOR_CUSTOM;
        if(0==memcmp(str, "button", 6))
            return HEADER_COLOR_BTN;
    }

    return def;
}

static EFocus toFocus(const char *str, EFocus def)
{
    if(str)
    {
        if(0==memcmp(str, "dots", 4))
            return FOCUS_DOTS;
        if(0==memcmp(str, "colored", 7))
            return FOCUS_COLORED;
    }

    return def;
}

/* Prior to 0.42 round was a bool - so need to read 'false' as 'none' */
static ERound toRound(const char *str, ERound def)
{
    if(str)
    {
        if(0==memcmp(str, "none", 4) || 0==memcmp(str, "false", 5))
            return ROUND_NONE;
        if(0==memcmp(str, "slight", 6))
            return ROUND_SLIGHT;
        if(0==memcmp(str, "full", 4))
            return ROUND_FULL;
    }

    return def;
}

static EScrollbar toScrollbar(const char *str, EScrollbar def)
{
    if(str)
    {
        if(0==memcmp(str, "kde", 3))
            return SCROLLBAR_KDE;
        if(0==memcmp(str, "windows", 7))
            return SCROLLBAR_WINDOWS;
        if(0==memcmp(str, "platinum", 8))
            return SCROLLBAR_PLATINUM;
        if(0==memcmp(str, "next", 4))
            return SCROLLBAR_NEXT;
        if(0==memcmp(str, "none", 4))
            return SCROLLBAR_NONE;
    }

    return def;
}

#endif

#ifdef CONFIG_WRITE
#include <kstandarddirs.h>
#endif

static const char * getHome()
{
    static const char *home=NULL;

    if(!home)
    {
        struct passwd *p=getpwuid(geteuid());

        if(p)
            home=p->pw_dir;
        else
        {
            char *env=getenv("HOME");

            if(env)
                home=env;
        }

        if(!home)
            home="/tmp";
    }

    return home;
}

static const char *xdgConfigFolder()
{
    static char xdgDir[QTC_MAX_FILENAME_LEN]={'\0'};

    if(!xdgDir[0])
    {
        static const char *home=NULL;

#if 0
        char *env=getenv("XDG_CONFIG_HOME");

        /*
            Check the setting of XDG_CONFIG_HOME
            For some reason, sudo leaves the env vars set to those of the
            caller - so XDG_CONFIG_HOME would point to the users setting, and
            not roots.

            Therefore, check that home is first part of XDG_CONFIG_HOME
        */

        if(env && 0==getuid())
        {
            if(!home)
                home=getHome();
            if(home && home!=strstr(env, home))
                env=NULL;
        }
#else
        /*
           Hmm... for 'root' dont bother to check env var, just set to ~/.config
           - as problems would arise if "sudo kcmshell style", and then
           "sudo su" / "kcmshell style". The 1st would write to ~/.config, but
           if root has a XDG_ set then that would be used on the second :-(
        */
        char *env=0==getuid() ? NULL : getenv("XDG_CONFIG_HOME");
#endif

        if(!env)
        {
            if(!home)
                home=getHome();

            sprintf(xdgDir, "%s/.config", home);
        }
        else
            strcpy(xdgDir, env);

#ifdef CONFIG_WRITE
#ifdef __cplusplus
        KStandardDirs::makeDir(xdgDir);
#endif
#endif
    }

    return xdgDir;
}

#ifdef CONFIG_READ

#ifdef __cplusplus
#define QTC_IS_BLACK(A) (0==(A).red() && 0==(A).green() && 0==(A).blue())
#else
#define QTC_IS_BLACK(A) (0==(A).red && 0==(A).green && 0==(A).blue)
#endif

static void checkColor(EShade *s, color *c)
{
    if(SHADE_CUSTOM==*s && QTC_IS_BLACK(*c))
        *s=SHADE_NONE;
}

#ifdef __cplusplus

#include <qmap.h>
#include <qfile.h>
#include <qtextstream.h>

class QtCConfig
{
    public:

    QtCConfig(const QString &filename);

    bool    ok() const { return values.count()>0; }
    QString readEntry(const char *key, const QString &def=QString::null);

    private:

    QMap<QString, QString> values;
};

QtCConfig::QtCConfig(const QString &filename)
{
    QFile f(filename);

    if(f.open(IO_ReadOnly))
    {
        QTextStream stream(&f);
        QString     line;

        while(!stream.atEnd())
        {
            line = stream.readLine();
            int pos=line.find('=');
            if(-1!=pos)
                values[line.left(pos)]=line.mid(pos+1);
        }
        f.close();
    }
}

inline QString QtCConfig::readEntry(const char *key, const QString &def)
{
    return values.contains(key) ? values[key] : def;
}

inline QString readStringEntry(QtCConfig &cfg, const char *key)
{
    return cfg.readEntry(key);
}

static int readNumEntry(QtCConfig &cfg, const char *key, int def)
{
    QString val(readStringEntry(cfg, key));

    return val.isEmpty() ? def : val.toInt();
}

static bool readBoolEntry(QtCConfig &cfg, const char *key, bool def)
{
    QString val(readStringEntry(cfg, key));

    return val.isEmpty() ? def : (val=="true" ? true : false);
}

#define QTC_LATIN1(A) A.latin1()

#define QTC_CFG_READ_COLOR(ENTRY) \
    { \
        QString sVal(cfg.readEntry(#ENTRY)); \
        if(sVal.isEmpty()) \
            opts->ENTRY=def->ENTRY; \
        else \
            setRgb(&(opts->ENTRY), sVal.latin1()); \
    }

#else

static char * lookupCfgHash(GHashTable **cfg, char *key, char *val)
{
    char *rv=NULL;

    if(!*cfg)
        *cfg=g_hash_table_new(g_str_hash, g_str_equal);
    else
        rv=(char *)g_hash_table_lookup(*cfg, key);

    if(!rv && val)
    {
        g_hash_table_insert(*cfg, g_strdup(key), g_strdup(val));
        rv=(char *)g_hash_table_lookup(*cfg, key);
    }

    return rv;
}

static GHashTable * loadConfig(const char *filename)
{
    FILE       *f=fopen(filename, "r");
    GHashTable *cfg=NULL;

    if(f)
    {
        char line[QTC_MAX_INPUT_LINE_LEN];

        while(NULL!=fgets(line, QTC_MAX_INPUT_LINE_LEN-1, f))
        {
            char *eq=strchr(line, '=');
            int  pos=eq ? eq-line : -1;

            if(pos>0)
            {
                char *endl=strchr(line, '\n');

                if(endl)
                    *endl='\0';

                line[pos]='\0';

                lookupCfgHash(&cfg, line, &line[pos+1]);
            }
        }

        fclose(f);
    }

    return cfg;
}

static void releaseConfig(GHashTable *cfg)
{
    g_hash_table_destroy(cfg);
}

static char * readStringEntry(GHashTable *cfg, char *key)
{
    return lookupCfgHash(&cfg, key, NULL);
}

static int readNumEntry(GHashTable *cfg, char *key, int def)
{
    char *str=readStringEntry(cfg, key);

    return str ? atoi(str) : def;
}

static gboolean readBoolEntry(GHashTable *cfg, char *key, gboolean def)
{
    char *str=readStringEntry(cfg, key);

    return str ? (0==memcmp(str, "true", 4) ? TRUE : FALSE) : def;
}

#define QTC_LATIN1(A) A

#define QTC_CFG_READ_COLOR(ENTRY) \
    { \
        const char *str=readStringEntry(cfg, #ENTRY); \
    \
        if(str) \
            setRgb(&(opts->ENTRY), str); \
        else \
            opts->ENTRY=def->ENTRY; \
    }
#endif

#define QTC_CFG_READ_BOOL(ENTRY) \
    opts->ENTRY=readBoolEntry(cfg, #ENTRY, def->ENTRY);

#define QTC_CFG_READ_ROUND(ENTRY) \
    opts->ENTRY=toRound(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_DI(ENTRY) \
    opts->ENTRY=((double)(readNumEntry(cfg, #ENTRY, ((int)(def->ENTRY*100))-100)+100))/100.0;

#define QTC_CFG_READ_TB_BORDER(ENTRY) \
    opts->ENTRY=toTBarBorder(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_APPEARANCE(ENTRY) \
    opts->ENTRY=toAppearance(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_FOCUS(ENTRY) \
    opts->ENTRY=toFocus(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_DEF_BTN(ENTRY) \
    opts->ENTRY=toInd(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_LINE(ENTRY) \
    opts->ENTRY=toLine(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_LV_LINES(ENTRY) \
    opts->ENTRY=toLvLines(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_LV_EXPANDER(ENTRY) \
    opts->ENTRY=toLvExpander(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_LV_HEADER_COL(ENTRY) \
    opts->ENTRY=toHeaderColor(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#define QTC_CFG_READ_SHADE(ENTRY, AD) \
    opts->ENTRY=toShade(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), AD, def->ENTRY);

#define QTC_CFG_READ_SCROLLBAR(ENTRY) \
    opts->ENTRY=toScrollbar(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);

#ifdef __cplusplus
static bool readConfig(const QString &file, Options *opts, Options *def)
#else
static bool readConfig(const char *file, Options *opts, Options *def)
#endif
{
#ifdef __cplusplus
    if(file.isEmpty())
    {
        const char *xdg=xdgConfigFolder();

        if(xdg)
        {
            QString filename(xdg);

            filename+="/"QTC_FILE;
            readConfig(filename, opts, def);
        }
    }
#else
    if(!file)
    {
        const char *xdg=xdgConfigFolder();

        if(xdg)
        {
            char filename[QTC_MAX_FILENAME_LEN];

            sprintf(filename, "%s/"QTC_FILE, xdg);
            readConfig(filename, opts, def);
        }
    }
#endif
    else
    {
#ifdef __cplusplus
        QtCConfig cfg(file);

        if(cfg.ok())
        {
#else
        GHashTable *cfg=loadConfig(file);

        if(cfg)
        {
#endif
            QTC_CFG_READ_ROUND(round)
            QTC_CFG_READ_DI(highlightFactor)
            QTC_CFG_READ_TB_BORDER(toolbarBorders)
            QTC_CFG_READ_APPEARANCE(appearance)
            QTC_CFG_READ_FOCUS(focus)
            QTC_CFG_READ_BOOL(vArrow)
            QTC_CFG_READ_BOOL(fillSlider)
            QTC_CFG_READ_BOOL(fixParentlessDialogs)
#ifdef __cplusplus
            QTC_CFG_READ_BOOL(drawDockWindowTitles)
            QTC_CFG_READ_BOOL(coloredSidebarButtons)
#endif
            QTC_CFG_READ_BOOL(lightSliderBorder)
            QTC_CFG_READ_BOOL(lightSbSliderBorder)
            QTC_CFG_READ_BOOL(stripedProgress)
            QTC_CFG_READ_BOOL(animatedProgress)
            QTC_CFG_READ_BOOL(lighterPopupMenuBgnd)
            QTC_CFG_READ_BOOL(embolden)
            QTC_CFG_READ_BOOL(crLabelHighlight)
            QTC_CFG_READ_DEF_BTN(defBtnIndicator)
            QTC_CFG_READ_LINE(sliderThumbs)
            QTC_CFG_READ_LINE(sbSliderThumbs)
            QTC_CFG_READ_LINE(handles)
            QTC_CFG_READ_LV_EXPANDER(lvExpander)
            QTC_CFG_READ_LV_LINES(lvLines)
            QTC_CFG_READ_BOOL(lvDark)
            QTC_CFG_READ_BOOL(drawStatusBarFrames)
            QTC_CFG_READ_APPEARANCE(lvHeaderAppearance)
            QTC_CFG_READ_LV_HEADER_COL(lvHeaderColor)
            QTC_CFG_READ_BOOL(highlightTab)
            QTC_CFG_READ_SHADE(shadeSliders, false)
            QTC_CFG_READ_SHADE(shadeSbSliders, false)
            QTC_CFG_READ_SHADE(shadeMenubars, true)
            QTC_CFG_READ_SHADE(shadeCheckRadio, false)
            QTC_CFG_READ_APPEARANCE(menubarAppearance)
            QTC_CFG_READ_BOOL(menubarRoundTopOnly)
            QTC_CFG_READ_APPEARANCE(toolbarAppearance)
            QTC_CFG_READ_LINE(toolbarSeparators)
            QTC_CFG_READ_LINE(splitters)
            QTC_CFG_READ_BOOL(highlightEdits)
            QTC_CFG_READ_APPEARANCE(normalTabAppearance)
            QTC_CFG_READ_APPEARANCE(selectedTabAppearance)
            QTC_CFG_READ_BOOL(customMenuTextColor)
            QTC_CFG_READ_BOOL(coloredMouseOver)
            QTC_CFG_READ_BOOL(roundMenuItems)
            QTC_CFG_READ_BOOL(borderMenuItems)
            QTC_CFG_READ_BOOL(roundMenubarItems)
            QTC_CFG_READ_BOOL(borderMenubarItems)
            QTC_CFG_READ_BOOL(menubarMouseOver)
            QTC_CFG_READ_BOOL(inactiveHighlight)
            QTC_CFG_READ_BOOL(shadeMenubarOnlyWhenActive)
            QTC_CFG_READ_BOOL(thinnerMenuItems)
            QTC_CFG_READ_COLOR(customSlidersColor)
            QTC_CFG_READ_COLOR(customSbSlidersColor)
            QTC_CFG_READ_COLOR(customMenubarsColor)
            QTC_CFG_READ_COLOR(customLvHeaderColor)
            QTC_CFG_READ_COLOR(customMenuSelTextColor)
            QTC_CFG_READ_COLOR(customMenuNormTextColor)
            QTC_CFG_READ_COLOR(customCheckRadioColor)
            QTC_CFG_READ_SCROLLBAR(scrollbarType)

#ifdef QTC_CONFIG_DIALOG
            opts->name=cfg.readEntry("name");
            opts->readOnly=(0!=access(QFile::encodeName(file), W_OK));
#endif

#ifndef __cplusplus
            releaseConfig(cfg);
#endif
            checkColor(&opts->shadeMenubars, &opts->customMenubarsColor);
            checkColor(&opts->shadeSliders, &opts->customSlidersColor);
            checkColor(&opts->shadeSbSliders, &opts->customSbSlidersColor);
            checkColor(&opts->shadeCheckRadio, &opts->customCheckRadioColor);

            if(HEADER_COLOR_CUSTOM==opts->lvHeaderColor &&
               QTC_IS_BLACK(opts->customLvHeaderColor))
                opts->lvHeaderColor=HEADER_COLOR_BGND;

            if(opts->animatedProgress && !opts->stripedProgress)
                opts->animatedProgress=false;

            if(APPEARANCE_RAISED==opts->toolbarAppearance)
                opts->toolbarAppearance=APPEARANCE_FLAT;
            if(APPEARANCE_RAISED==opts->menubarAppearance)
                opts->toolbarAppearance=APPEARANCE_FLAT;
            if(APPEARANCE_RAISED==opts->normalTabAppearance)
                opts->normalTabAppearance=APPEARANCE_FLAT;
            if(APPEARANCE_RAISED==opts->selectedTabAppearance)
                opts->selectedTabAppearance=APPEARANCE_FLAT;
            if(APPEARANCE_RAISED==opts->lvHeaderAppearance)
                opts->lvHeaderAppearance=APPEARANCE_FLAT;
            if(opts->roundMenuItems && !opts->borderMenuItems)
                opts->roundMenuItems=false;
            if(opts->roundMenubarItems && !opts->borderMenubarItems)
                opts->roundMenubarItems=false;
            if(opts->menubarRoundTopOnly && !opts->roundMenubarItems)
                opts->menubarRoundTopOnly=false;
            if(opts->highlightFactor<((100.0+MIN_HIGHLIGHT_FACTOR)/100.0) ||
               opts->highlightFactor>((100.0+MAX_HIGHLIGHT_FACTOR)/100.0))
                opts->highlightFactor=DEFAULT_HIGHLIGHT_FACTOR;

            return true;
        }
    }

    return false;
}

static bool fileExists(const char *path)
{
    struct stat info;

    return 0==lstat(path, &info) && (info.st_mode&S_IFMT)==S_IFREG;
}

static const char * getSystemConfigFile()
{
    static const char * constFiles[]={ "/etc/qt4/"QTC_FILE, "/etc/qt3/"QTC_FILE, "/etc/qt/"QTC_FILE, NULL };

    int i;

    for(i=0; constFiles[i]; ++i)
        if(fileExists(constFiles[i]))
            return constFiles[i];
    return NULL;
}

static void defaultSettings(Options *opts)
{
    /* Set hard-coded defaults... */
    opts->contrast=7;
    opts->highlightFactor=DEFAULT_HIGHLIGHT_FACTOR;
    opts->round=ROUND_FULL;
    opts->lighterPopupMenuBgnd=true;
    opts->vArrow=false;
    opts->stripedProgress=true;
    opts->animatedProgress=true;
    opts->highlightTab=true;
    opts->highlightEdits=true;
    opts->crLabelHighlight=false;
    opts->embolden=false;
    opts->menubarRoundTopOnly=true;
    opts->fillSlider=true;
    opts->lightSliderBorder=false;
    opts->lightSbSliderBorder=false;
    opts->focus=FOCUS_COLORED;
    opts->lvDark=false;
    opts->drawStatusBarFrames=false;
    opts->appearance=APPEARANCE_BEVELLED;
    opts->menubarAppearance=APPEARANCE_GRADIENT;
    opts->toolbarAppearance=APPEARANCE_GRADIENT;
    opts->selectedTabAppearance=APPEARANCE_GRADIENT;
    opts->normalTabAppearance=APPEARANCE_GRADIENT;
    opts->defBtnIndicator=IND_COLORED;
    opts->sliderThumbs=LINE_DOTS;
    opts->sbSliderThumbs=LINE_DOTS;
    opts->handles=LINE_DOTS;
    opts->shadeSliders=SHADE_SELECTED;
    opts->shadeSbSliders=SHADE_SELECTED;
    opts->shadeMenubars=SHADE_DARKEN;
    opts->shadeCheckRadio=SHADE_NONE;
    opts->toolbarBorders=TB_NONE;
    opts->lvExpander=LV_EXP_ARR;
    opts->lvLines=LV_LINES_NONE;
    opts->lvHeaderAppearance=APPEARANCE_BEVELLED;
    opts->lvHeaderColor=HEADER_COLOR_BGND;
    opts->toolbarSeparators=LINE_DOTS;
    opts->splitters=LINE_DOTS;
    opts->fixParentlessDialogs=false;
    opts->customMenuTextColor=false;
    opts->coloredMouseOver=true;
    opts->roundMenuItems=true;
    opts->borderMenuItems=true;
    opts->roundMenubarItems=true;
    opts->borderMenubarItems=true;
    opts->menubarMouseOver=true;
    opts->inactiveHighlight=true;
    opts->shadeMenubarOnlyWhenActive=false;
    opts->thinnerMenuItems=false;
    opts->scrollbarType=SCROLLBAR_WINDOWS;
#ifdef __cplusplus
    opts->customMenubarsColor.setRgb(0, 0, 0);
    opts->customSlidersColor.setRgb(0, 0, 0);
    opts->customSbSlidersColor.setRgb(0, 0, 0);
    opts->customLvHeaderColor.setRgb(0, 0, 0);
    opts->customMenuNormTextColor.setRgb(0, 0, 0);
    opts->customMenuSelTextColor.setRgb(0, 0, 0);
    opts->customCheckRadioColor.setRgb(0, 0, 0);
    opts->drawDockWindowTitles=true;
    opts->coloredSidebarButtons=true;
#else
    opts->customMenubarsColor.red=opts->customMenubarsColor.green=opts->customMenubarsColor.blue=0;
    opts->customSlidersColor.red=opts->customSlidersColor.green=opts->customSlidersColor.blue=0;
    opts->customSbSlidersColor.red=opts->customSlidersColor.green=opts->customSlidersColor.blue=0;
    opts->customLvHeaderColor.red=opts->customLvHeaderColor.green=opts->customLvHeaderColor.blue=0;
    opts->customMenuNormTextColor.red=opts->customMenuNormTextColor.green=opts->customMenuNormTextColor.blue=0;
    opts->customMenuSelTextColor.red=opts->customMenuSelTextColor.green=opts->customMenuSelTextColor.blue=0;
    opts->customCheckRadioColor.red=opts->customCheckRadioColor.green=opts->customCheckRadioColor.blue=0;
#endif
#ifdef QTC_CONFIG_DIALOG
    opts->readOnly=true;
#endif

    /* Read system config file... */
    {
    static const char * systemFilename=NULL;

    if(!systemFilename)
        systemFilename=getSystemConfigFile();

    if(systemFilename)
        readConfig(systemFilename, opts, opts);
    }
}

#endif

#ifdef CONFIG_WRITE
static const char *toStr(EDefBtnIndicator ind)
{
    switch(ind)
    {
        case IND_NONE:
            return "none";
        case IND_FONT_COLOR:
            return "fontcolor";
        case IND_CORNER:
            return "corner";
        default:
            return "colored";
    }
}

static const char *toStr(ELine ind, bool none)
{
    switch(ind)
    {
        case LINE_RAISED:
            return "raised";
        case LINE_DOTS:
            return "dots";
        case LINE_LARGE_DOTS:
            return "largedots";
        case LINE_DASHES:
            return none ? "none" : "dashes";
        default:
            return "sunken";
    }
}

static const char *toStr(ETBarBorder ind)
{
    switch(ind)
    {
        case TB_DARK:
            return "dark";
        case TB_NONE:
            return "none";
        default:
            return "light";
    }
}

static const char *toStr(ELvExpander exp)
{
    switch(exp)
    {
        case LV_EXP_PM:
            return "plusminus";
        default:
            return "arrow";
    }
}

static const char *toStr(ELvLines exp)
{
    switch(exp)
    {
        case LV_LINES_NONE:
            return "none";
        case LV_LINES_DOTTED:
            return "dotted";
        default:
            return "solid";
    }
}

static const char *toStr(EAppearance exp)
{
    switch(exp)
    {
        case APPEARANCE_FLAT:
            return "flat";
        case APPEARANCE_RAISED:
            return "raised";
        case APPEARANCE_GRADIENT:
            return "gradient";
        case APPEARANCE_DULL_GLASS:
            return "dullglass";
        case APPEARANCE_BEVELLED:
            return "bevelled";
        case APPEARANCE_INVERTED:
            return "inverted";
        default:
            return "shinyglass";
    }
}

static const char *toStr(EShade exp)
{
    switch(exp)
    {
        default:
        case SHADE_NONE:
            return "none";
        case SHADE_SELECTED:
            return "selected";
        case SHADE_CUSTOM:
            return "custom";
        case SHADE_DARKEN:
            return "darken";
    }
}

static const char *toStr(EHeaderColor exp)
{
    switch(exp)
    {
        default:
        case HEADER_COLOR_BGND:
            return "background";
        case HEADER_COLOR_BTN:
            return "button";
        case HEADER_COLOR_CUSTOM:
            return "custom";
    }
}

static const char *toStr(EFocus exp)
{
    switch(exp)
    {
        case FOCUS_DOTS:
            return "dots";
        default:
        case FOCUS_COLORED:
            return "colored";
    }
}

static const char *toStr(ERound exp)
{
    switch(exp)
    {
        case ROUND_NONE:
            return "none";
        case ROUND_SLIGHT:
            return "slight";
        default:
        case ROUND_FULL:
            return "full";
    }
}

static const char *toStr(EScrollbar sb)
{
    switch(sb)
    {
        case SCROLLBAR_KDE:
            return "kde";
        default:
        case SCROLLBAR_WINDOWS:
            return "windows";
        case SCROLLBAR_PLATINUM:
            return "platinum";
        case SCROLLBAR_NEXT:
            return "next";
        case SCROLLBAR_NONE:
            return "none";
    }
}

inline const char * toStr(bool b) { return b ? "true" : "false"; }

static QString toStr(const QColor &col)
{
    QString colorStr;

    colorStr.sprintf("#%02X%02X%02X", col.red(), col.green(), col.blue());
    return colorStr;
}

#define CFG_WRITE_ENTRY(ENTRY) \
    cfg->writeEntry(#ENTRY, toStr(opts.ENTRY))

#define CFG_WRITE_ENTRY_B(ENTRY, B) \
    cfg->writeEntry(#ENTRY, toStr(opts.ENTRY, B))

#define CFG_WRITE_ENTRY_D(ENTRY) \
    cfg->writeEntry(#ENTRY, ((int)(opts.ENTRY*100))-100)

bool static writeConfig(KConfig *cfg, const Options &opts)
{
    if(!cfg)
    {
        const char *xdg=xdgConfigFolder();

        if(xdg)
        {
            char filename[QTC_MAX_FILENAME_LEN];

            sprintf(filename, "%s/"QTC_FILE, xdg);

            KConfig defCfg(filename, false, false);

            return writeConfig(&defCfg, opts);
        }
    }
    else
    {
        cfg->setGroup(QTC_GROUP);
        CFG_WRITE_ENTRY(round);
        CFG_WRITE_ENTRY_D(highlightFactor);
        CFG_WRITE_ENTRY(toolbarBorders);
        CFG_WRITE_ENTRY(appearance);
        CFG_WRITE_ENTRY(focus);
        CFG_WRITE_ENTRY(vArrow);
        CFG_WRITE_ENTRY(fillSlider);
        CFG_WRITE_ENTRY(fixParentlessDialogs);
        CFG_WRITE_ENTRY(drawDockWindowTitles);
        CFG_WRITE_ENTRY(coloredSidebarButtons);
        CFG_WRITE_ENTRY(lightSliderBorder);
        CFG_WRITE_ENTRY(lightSbSliderBorder);
        CFG_WRITE_ENTRY(stripedProgress);
        CFG_WRITE_ENTRY(animatedProgress);
        CFG_WRITE_ENTRY(lighterPopupMenuBgnd);
        CFG_WRITE_ENTRY(embolden);
        CFG_WRITE_ENTRY(crLabelHighlight);
        CFG_WRITE_ENTRY(defBtnIndicator);
        CFG_WRITE_ENTRY_B(sliderThumbs, true);
        CFG_WRITE_ENTRY_B(sbSliderThumbs, true);
        CFG_WRITE_ENTRY_B(handles, false);
        CFG_WRITE_ENTRY(lvExpander);
        CFG_WRITE_ENTRY(lvLines);
        CFG_WRITE_ENTRY(lvDark);
        CFG_WRITE_ENTRY(drawStatusBarFrames);
        CFG_WRITE_ENTRY(lvHeaderAppearance);
        CFG_WRITE_ENTRY(lvHeaderColor);
        CFG_WRITE_ENTRY(highlightTab);
        CFG_WRITE_ENTRY(shadeSliders);
        CFG_WRITE_ENTRY(shadeSbSliders);
        CFG_WRITE_ENTRY(shadeMenubars);
        CFG_WRITE_ENTRY(shadeCheckRadio);
        CFG_WRITE_ENTRY(menubarAppearance);
        CFG_WRITE_ENTRY(menubarRoundTopOnly);
        CFG_WRITE_ENTRY(toolbarAppearance);
        CFG_WRITE_ENTRY_B(toolbarSeparators, true);
        CFG_WRITE_ENTRY_B(splitters, false);
        CFG_WRITE_ENTRY(highlightEdits);
        CFG_WRITE_ENTRY(normalTabAppearance);
        CFG_WRITE_ENTRY(selectedTabAppearance);
        CFG_WRITE_ENTRY(customMenuTextColor);
        CFG_WRITE_ENTRY(coloredMouseOver);
        CFG_WRITE_ENTRY(roundMenuItems);
        CFG_WRITE_ENTRY(borderMenuItems);
        CFG_WRITE_ENTRY(roundMenubarItems);
        CFG_WRITE_ENTRY(borderMenubarItems);
        CFG_WRITE_ENTRY(menubarMouseOver);
        CFG_WRITE_ENTRY(inactiveHighlight);
        CFG_WRITE_ENTRY(shadeMenubarOnlyWhenActive);
        CFG_WRITE_ENTRY(thinnerMenuItems);
        CFG_WRITE_ENTRY(customSlidersColor);
        CFG_WRITE_ENTRY(customSbSlidersColor);
        CFG_WRITE_ENTRY(customMenubarsColor);
        CFG_WRITE_ENTRY(customLvHeaderColor);
        CFG_WRITE_ENTRY(customMenuSelTextColor);
        CFG_WRITE_ENTRY(customMenuNormTextColor);
        CFG_WRITE_ENTRY(customCheckRadioColor);
        CFG_WRITE_ENTRY(scrollbarType);
        if(!opts.name.isEmpty())
        {
            cfg->setGroup(QTC_DESCR_GROUP);
            cfg->writeEntry("name", opts.name);
        }

        cfg->sync();
        return true;
    }
    return false;
}
#endif

