/*
  QtCurve (C) Craig Drummond, 2003 Craig.Drummond@lycos.co.uk

----

  Based upon B???Curve style (Copyright (c) 2002 R?d H?t, Inc)
      Bernhard Rosenkrazer <bero@r?dh?t.com>
      Preston Brown <pbrown@r?dh?t.com>
      Than Ngo <than@r?dh?t.com>

  Released under the GNU General Public License (GPL) v2.

----

  B???Curve is based on the KDE Light style, 2nd revision:
  Copyright(c)2000-2001 Trolltech AS (info@trolltech.com)

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <kdeversion.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qdrawutil.h>
#include <qscrollbar.h>
#include <qtabbar.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qbitmap.h>
#include <qtable.h>
#include <qimage.h>
#include <qcombobox.h>
#include <qslider.h>
#include <qcleanuphandler.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qprogressbar.h>
#include <qheader.h>
#include <qwidgetstack.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qtimer.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include "qtcurve.h"
#define CONFIG_READ
#include "config_file.c"
#include "pixmaps.h"
#include <qdialog.h>
#include <qprogressdialog.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <fixx11h.h>
#define QTC_SKIP_TASKBAR (APP_SKIP_TASKBAR==itsThemedApp || APP_KPRINTER==itsThemedApp || APP_KDIALOG==itsThemedApp)

#if KDE_VERSION >= 0x30200
#include <qfile.h>
#include <qdir.h>
#endif

#define QTC_BORDERED_FRAME_WIDTH 2
#define QTC_DEF_FRAME_WIDTH (QTC_ROUNDED ? 2 : 1)
#define QTC_NO_SECT -1

#define QTC_VERTICAL_TB_BUTTON 0x10000000
#define QTC_CHECK_BUTTON       0x20000000
#define QTC_STD_TOOLBUTTON     0x40000000
#define QTC_TOGGLE_BUTTON      0x80000000

#if KDE_VERSION >= 0x30200
// Try to read $KDEHOME/share/config/kickerrc to find out if kicker is transparent...

static QString readEnvPath(const char *env)
{
   QCString path=getenv(env);

   return path.isEmpty() ? QString::null : QFile::decodeName(path);
}

static bool kickerIsTrans()
{
    QString kdeHome(readEnvPath(getuid() ? "KDEHOME" : "KDEROOTHOME")),
            cfgFileName(kdeHome.isEmpty() ? QDir::homeDirPath()+"/.kde/share/config/kickerrc"
                                          : kdeHome+"/share/config/kickerrc");
    bool    trans(false);
    QFile   cfgFile(cfgFileName);

    if (cfgFile.open(IO_ReadOnly))
    {
        QTextStream stream(&cfgFile);
        QString     line;
        bool        stop(false),
                    inGen(false);

        while (!stream.atEnd() && !stop)
        {
            line=stream.readLine();

            if(inGen)
            {
                if(0==line.find("Transparent="))  // Found it!
                {
                    if(-1!=line.find("true"))
                        trans=true;
                    stop=true;
                }
                else if(line[0]==QChar('['))   // Then wasn't in General section...
                    stop=true;
            }
            else if(0==line.find("[General]"))
                inGen=true;
        }
        cfgFile.close();
    }

    return trans;
}
#endif

inline int limit(double c)
{
    return c < 0.0 ? 0 : (c > 255.0  ? 255 : (int)c);
}

inline QColor midColor(const QColor &a, const QColor &b, double factor=1.0)
{
    return QColor((a.red()+limit(b.red()*factor))>>1, 
                  (a.green()+limit(b.green()*factor))>>1, 
                  (a.blue()+limit(b.blue()*factor))>>1);
}

// Copied from Keramik...
static bool isFormWidget(const QWidget *widget)
{
    //Form widgets are in the KHTMLView, but that has 2 further inner levels
    //of widgets - QClipperWidget, and outside of that, QViewportWidget
    QWidget *potentialClipPort(widget ? widget->parentWidget() : NULL);

    if (!potentialClipPort || potentialClipPort->isTopLevel())
        return false;

    QWidget *potentialViewPort(potentialClipPort->parentWidget());

    if (!potentialViewPort || potentialViewPort->isTopLevel() ||
        qstrcmp(potentialViewPort->name(), "qt_viewport"))
        return false;

    QWidget *potentialKHTML(potentialViewPort->parentWidget());

    if (!potentialKHTML || potentialKHTML->isTopLevel() ||
        qstrcmp(potentialKHTML->className(), "KHTMLView"))
        return false;

    return true;
}

static void shade(const QColor &a, QColor *b, float k)
{
    if(equal(k, 1.0))
        *b=a;
    else
    {
        double red(a.red()/256.0),
               green(a.green()/256.0),
               blue(a.blue()/256.0);

        rgb2Hls(&red, &green, &blue);

        green *=k;
        if(green > 1.0)
            green=1.0;
        else if(green < 0.0)
            green=0.0;

        blue *=k;
        if(blue > 1.0)
            blue=1.0;
        else if(blue < 0.0)
            blue=0.0;

        hls2Rgb(&red, &green, &blue);
        b->setRgb(limit(red*256), limit(green*256), limit(blue*256));
    }
}

QColor shade(const QColor &a, float k)
{
    QColor mod;

    shade(a, &mod, k);
    return mod;
}

static void drawLines(QPainter *p, const QRect &r, bool horiz, int nLines, int offset,
                      const QColor *cols, int startOffset, bool etched, int dark, int etchedDisp=1)
{
    int space((nLines*2)+(etchedDisp ? (nLines-1) : 0)),
        step(etchedDisp ? 3 : 2),
        x(horiz ? r.x(): r.x()+((r.width()-space)>>1)),
        y(horiz ? r.y()+((r.height()-space)>>1): r.y()),
        x2(r.x()+r.width()-1),
        y2(r.y()+r.height()-1),
        i,
        displacement(etched ? etchedDisp : 0);

    if(horiz)
    {
        if(startOffset && y+startOffset>0)
            y+=startOffset;

        p->setPen(cols[etched ? dark : 0]);
        for(i=0; i<space; i+=step)
            p->drawLine(x+offset, y+i, x2-(offset+displacement), y+i);

        p->setPen(cols[etched ? 0 : dark]);
        for(i=1; i<space; i+=step)
            p->drawLine(x+offset+displacement, y+i, x2-offset, y+i);
    }
    else
    {
        if(startOffset && x+startOffset>0)
            x+=startOffset;

        p->setPen(cols[etched ? dark : 0]);
        for(i=0; i<space; i+=step)
            p->drawLine(x+i, y+offset, x+i, y2-(offset+displacement));

        p->setPen(cols[etched ? 0 : dark]);
        for(i=1; i<space; i+=step)
            p->drawLine(x+i, y+offset+displacement, x+i, y2-offset);
    }
}

static void drawDots(QPainter *p, const QRect &r, bool horiz, int nLines, int offset,
                     const QColor *cols, int startOffset, int dark, bool large=false)
{
    int space((nLines*2)+(nLines-1)),
        x(horiz ? r.x() : r.x()+((r.width()-space)>>1)),
        y(horiz ? r.y()+((r.height()-space)>>1) : r.y()),
        i, j,
        numDots((horiz ? (r.width()-(2*offset))/3 : (r.height()-(2*offset))/3)+1);

    if(large && 5==dark)
        dark=3;

    if(horiz)
    {
        if(startOffset && y+startOffset>0)
            y+=startOffset;

        p->setPen(cols[dark]);
        for(i=0; i<space; i+=3)
            for(j=0; j<numDots; j++)
                if(large)
                    p->drawRect(x+offset+(3*j), y+i, 2, 2);
                else
                    p->drawPoint(x+offset+(3*j), y+i);

        p->setPen(cols[0]);
        for(i=1; i<space; i+=3)
            for(j=0; j<numDots; j++)
                p->drawPoint(x+offset+1+(3*j), y+i);
    }
    else
    {
        if(startOffset && x+startOffset>0)
            x+=startOffset;

        p->setPen(cols[dark]);
        for(i=0; i<space; i+=3)
            for(j=0; j<numDots; j++)
                if(large)
                    p->drawRect(x+i, y+offset+(3*j), 2, 2);
                else
                    p->drawPoint(x+i, y+offset+(3*j));

        p->setPen(cols[0]);
        for(i=1; i<space; i+=3)
            for(j=0; j<numDots; j++)
                p->drawPoint(x+i, y+offset+1+(3*j));
    }
}

static bool onToolBar(QWidget *widget, int l=0)
{
    return l<3 && widget && widget->parentWidget()
               ? widget->parentWidget()->inherits("QToolBar") ||
                 onToolBar(widget->parentWidget(), ++l)
               : false;
}

enum ECacheAppearance
{
    CACHE_APPEARANCE_SELECTED=APPEARANCE_BEVELLED+1
};

inline int app2App(EAppearance app, bool sel)
{
    return sel ? (int)CACHE_APPEARANCE_SELECTED : (int)app;
}

#define QTC_PIXMAP_DIMENSION 10

static int     double2int(double d) { return (int)(d*100); }
static QString createKey(int size, QRgb color, bool horiz, bool increase=false, int app=0,
                         EWidget w=WIDGET_OTHER, double shadeTop=0.0, double shadeBot=0.0)
{
    QString key;

    if(WIDGET_DEF_BUTTON==w && !IS_GLASS(app)) // Glass uses different shading for def button...
        w=WIDGET_STD_BUTTON;

    QTextOStream(&key) << size << color << horiz << increase << app << (int)w
                       << ' ' << double2int(shadeTop) << ' ' << double2int(shadeBot);

    return key;
}

static QString createKey(QRgb color)
{
    QString key;

    QTextOStream(&key) << 'p' << color;

    return key;
}

static QString createKey(QRgb color, EPixmap p)
{
    QString key;

    QTextOStream(&key) << 'P' << color << p;

    return key;
}

//
// OO.o 2.x checks to see whether the used theme "inherits" from HighContrastStyle,
// if so it uses the highlightedText color to draw highlighted menubar and popup menu
// items. Otherwise it uses the standard color. So faking this inheritance, solves this
// problem.
HighContrastStyle::HighContrastStyle()
                 : KStyle(AllowMenuTransparency, WindowsStyleScrollBar)
{
}

QtCurveStyle::QtCurveStyle()
            : itsSliderCols(NULL),
              itsSbSliderCols(NULL),
              itsListviewCols(NULL),
              itsDefBtnCols(NULL),
              itsMouseOverCols(NULL),
              itsSidebarButtonsCols(NULL),
              itsThemedApp(APP_OTHER),
              itsPixmapCache(150000, 499),
#if KDE_VERSION >= 0x30200
              itsIsTransKicker(false),
#endif
              itsHover(HOVER_NONE),
              itsOldPos(-1, -1),
              itsFormMode(false),
              itsHoverWidget(NULL),
              itsHoverSect(QTC_NO_SECT),
              itsHoverTab(NULL),
              itsMactorPal(NULL),
              itsActive(true)
{
    defaultSettings(&opts);
    readConfig(NULL, &opts, &opts);
    itsPixmapCache.setAutoDelete(true);

    shadeColors(qApp->palette().active().highlight(), itsMenuitemCols);
    shadeColors(qApp->palette().active().background(), itsBackgroundCols);
    shadeColors(qApp->palette().active().button(), itsButtonCols);
    if(SHADE_NONE!=opts.shadeSliders)
    {
        itsSliderCols=new QColor [TOTAL_SHADES+1];
        shadeColors(SHADE_SELECTED==opts.shadeSliders
                        ? midColor(itsMenuitemCols[ORIGINAL_SHADE],
                                   itsButtonCols[ORIGINAL_SHADE])
                        : opts.customSlidersColor,
                    itsSliderCols);
    }

    if(SHADE_NONE!=opts.shadeSbSliders)
    {
        if((SHADE_SELECTED==opts.shadeSbSliders && SHADE_SELECTED==opts.shadeSliders) ||
           (SHADE_CUSTOM==opts.shadeSbSliders && SHADE_CUSTOM==opts.shadeSliders &&
            opts.customSlidersColor==opts.customSbSlidersColor))
            itsSbSliderCols=itsSliderCols;
        else
        {
            itsSbSliderCols=new QColor [TOTAL_SHADES+1];
            shadeColors(SHADE_SELECTED==opts.shadeSbSliders
                            ? midColor(itsMenuitemCols[ORIGINAL_SHADE],
                                       itsButtonCols[ORIGINAL_SHADE])
                            : opts.customSbSlidersColor,
                        itsSbSliderCols);
        }
    }

    if(IND_COLORED==opts.defBtnIndicator)
        if(SHADE_SELECTED==opts.shadeSliders)
            itsDefBtnCols=itsSliderCols;
        else
        {
            itsDefBtnCols=new QColor [TOTAL_SHADES+1];
            shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                                 itsButtonCols[ORIGINAL_SHADE]), itsDefBtnCols);
        }

    if(opts.coloredMouseOver)
        if(itsDefBtnCols)
            itsMouseOverCols=itsDefBtnCols;
        else
        {
            itsMouseOverCols=new QColor [TOTAL_SHADES+1];
            shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                                 itsButtonCols[ORIGINAL_SHADE]), itsMouseOverCols);
        }

    switch(opts.lvHeaderColor)
    {
        default:
        case HEADER_COLOR_BGND:
            itsListviewCols=itsBackgroundCols;
            break;
        case HEADER_COLOR_BTN:
            itsListviewCols=itsButtonCols;
            break;
        case HEADER_COLOR_CUSTOM:
            itsListviewCols=new QColor [TOTAL_SHADES+1];
            shadeColors(opts.customLvHeaderColor, itsListviewCols);
    }

    setMenuColors(qApp->palette().active());

    if(opts.lighterPopupMenuBgnd)
        itsLighterPopupMenuBgndCol=shade(itsBackgroundCols[ORIGINAL_SHADE],
                                         POPUPMENU_LIGHT_FACTOR);

    if(FOCUS_COLORED==opts.focus)
        itsFocusCol=midColor(qApp->palette().active().highlight(),
                             qApp->palette().active().background());

    switch(opts.shadeCheckRadio)
    {
        default:
            itsCheckRadioCol=qApp->palette().active().text();
            break;
        case SHADE_SELECTED:
            itsCheckRadioCol=qApp->palette().active().highlight();
            break;
        case SHADE_CUSTOM:
            itsCheckRadioCol=opts.customCheckRadioColor;
    }

    if (opts.animatedProgress)
    {
        itsAnimationTimer = new QTimer(this);
        connect(itsAnimationTimer, SIGNAL(timeout()), this, SLOT(updateProgressPos()));
    }

    setSbType();

    if(!opts.inactiveHighlight)
        itsInactiveHighlightCol=midColor(itsBackgroundCols[ORIGINAL_SHADE],
                                         itsMenuitemCols[ORIGINAL_SHADE],
                                         INACTIVE_HIGHLIGHT_FACTOR);
}

QtCurveStyle::~QtCurveStyle()
{
    if(itsSidebarButtonsCols!=itsSliderCols && itsSidebarButtonsCols!=itsSbSliderCols &&
       itsSidebarButtonsCols!=itsDefBtnCols)
        delete [] itsSidebarButtonsCols;
    if(itsMouseOverCols && itsMouseOverCols!=itsDefBtnCols &&
       itsMouseOverCols!=itsSliderCols)
        delete [] itsMouseOverCols;
    if(itsDefBtnCols && itsDefBtnCols!=itsSliderCols)
        delete [] itsDefBtnCols;
    if(itsSbSliderCols && itsSbSliderCols!=itsSliderCols)
        delete [] itsSbSliderCols;
    if(itsSliderCols)
        delete [] itsSliderCols;
    if(HEADER_COLOR_CUSTOM==opts.lvHeaderColor)
        delete [] itsListviewCols;
    if(itsMactorPal)
        delete itsMactorPal;
}

static QString getFile(const QString &f)
{
    QString d(f);

    int slashPos(d.findRev('/'));

    if(slashPos!=-1)
        d.remove(0, slashPos+1);

    return d;
}

void QtCurveStyle::polish(QApplication *app)
{
    QString appName(getFile(app->argv()[0]));

    if ("kicker"==appName || "appletproxy"==appName)
    {
        itsThemedApp=APP_KICKER;
#if KDE_VERSION >= 0x30200
        itsIsTransKicker=QTC_ROUNDED && kickerIsTrans();
#endif
    }
    else if ("korn"==appName)
    {
        itsThemedApp=APP_KORN;
#if KDE_VERSION >= 0x30200
        itsIsTransKicker=QTC_ROUNDED && kickerIsTrans();
#endif
    }
    else if ("konqueror"==appName)
        itsThemedApp=APP_KONQUEROR;
    else if ("mactor"==appName)
    {
        if(!itsMactorPal)
            itsMactorPal=new QPalette(qApp->palette());
        itsThemedApp=APP_MACTOR;
    }
    else if ("kpresenter"==appName)
        itsThemedApp=APP_KPRESENTER;
    else if ("soffice.bin"==appName)
        itsThemedApp=APP_OPENOFFICE;
    else if ("kdefilepicker"==appName)
        itsThemedApp=APP_SKIP_TASKBAR;
    else if ("kprinter"==appName)
        itsThemedApp=APP_KPRINTER;
    else if ("kdialog"==appName)
        itsThemedApp=APP_KDIALOG;
    else if ("kdialogd"==appName)
        itsThemedApp=APP_KDIALOGD;
    else if ("tora"==appName)
        itsThemedApp=APP_TORA;
    else
        itsThemedApp=APP_OTHER;

    if(APP_OPENOFFICE==itsThemedApp)
    {
        if(opts.scrollbarType==SCROLLBAR_NEXT)
            opts.scrollbarType=SCROLLBAR_KDE;
        else if(opts.scrollbarType==SCROLLBAR_NONE)
            opts.scrollbarType=SCROLLBAR_WINDOWS;
        setSbType();
    }
}

void QtCurveStyle::polish(QPalette &pal)
{
    if(APP_MACTOR==itsThemedApp && itsMactorPal &&
       pal.active().background()!=itsMactorPal->active().background())
        return;

    int  c(QSettings().readNumEntry("/Qt/KDE/contrast", 7));
    bool newContrast(false);

    if(c<0 || c>10)
        c=7;

    if(c!=opts.contrast)
    {
        opts.contrast=c;
        newContrast=true;
    }

    bool newMenu(newContrast ||
                 itsMenuitemCols[ORIGINAL_SHADE]!=qApp->palette().active().highlight()),
         newGray(newContrast ||
                 itsBackgroundCols[ORIGINAL_SHADE]!=qApp->palette().active().background()),
         newButton(newContrast ||
                   itsButtonCols[ORIGINAL_SHADE]!=qApp->palette().active().button()),
         newSlider(itsSliderCols && SHADE_SELECTED==opts.shadeSliders &&
                   (newContrast || newButton || newMenu)),
         newSbSlider(itsSbSliderCols && itsSbSliderCols!=itsSliderCols &&
                     SHADE_SELECTED==opts.shadeSbSliders &&
                     (newContrast || newButton || newMenu)),
         newDefBtn(itsDefBtnCols && IND_COLORED==opts.defBtnIndicator &&
                   SHADE_SELECTED!=opts.shadeSliders &&
                   (newContrast || newButton || newMenu)),
         newMouseOver(itsMouseOverCols && itsMouseOverCols!=itsDefBtnCols &&
                      itsMouseOverCols!=itsSliderCols &&
                     (newContrast || newButton || newMenu));

    if(newGray)
        shadeColors(qApp->palette().active().background(), itsBackgroundCols);

    if(newButton)
        shadeColors(qApp->palette().active().button(), itsButtonCols);

    if(newMenu)
        shadeColors(qApp->palette().active().highlight(), itsMenuitemCols);

    setMenuColors(qApp->palette().active());

    if(newSlider)
        shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                    itsButtonCols[ORIGINAL_SHADE]), itsSliderCols);

    if(newSbSlider)
        shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                    itsButtonCols[ORIGINAL_SHADE]), itsSbSliderCols);

    if(newDefBtn)
        shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                    itsButtonCols[ORIGINAL_SHADE]), itsDefBtnCols);

    if(newMouseOver)
        shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                    itsButtonCols[ORIGINAL_SHADE]), itsMouseOverCols);

    if(itsSidebarButtonsCols && SHADE_SELECTED!=opts.shadeSliders &&
       SHADE_SELECTED!=opts.shadeSbSliders && IND_COLORED!=opts.defBtnIndicator)
        shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE],
                   itsButtonCols[ORIGINAL_SHADE]), itsSidebarButtonsCols);

    if(opts.lighterPopupMenuBgnd && newGray)
        itsLighterPopupMenuBgndCol=shade(itsBackgroundCols[ORIGINAL_SHADE],
                                         POPUPMENU_LIGHT_FACTOR);

    if(FOCUS_COLORED==opts.focus)
        itsFocusCol=midColor(qApp->palette().active().highlight(),
                            qApp->palette().active().background());

    const QColorGroup      &actGroup(pal.active()),
                           &inactGroup(pal.inactive()),
                           &disGroup(pal.disabled());
    const QColor           *use(backgroundColors(actGroup));
    QColorGroup            newAct(actGroup.foreground(), actGroup.button(), use[0], use[5],
                                  actGroup.mid(), actGroup.text(), actGroup.brightText(),
                                  actGroup.base(), actGroup.background());
    QColorGroup::ColorRole roles[]={QColorGroup::Midlight, QColorGroup::ButtonText,
                                    QColorGroup::Shadow, QColorGroup::Highlight,
                                    QColorGroup::HighlightedText,
                                    QColorGroup::NColorRoles };
    int                    r(0);

    for(r=0; roles[r]!=QColorGroup::NColorRoles; ++r)
        newAct.setColor(roles[r], actGroup.color(roles[r]));
    pal.setActive(newAct);

    use=backgroundColors(inactGroup);

    QColorGroup newInact(inactGroup.foreground(), inactGroup.button(), use[0], use[5],
                         inactGroup.mid(), inactGroup.text(), inactGroup.brightText(),
                         inactGroup.base(), inactGroup.background());

    for(r=0; roles[r]!=QColorGroup::NColorRoles; ++r)
        newInact.setColor(roles[r], inactGroup.color(roles[r]));
    pal.setInactive(newInact);

    use=backgroundColors(disGroup);

    QColorGroup newDis(disGroup.foreground(), disGroup.button(), use[0], use[5],
                       disGroup.mid(), disGroup.text(), disGroup.brightText(),
                       disGroup.base(), disGroup.background());

    for(r=0; roles[r]!=QColorGroup::NColorRoles; ++r)
        newDis.setColor(roles[r], disGroup.color(roles[r]));
    pal.setDisabled(newDis);

    if(!opts.inactiveHighlight)
    {
        itsInactiveHighlightCol=midColor(itsBackgroundCols[ORIGINAL_SHADE],
                                         itsMenuitemCols[ORIGINAL_SHADE],
                                         INACTIVE_HIGHLIGHT_FACTOR);
        pal.setColor(QPalette::Inactive, QColorGroup::Highlight, itsInactiveHighlightCol);
    }

    switch(opts.shadeCheckRadio)
    {
        default:
            itsCheckRadioCol=qApp->palette().active().text();
            break;
        case SHADE_SELECTED:
            itsCheckRadioCol=qApp->palette().active().highlight();
            break;
        case SHADE_CUSTOM:
            itsCheckRadioCol=opts.customCheckRadioColor;
    }

    if(itsMactorPal)
        *itsMactorPal=pal;
}

static const char * kdeToolbarWidget="kde toolbar widget";

void QtCurveStyle::polish(QWidget *widget)
{
    bool enableFilter(opts.highlightFactor>1.0 || opts.coloredMouseOver);

    if (APP_MACTOR==itsThemedApp && itsMactorPal && !widget->inherits("QTipLabel"))
        widget->setPalette(*itsMactorPal);

    if (::qt_cast<QRadioButton *>(widget) || ::qt_cast<QCheckBox *>(widget))
    {
        if(enableFilter)
        {
#if QT_VERSION >= 0x030200
            if(!isFormWidget(widget))
                widget->setMouseTracking(true); 
#endif
            widget->installEventFilter(this);
        }
    }
    else if (::qt_cast<QSpinWidget *>(widget))
    {
        modifyInactivePal(widget, false);
        if(enableFilter)
        {
            widget->setMouseTracking(true);
            widget->installEventFilter(this);
        }
    }
    else if (::qt_cast<QHeader *>(widget) || ::qt_cast<QTabBar *>(widget))
    {
        if(enableFilter)
        {
            widget->setMouseTracking(true);
            widget->installEventFilter(this);
        }
    }
    else if (::qt_cast<QButton *>(widget) || ::qt_cast<QButton *>(widget) ||
             widget->inherits("QToolBarExtensionWidget"))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteBackground);
        if(enableFilter)
            widget->installEventFilter(this);
    }
    else if (::qt_cast<QToolButton *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteBackground);
        if(enableFilter)
        {
            widget->installEventFilter(this);
#if KDE_VERSION >= 0x30400 && KDE_VERSION < 0x30500
            widget->setMouseTracking(true);
#endif
        }
    }
    else if (::qt_cast<QComboBox *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteBackground);
        widget->installEventFilter(this);

        if(((QComboBox *)widget)->listBox())
            ((QComboBox *)widget)->listBox()->installEventFilter(this);
    }
    else if(::qt_cast<QMenuBar *>(widget) || ::qt_cast<QToolBar *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode())
            widget->setBackgroundMode(QWidget::PaletteBackground);
        if(SHADE_NONE!=opts.shadeMenubars)
            widget->installEventFilter(this);
    }
    else if(::qt_cast<QPopupMenu *>(widget))
            widget->setBackgroundMode(QWidget::NoBackground); // QWidget::PaletteBackground);
    else if (widget->inherits("KToolBarSeparator") ||
             (widget->inherits("KListViewSearchLineWidget") &&
              widget->parent() && ::qt_cast<QToolBar *>(widget->parent())))
    {
        widget->setName(kdeToolbarWidget);
        widget->setBackgroundMode(QWidget::NoBackground);
        widget->installEventFilter(this);
    }
    else if (::qt_cast<QScrollView *>(widget))
        modifyInactivePal(widget, false);
    else if (::qt_cast<QScrollBar *>(widget))
    {
#ifdef QTC_SLIDER_SHADING
        modifyInactivePal(widget, false);
#endif
        if(enableFilter)
        {
            if(enableFilter)
                widget->setMouseTracking(true);
            widget->installEventFilter(this);
        }
        //widget->setBackgroundMode(QWidget::NoBackground);
    }
    else if (::qt_cast<QSlider *>(widget))
    {
#ifdef QTC_SLIDER_SHADING
        modifyInactivePal(widget, false);
#endif
        if(enableFilter)
            widget->installEventFilter(this);
        if(widget->parent() && ::qt_cast<QToolBar *>(widget->parent()))
        {
            widget->setName(kdeToolbarWidget);
            widget->setBackgroundMode(QWidget::NoBackground);  // We paint whole background.
        }

        if(opts.fillSlider)
        {
            // This bit stolen form polyester...
            connect(widget, SIGNAL(sliderMoved(int)), this, SLOT(sliderThumbMoved(int)));
            connect(widget, SIGNAL(valueChanged(int)), this, SLOT(sliderThumbMoved(int)));
        }
    }
    else if (::qt_cast<QLineEdit*>(widget) || ::qt_cast<QTextEdit*>(widget))
    {
        modifyInactivePal(widget, false);
        widget->installEventFilter(this);
    }
    else if (widget->inherits("QSplitterHandle") || widget->inherits("QDockWindowHandle"))
    {
        if(enableFilter)
            widget->installEventFilter(this);
    }
    else if (0==qstrcmp(widget->name(), kdeToolbarWidget))
    {
        if(!widget->parent() ||
           0!=qstrcmp(static_cast<QWidget *>(widget->parent())->className(),
                                             "KListViewSearchLineWidget") ||
           onToolBar(widget))
        {
            widget->installEventFilter(this);
            widget->setBackgroundMode(QWidget::NoBackground);  // We paint whole background.
        }
    }
    else if (::qt_cast<QLabel *>(widget) &&
             widget->parent() &&
             ( 0==qstrcmp(static_cast<QWidget *>(widget->parent())->className(),
                          "MainWindow") ||
               ::qt_cast<QToolBar *>(widget->parent())))
    {
        widget->setName(kdeToolbarWidget);
        widget->installEventFilter(this);
        widget->setBackgroundMode(QWidget::NoBackground);  // We paint the whole background.
    }
    else if(opts.animatedProgress && ::qt_cast<QProgressBar*>(widget))
    {
        widget->installEventFilter(this);
        itsProgAnimWidgets[widget] = 0;
        connect(widget, SIGNAL(destroyed(QObject *)), this, SLOT(progressBarDestroyed(QObject *)));
        if (!itsAnimationTimer->isActive())
            itsAnimationTimer->start(PROGRESS_ANIMATION, false);
    }
    else if(!qstrcmp(widget->className(), "KonqFrameStatusBar"))
    {
        // This disables the white background of the KonquerorFrameStatusBar.
        // When the green led is shown the background is set to
        // applications cg.midlight() so we override it to standard background.
        // Thanks Comix! (because this was ugly from day one!)
        // NOTE: Check if we can set it earlier (before painting), cause
        // on slow machines we can see the repainting of the bar (from white to background...)
        QPalette pal(QApplication::palette());

        pal.setColor(QColorGroup::Midlight, pal.active().background());
        QApplication::setPalette(pal);
    }
    else if(widget->inherits("KTabCtl"))
        widget->installEventFilter(this);
    else if(opts.fixParentlessDialogs && ::qt_cast<QDialog *>(widget))
    {
        QDialog *dlg=(QDialog *)widget;

        // The parent->isShown is needed for KWord. It's insert picure file dialog is a child of
        // the insert picture dialog - but the file dialog is shown *before* the picture dialog!
        if( (QTC_SKIP_TASKBAR && !dlg->parentWidget()) ||
            ( (!dlg->parentWidget() || !dlg->parentWidget()->isShown())// &&
              /*(dlg->isModal() || ::qt_cast<QProgressDialog *>(widget))*/) )
            widget->installEventFilter(this);
    }

    if(opts.fixParentlessDialogs && (APP_KPRINTER==itsThemedApp || APP_KDIALOG==itsThemedApp ||
                                     APP_KDIALOGD==itsThemedApp))
    {
        QString cap(widget->caption());
        int     index=-1;

        // Remove horrible "Open - KDialog" titles...
        if( cap.length() &&
            ( (APP_KPRINTER==itsThemedApp && (-1!=(index=cap.find(" - KPrinter"))) &&
               (index+11)==(int)cap.length()) ||
              (APP_KDIALOG==itsThemedApp && (-1!=(index=cap.find(" - KDialog"))) &&
               (index+10)==(int)cap.length()) ||
              (APP_KDIALOGD==itsThemedApp && (-1!=(index=cap.find(" - KDialog Daemon"))) &&
               (index+17)==(int)cap.length())) )
            widget->QWidget::setCaption(cap.left(index));
    }

    KStyle::polish(widget);
}

void QtCurveStyle::unPolish(QWidget *widget)
{
    if (::qt_cast<QRadioButton *>(widget) || ::qt_cast<QCheckBox *>(widget))
    {
#if QT_VERSION >= 0x030200
        widget->setMouseTracking(false);
#endif
        widget->removeEventFilter(this);
    }
    else if (::qt_cast<QSpinWidget *>(widget))
    {
        resetInactivePal(widget);
        widget->setMouseTracking(false);
        widget->removeEventFilter(this);
    }
    else if (::qt_cast<QHeader *>(widget) || ::qt_cast<QTabBar *>(widget))
    {
        widget->setMouseTracking(false);
        widget->removeEventFilter(this);
    }
    else if (::qt_cast<QButton *>(widget) || widget->inherits("QToolBarExtensionWidget"))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteButton);
        widget->removeEventFilter(this);
    }
    else if (::qt_cast<QToolButton *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteButton);
        widget->removeEventFilter(this);
#if KDE_VERSION >= 0x30400 && KDE_VERSION < 0x30500
        widget->setMouseTracking(false);
#endif
    }
    else if (::qt_cast<QComboBox *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode()) //  && onToolBar(widget))
            widget->setBackgroundMode(QWidget::PaletteButton);
        widget->removeEventFilter(this);
        if(((QComboBox *)widget)->listBox())
            ((QComboBox *)widget)->listBox()->removeEventFilter(this);
    }
    else if (::qt_cast<QToolBar *>(widget) || ::qt_cast<QPopupMenu *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode())
           widget->setBackgroundMode(QWidget::PaletteBackground);
    }
    else if (::qt_cast<QMenuBar *>(widget))
    {
        if(QWidget::NoBackground!=widget->backgroundMode())
           widget->setBackgroundMode(QWidget::PaletteBackground);
        if(SHADE_SELECTED==opts.shadeMenubars || SHADE_CUSTOM==opts.shadeMenubars)
            widget->installEventFilter(this);
    }
    else if (widget->inherits("KToolBarSeparator"))
    {
        widget->setBackgroundMode(PaletteBackground);
        widget->removeEventFilter(this);
    }
    else if (::qt_cast<QScrollView *>(widget))
        resetInactivePal(widget);
    else if (::qt_cast<QScrollBar *>(widget))
    {
#ifdef QTC_SLIDER_SHADING
        resetInactivePal(widget, false);
#endif
        widget->setMouseTracking(false);
        widget->removeEventFilter(this);
        widget->setBackgroundMode(QWidget::PaletteButton);
    }
    else if (::qt_cast<QSlider *>(widget))
    {
#ifdef QTC_SLIDER_SHADING
        resetInactivePal(widget, false);
#endif
        widget->removeEventFilter(this);
        if(widget->parent() && ::qt_cast<QToolBar *>(widget->parent()))
            widget->setBackgroundMode(PaletteBackground);
    }
    else if (::qt_cast<QLineEdit*>(widget) || ::qt_cast<QTextEdit*>(widget))
    {
        resetInactivePal(widget);
        widget->removeEventFilter(this);
    }
    else if (widget->inherits("QSplitterHandle") || widget->inherits("QDockWindowHandle"))
        widget->removeEventFilter(this);
    else if (::qt_cast<QProgressBar*>(widget))
    {
        itsProgAnimWidgets.remove(widget);
        widget->removeEventFilter(this);
    }
    else if(0==qstrcmp(widget->name(), kdeToolbarWidget))
    {
        widget->removeEventFilter(this);
        widget->setBackgroundMode(PaletteBackground);
    }
    else if(widget->inherits("KTabCtl"))
        widget->removeEventFilter(this);
    else if(opts.fixParentlessDialogs && ::qt_cast<QDialog *>(widget))
        widget->removeEventFilter(this);
    KStyle::unPolish(widget);
}

static void sendXEvent(QDialog *dlg, const char *msg)
{
    static Atom msgTypeAtom = XInternAtom(qt_xdisplay(), "_NET_WM_STATE", False);

    XEvent xev;
    Atom   atom=XInternAtom(qt_xdisplay(), msg, False);

    xev.xclient.type = ClientMessage;
    xev.xclient.serial = 0;
    xev.xclient.send_event = True;
    xev.xclient.window = dlg->winId();
    xev.xclient.message_type = msgTypeAtom;
    xev.xclient.format = 32;
    xev.xclient.data.l[0] = 1;
    xev.xclient.data.l[1] = atom;
    xev.xclient.data.l[2] = 0;
    xev.xclient.data.l[3] = 0;
    xev.xclient.data.l[4] = 0;

    XSendEvent(qt_xdisplay(), qt_xrootwin(), False,
               SubstructureRedirectMask | SubstructureNotifyMask, &xev);
}

bool QtCurveStyle::appIsNotEmbedded(QDialog *dlg)
{
    Window win;

    if(!XGetTransientForHint(qt_xdisplay(), dlg->winId(), &win) || (int)win < 1000)
        return true;

    // OK, dialog has been set transient, so there is no need for this event filter anymore :-)
    dlg->removeEventFilter(this);
    return false;
}

bool QtCurveStyle::eventFilter(QObject *object, QEvent *event)
{
    if(object->parent() && 0==qstrcmp(object->name(), kdeToolbarWidget))
    {
        // Draw background for custom widgets in the toolbar that have specified a
        // "kde toolbar widget" name.
        if(QEvent::Paint==event->type())
        {
            QWidget *widget(static_cast<QWidget*>(object)),
                    *parent(static_cast<QWidget*>(object->parent()));

            if(IS_FLAT(opts.toolbarAppearance))
                QPainter(widget).fillRect(widget->rect(), parent->colorGroup().background());
            else
            {
                int y_offset(widget->y());

                while(parent && parent->parent() && 0==qstrcmp(parent->name(), kdeToolbarWidget))
                {
                    y_offset += parent->y();
                    parent = static_cast<QWidget*>(parent->parent());
                }

                QRect    r(widget->rect()),
                         pr(parent->rect());
                bool     horiz=pr.width() > pr.height();
                QPainter p(widget);
                QToolBar *tb(::qt_cast<QToolBar*>(parent));

                // If parent is a QToolbar use its orientation, else just base on width>height.
                if (tb)
                    horiz = Qt::Horizontal==tb->orientation();

                QRect bgndRect(r.x(), r.y()-y_offset, r.width(), pr.height());

                if(!IS_FLAT(opts.toolbarAppearance))
                    if(horiz)
                        bgndRect.addCoords(0, -1, 0, 1);
                    else
                        bgndRect.addCoords(-1, 0, 1, 0);

                drawMenuOrToolBarBackground(&p, bgndRect, parent->colorGroup(), false, horiz);
            }
            return false;   // Now draw the contents
        }
    }
    else if (QEvent::Paint==event->type())
    {
        if (object->inherits("KToolBarSeparator"))
        {
            QFrame *frame(::qt_cast<QFrame*>(object));

            if(frame && QFrame::NoFrame!=frame->frameShape())
            {
                QPainter painter(frame);
                if (QFrame::VLine==frame->frameShape())
                    drawPrimitive(PE_DockWindowSeparator, &painter, frame->rect(),
                                  frame->colorGroup(), Style_Horizontal);
                else if (QFrame::HLine==frame->frameShape())
                    drawPrimitive(PE_DockWindowSeparator, &painter, frame->rect(),
                                  frame->colorGroup());
                else
                    return false;
                return true; // been drawn!
            }
        }
        else if(object->inherits("KTabCtl") && ::qt_cast<QWidget*>(object))
        {
            QWidget  *widget((QWidget*)object);
            QObject  *child(object->child("_tabbar"));
            QTabBar  *tb(child ? ::qt_cast<QTabBar *>(child) : NULL);
            QPainter painter(widget);
            QRect    r(widget->rect());
            int      tbHeight(tb ? tb->height()-1 : 28);

            if(tb && (QTabBar::RoundedBelow == tb->shape() ||
                      QTabBar::TriangularBelow == tb->shape()))
                r.addCoords(0, 0, 0, -tbHeight);
            else
                r.addCoords(0, tbHeight, 0, 0);
            drawPrimitive(PE_PanelTabWidget, &painter, r, widget->colorGroup(),
                          Style_Horizontal|Style_Enabled);
            return true;
        }
    }

    // Taken from plastik...
    // focus highlight
    if (::qt_cast<QLineEdit*>(object) || ::qt_cast<QTextEdit*>(object))
    {
        QWidget *widget(static_cast<QWidget*>(object));

        if (::qt_cast<QSpinWidget*>(widget->parentWidget()))
        {
            QWidget *spinbox = widget->parentWidget();
            if (QEvent::FocusIn==event->type() || QEvent::FocusOut==event->type())
                spinbox->repaint(false);
            return false;
        }

         // For some reason, for comboboxes I need to set the palette here :-(
        if(widget->parentWidget() && ::qt_cast<QComboBox*>(widget->parentWidget()) &&
            (QEvent::WindowDeactivate==event->type() || QEvent::FocusOut==event->type()))
            modifyInactivePal(widget);

        if (QEvent::FocusIn==event->type() || QEvent::FocusOut==event->type())
            widget->repaint(false);
        return false;
    }

    if((opts.shadeMenubarOnlyWhenActive && SHADE_NONE!=opts.shadeMenubars &&
       ::qt_cast<QMenuBar *>(object))
#ifdef QTC_SLIDER_SHADING
        ||
       (!opts.inactiveHighlight && ( (SHADE_NONE!=opts.shadeSbSliders &&
                                      ::qt_cast<QScrollBar *>(object)) ||
                                      (SHADE_NONE!=opts.shadeSliders &&
                                      ::qt_cast<QSlider *>(object))))
#endif
        )
        switch(event->type())
        {
            case QEvent::WindowActivate:
                itsActive=true;
                ((QWidget *)object)->repaint(false);
                return false;
            case QEvent::WindowDeactivate:
                itsActive=false;
                ((QWidget *)object)->repaint(false);
                return false;
            default:
                break;
        }

    if(opts.fixParentlessDialogs && ::qt_cast<QDialog *>(object))
    {
        QDialog *dlg=(QDialog *)object;

        switch(event->type())
        {
            case QEvent::ShowMinimized:
                if(QTC_SKIP_TASKBAR && appIsNotEmbedded(dlg))
                {
                    // Ugly hack :-( Cant seem to get KWin to remove the minimize button. So when
                    // the dialog gets minimized, restore.
                    dlg->setWindowState(dlg->windowState() & ~WindowMinimized | WindowActive);
                    return true;
                }
                break;
            case QEvent::WindowActivate:
                if(QTC_SKIP_TASKBAR && appIsNotEmbedded(dlg))
                {
                    // OO.o's filepicker is a spawned process - but is not set transient :-(
                    // --plus no reliable way of finding which widget to make it transient for...
                    sendXEvent(dlg, "_NET_WM_STATE_SKIP_PAGER");
                    sendXEvent(dlg, "_NET_WM_STATE_SKIP_TASKBAR");
                    sendXEvent(dlg, "_NET_WM_STATE_ABOVE");
                    sendXEvent(dlg, "_NET_WM_STATE_STAYS_ON_TOP");
                    //setActions(dlg);
                }
                break;
            case QEvent::Show:
                // The parent->isShown is needed for KWord. It's insert picure file dialog is a
                // child of the insert picture dialog - but the file dialog is shown *before* the
                // picture dialog!
                if((!dlg->parentWidget() || !dlg->parentWidget()->isShown())) // &&
                   //(dlg->isModal() || ::qt_cast<QProgressDialog *>(object)))
                {
                    QWidget *activeWindow=qApp->activeWindow();

                    if(activeWindow)
                    {
                        XWindowAttributes attr;
                        int               rx, ry;
                        Window            win;

                        if(!XGetTransientForHint(qt_xdisplay(), dlg->winId(), &win) ||
                           win!=activeWindow->winId())
                        {
                            XSetTransientForHint(qt_xdisplay(), dlg->winId(),
                                                 activeWindow->winId());

                            if(XGetWindowAttributes(qt_xdisplay(), activeWindow->winId(), &attr))
                            {
                                XTranslateCoordinates(qt_xdisplay(), activeWindow->winId(),
                                                      attr.root, -attr.border_width, -16,
                                                      &rx, &ry, &win);

                                rx=(rx+(attr.width/2))-(dlg->width()/2);
                                if(rx<0)
                                    rx=0;
                                ry=(ry+(attr.height/2))-(dlg->height()/2);
                                if(ry<0)
                                    ry=0;
                                dlg->move(rx, ry);
                                if(!dlg->isModal())
                                    dlg->setModal(true);
                            }
                        }
                    }
                }
            default:
                break;
        }
        return false;
    }

    // Track show events for progress bars
    if (opts.animatedProgress && ::qt_cast<QProgressBar*>(object))
    {
        if(QEvent::Show==event->type() && !itsAnimationTimer->isActive())
            itsAnimationTimer->start(PROGRESS_ANIMATION, false);
        return false;
    }

    switch(event->type())
    {
        case QEvent::Hide:
        case QEvent::Show:
            if(::qt_cast<QListBox *>(object) &&
              (((QListBox *)object)->parentWidget() &&
               ::qt_cast<QComboBox *>(((QListBox *)object)->parentWidget())))
                ((QComboBox *)(((QListBox *)object)->parentWidget()))->repaint(false);
            break;
        case QEvent::Destroy:
            if(itsHoverWidget && object==itsHoverWidget)
            {
                itsOldPos.setX(-1);
                itsOldPos.setY(-1);
                itsHoverWidget=NULL;
                itsHoverSect=QTC_NO_SECT;
                itsHover=HOVER_NONE;
            }
            break;
        case QEvent::Enter:
            if(object->isWidgetType())
            {
                itsHoverWidget=(QWidget *)object;
                if(itsHoverWidget && itsHoverWidget->isEnabled())
                {
                    if(::qt_cast<QTabBar*>(object) && static_cast<QWidget*>(object)->isEnabled())
                    {
                        itsHoverTab=NULL;
                        itsHoverWidget->repaint(false);
                    }
                    else if(!itsHoverWidget->hasMouseTracking() ||
                            (itsFormMode=isFormWidget(itsHoverWidget)))
                    {
                        itsHoverWidget->repaint(false);
                        itsFormMode=false;
                    }
                }
                else
                    itsHoverWidget=NULL;
            }
            break;
        case QEvent::Leave:
            if(itsHoverWidget && object==itsHoverWidget)
            {
                itsOldPos.setX(-1);
                itsOldPos.setY(-1);
                itsHoverWidget=NULL;
                itsHoverSect=QTC_NO_SECT;
                itsHover=HOVER_NONE;
                itsHoverTab=NULL;
                ((QWidget *)object)->repaint(false);
            }
            break;
        case QEvent::MouseMove:  // Only occurs for widgets with mouse tracking enabled
        {
            QMouseEvent *me = static_cast<QMouseEvent*>(event);

            if(me && itsHoverWidget && object->isWidgetType())
            {
                if(!me->pos().isNull() && redrawHoverWidget(me->pos()))
                    itsHoverWidget->repaint(false);
                itsOldPos=me->pos();
            }
            break;
        }
        default:
            break;
    }

    return KStyle::eventFilter(object, event);
}

void QtCurveStyle::drawLightBevel(const QColor &bgnd, QPainter *p, const QRect &r,
                                  const QColorGroup &cg, SFlags flags,
                                  int round, const QColor &fill, const QColor *custom,
                                  bool doBorder, bool doCorners, EWidget w) const
{
    QRect br(r);
    bool  bevelledButton(WIDGET_BUTTON(w) && APPEARANCE_BEVELLED==opts.appearance),
          sunken(flags &(QStyle::Style_Down | QStyle::Style_On | QStyle::Style_Sunken)),
          doFill(true),
          lightBorder((opts.lightSliderBorder && WIDGET_SLIDER==w) ||
                      (opts.lightSbSliderBorder && WIDGET_SB_SLIDER==w)),
          colouredMouseOver(doBorder &&
                            opts.coloredMouseOver && flags&QStyle::Style_MouseOver &&
                            !IS_SLIDER(w) &&
                            (flags&QTC_CHECK_BUTTON || flags&QTC_TOGGLE_BUTTON || !sunken));
    int   dark(bevelledButton ? 2 : 4),
          c1(sunken ? dark : 0);
    const QColor *cols(custom ? custom : itsBackgroundCols),
                 *border(colouredMouseOver ? borderColors(flags, cols) : cols);

    p->save();

    if(lightBorder)
        br.addCoords(1, 1,-1,-1);
    else if(colouredMouseOver || (!IS_GLASS(opts.appearance) && !sunken &&
                                   flags&QStyle::Style_Raised))
    {
        if(colouredMouseOver)
            p->setPen(border[sunken ? 1 : 0]);
        else
            p->setPen(border[c1]);
        if(colouredMouseOver || bevelledButton || APPEARANCE_RAISED==opts.appearance)
        {
            //Left & top
            p->drawLine(br.x()+1, br.y()+2, br.x()+1, br.y()+br.height()-3);
            p->drawLine(br.x()+1, br.y()+1, br.x()+br.width()-2, br.y()+1);

            if(colouredMouseOver)
                p->setPen(border[1]);
            else
                p->setPen(border[sunken ? 0 : dark]);
            //Right & bottom
            p->drawLine(br.x()+br.width()-2, br.y()+1, br.x()+br.width()-2, br.y()+br.height()-3);
            p->drawLine(br.x()+1, br.y()+br.height()-2, br.x()+br.width()-2, br.y()+br.height()-2);

            doFill=r.width()>4 && r.height()>4;
            br.addCoords(2, 2,-2,-2);
        }
        else
        {
            //Left & top
            p->drawLine(br.x()+1, br.y()+2, br.x()+1, br.y()+br.height()-2);
            p->drawLine(br.x()+1, br.y()+1, br.x()+br.width()-2, br.y()+1);

            doFill=r.width()>3 && r.height()>3;
            br.addCoords(2, 2,-1,-1);
        }
    }
    else
    {
        doFill=r.width()>2 && r.height()>2;
        br.addCoords(1, 1,-1,-1);
    }

    // fill
    if(doFill)
        drawBevelGradient(fill, !sunken, p, br, flags&Style_Horizontal, getWidgetShade(w, true,
                          sunken), getWidgetShade(w, false, sunken), sunken, opts.appearance, w);

    if(lightBorder)
    {
        p->setPen(cols[0]);
        br=r;
        br.addCoords(1,1,-1,-1);
        p->drawRect(br);
    }

    if(doBorder)
        drawBorder(bgnd, p, r, cg, flags, round, cols, doCorners);

    p->restore();
}

void QtCurveStyle::drawBorder(const QColor &bgnd, QPainter *p, const QRect &r, const QColorGroup &cg,
                              SFlags flags, int round, const QColor *custom, bool doCorners,
                              EBorder borderProfile, bool blendBorderColors) const
{
    const QColor *cols(custom ? custom : itsBackgroundCols);
    QColor       border(flags&Style_ButtonDefault && IND_FONT_COLOR==opts.defBtnIndicator &&
                        flags&Style_Enabled
                          ? cg.buttonText()
                          : cols[5]);

    switch(borderProfile)
    {
        case BORDER_FLAT:
            break;
        case BORDER_RAISED:
        case BORDER_SUNKEN:
            p->setPen(flags&Style_Enabled && (BORDER_RAISED==borderProfile ||
                      APPEARANCE_FLAT!=opts.appearance)
                         ? blendBorderColors
                               ? midColor(cg.base(), cols[BORDER_RAISED==borderProfile ? 0 : 4])
                               : cols[BORDER_RAISED==borderProfile ? 0 : 4]
                         : cg.background());
            p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.y()+r.height()-2);
            p->drawLine(r.x()+1, r.y()+1, r.x()+r.width()-2, r.y()+1);
            p->setPen(flags&Style_Enabled && (BORDER_SUNKEN==borderProfile ||
                      APPEARANCE_FLAT!=opts.appearance)
                         ? blendBorderColors
                               ? midColor(cg.base(), cols[BORDER_RAISED==borderProfile ? 4 : 0])
                               : cols[BORDER_RAISED==borderProfile ? 4 : 0]
                         : cg.background());
            p->drawLine(r.x()+r.width()-2, r.y()+1, r.x()+r.width()-2, r.y()+r.height()-2);
            p->drawLine(r.x()+1, r.y()+r.height()-2, r.x()+r.width()-2, r.y()+r.height()-2);
    }

    if(QTC_ROUNDED && ROUNDED_NONE!=round)
    {
        bool largeArc(ROUND_FULL==opts.round && !(flags&QTC_CHECK_BUTTON) &&
                      r.width()>=QTC_MIN_BTN_SIZE && r.height()>=QTC_MIN_BTN_SIZE);

        p->setPen(border);
        if(itsFormMode)
        {
            // If we're itsFormMode (KHTML) then we need to draw the aa borders using pixmaps
            //  - so we need to draw 2 pixels away from each corner (so that the alpha
            // blend lets through the background color...
            p->drawLine(r.x()+2, r.y(), r.x()+r.width()-3, r.y());
            p->drawLine(r.x()+2, r.y()+r.height()-1, r.x()+r.width()-3, r.y()+r.height()-1);
            p->drawLine(r.x(), r.y()+2, r.x(), r.y()+r.height()-3);
            p->drawLine(r.x()+r.width()-1, r.y()+2, r.x()+r.width()-1, r.y()+r.height()-3);

            // If not rounding a corner need to draw the missing pixels!
            if(!(round&CORNER_TL) || !largeArc)
            {
                p->drawPoint(r.x()+1, r.y());
                p->drawPoint(r.x(), r.y()+1);
            }
            if(!(round&CORNER_TR) || !largeArc)
            {
                p->drawPoint(r.x()+r.width()-2, r.y());
                p->drawPoint(r.x()+r.width()-1, r.y()+1);
            }
            if(!(round&CORNER_BR) || !largeArc)
            {
                p->drawPoint(r.x()+r.width()-2, r.y()+r.height()-1);
                p->drawPoint(r.x()+r.width()-1, r.y()+r.height()-2);
            }
            if(!(round&CORNER_BL) || !largeArc)
            {
                p->drawPoint(r.x()+1, r.y()+r.height()-1);
                p->drawPoint(r.x(), r.y()+r.height()-2);
            }
        }
        else
        {
            // If we're not itsFormMode (ie. not KHTML) then we can just draw 1 pixel in - as
            // we can overwrite with the alpha colour.
            p->drawLine(r.x()+1, r.y(), r.x()+r.width()-2, r.y());
            p->drawLine(r.x()+1, r.y()+r.height()-1, r.x()+r.width()-2, r.y()+r.height()-1);
            p->drawLine(r.x(), r.y()+1, r.x(), r.y()+r.height()-2);
            p->drawLine(r.x()+r.width()-1, r.y()+1, r.x()+r.width()-1, r.y()+r.height()-2);
        }

        QColor  largeArcMid(midColor(border, bgnd)),
                aaColor(midColor(custom ? custom[3] : itsBackgroundCols[3], bgnd));
        QPixmap *pix=itsFormMode ? getPixelPixmap(border) : NULL;

        if(round&CORNER_TL)
        {
            if(largeArc)
            {
                p->drawPoint(r.x()+1, r.y()+1);
                if(itsFormMode)
                {
                    p->drawPixmap(r.x(), r.y()+1, *pix);
                    p->drawPixmap(r.x()+1, r.y(), *pix);
                }
                else
                {
                    p->setPen(largeArcMid);
                    p->drawLine(r.x(), r.y()+1, r.x()+1, r.y());
                }
            }
            if(doCorners)
                if(itsFormMode)
                {
                    if(!largeArc)
                        p->drawPixmap(r.x(), r.y(), *pix);
                }
                else
                {
                    p->setPen(largeArc ? bgnd : aaColor);
                    p->drawPoint(r.x(), r.y());
                }
        }
        else
            p->drawPoint(r.x(), r.y());

        p->setPen(border);
        if(round&CORNER_TR)
        {
            if(largeArc)
            {
                p->drawPoint(r.x()+r.width()-2, r.y()+1);
                if(itsFormMode)
                {
                    p->drawPixmap(r.x()+r.width()-2, r.y(), *pix);
                    p->drawPixmap(r.x()+r.width()-1, r.y()+1, *pix);
                }
                else
                {
                    p->setPen(largeArcMid);
                    p->drawLine(r.x()+r.width()-2, r.y(), r.x()+r.width()-1, r.y()+1);
                }
            }
            if(doCorners)
                if(itsFormMode)
                {
                    if(!largeArc)
                        p->drawPixmap(r.x()+r.width()-1, r.y(), *pix);
                }
                else
                {
                    p->setPen(largeArc ? bgnd : aaColor);
                    p->drawPoint(r.x()+r.width()-1, r.y());
                }
        }
        else
            p->drawPoint(r.x()+r.width()-1, r.y());

        p->setPen(border);
        if(round&CORNER_BR)
        {
            if(largeArc)
            {
                p->drawPoint(r.x()+r.width()-2, r.y()+r.height()-2);
                if(itsFormMode)
                {
                    p->drawPixmap(r.x()+r.width()-2, r.y()+r.height()-1, *pix);
                    p->drawPixmap(r.x()+r.width()-1, r.y()+r.height()-2, *pix);
                }
                else
                {
                    p->setPen(largeArcMid);
                    p->drawLine(r.x()+r.width()-2, r.y()+r.height()-1, r.x()+r.width()-1,
                                r.y()+r.height()-2);
                }
            }
            if(doCorners)
                if(itsFormMode)
                {
                    if(!largeArc)
                        p->drawPixmap(r.x()+r.width()-1, r.y()+r.height()-1, *pix);
                }
                else
                {
                    p->setPen(largeArc ? bgnd : aaColor);
                    p->drawPoint(r.x()+r.width()-1, r.y()+r.height()-1);
                }
        }
        else
            p->drawPoint(r.x()+r.width()-1, r.y()+r.height()-1);

        p->setPen(border);
        if(round&CORNER_BL)
        {
            if(largeArc)
            {
                p->drawPoint(r.x()+1, r.y()+r.height()-2);
                if(itsFormMode)
                {
                    p->drawPixmap(r.x(), r.y()+r.height()-2, *pix);
                    p->drawPixmap(r.x()+1, r.y()+r.height()-1, *pix);
                }
                else
                {
                    p->setPen(largeArcMid);
                    p->drawLine(r.x(), r.y()+r.height()-2, r.x()+1, r.y()+r.height()-1);
                }
            }
            if(doCorners)
                if(itsFormMode)
                {
                    if(!largeArc)
                        p->drawPixmap(r.x(), r.y()+r.height()-1, *pix);
                }
                else
                {
                    p->setPen(largeArc ? bgnd : aaColor);
                    p->drawPoint(r.x(), r.y()+r.height()-1);
                }
        }
        else
            p->drawPoint(r.x(), r.y()+r.height()-1);
    }
    else
    {
        p->setPen(border);
        p->setBrush(NoBrush);
        p->drawRect(r);
    }
}

void QtCurveStyle::drawEntryField(QPainter *p, const QRect &rx, const QColorGroup &cg,
                                  SFlags flags, bool highlight, int round,
                                  bool isSpin) const
{
    bool         doHighlight(highlight && opts.highlightEdits);
    const QColor *use(doHighlight ? itsMenuitemCols : itsBackgroundCols);

    QRect r(rx);

    if(isSpin)
    {
        if(QApplication::reverseLayout())
            r.addCoords(-1, 0, 0, 0);

        p->setPen(flags&Style_Enabled ? cg.base() : cg.background());
        p->drawLine(r.x()+r.width()-2, r.y(), r.x()+r.width()-2, r.y()+r.height()-1);
        p->drawLine(r.x()+r.width()-3, r.y(), r.x()+r.width()-3, r.y()+r.height()-1);
    }

    p->setPen(APPEARANCE_FLAT!=opts.appearance || doHighlight
                ? midColor(flags&Style_Enabled ? cg.base() : cg.background(), use[3])
                : cg.base());
    p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.y()+r.height()-2);
    p->drawLine(r.x()+1, r.y()+1, r.x()+r.width()-(isSpin ? 0 : 2), r.y()+1);

    p->setPen(flags&Style_Enabled ? midColor(cg.base(), use[0]) : cg.background());
    p->drawLine(r.x()+r.width()-(isSpin && doHighlight ? 3 : 2), r.y()+1,
                r.x()+r.width()-(isSpin && doHighlight ? 3 : 2), r.y()+r.height()-2);
    p->drawLine(r.x()+1, r.y()+r.height()-2, r.x()+r.width()-(isSpin ? 0 : 2), r.y()+r.height()-2);

    if(doHighlight && isSpin)
    {
        p->setPen(use[5]);
        p->drawLine(r.x()+r.width()-2, r.y(), r.x()+r.width()-2, r.y()+r.height()-1);
    }

    drawBorder(cg.background(), p, r, cg, (SFlags)(flags|Style_Horizontal), round, use);
}

void QtCurveStyle::drawArrow(QPainter *p, const QRect &r, const QColorGroup &cg, SFlags flags,
                             QStyle::PrimitiveElement pe, bool small, bool checkActive) const
{
    QPointArray  a, b;
    const QColor &col(flags&Style_Enabled
                          ? checkActive && flags&Style_Active
                                ? cg.highlightedText()
                                : cg.text()
                          : cg.mid());

    if(small)
        if(opts.vArrow)
            switch(pe)
            {
                case QStyle::PE_ArrowUp:
                    a.setPoints(7,  2,1,  2,0,  0,-2,  -2,0,  -2,1,  -2,0,  2,0);
                    break;
                case QStyle::PE_ArrowDown:
                    a.setPoints(7,  2,-1,  2,0,  0,2,  -2,0,  -2,-1,  -2,0,  2,0);
                    break;
                case QStyle::PE_ArrowRight:
                    a.setPoints(7,  1,-2,  0,-2,  -2,0,  0,2,  1,2,  0,2,  0,-2);
                    break;
                case QStyle::PE_ArrowLeft:
                    a.setPoints(7,  -1,-2,  0,-2,  2,0,  0,2,  -1,2,  0,2,  0,-2);
                    break;
                default:
                    return;
            }
        else
            switch(pe)
            {
                case QStyle::PE_ArrowUp:
                    a.setPoints(4,  2,0,  0,-2,  -2,0,  2,0);
                    break;
                case QStyle::PE_ArrowDown:
                    a.setPoints(4,  2,0,  0,2,  -2,0,   2,0);
                    break;
                case QStyle::PE_ArrowRight:
                    a.setPoints(4,  0,-2,  -2,0,  0,2,  0,-2);
                    break;
                case QStyle::PE_ArrowLeft:
                    a.setPoints(4,  0,-2,  2,0,  0,2,   0,-2);
                    break;
                default:
                    return;
            }
    else // Large arrows...
        switch(pe)
        {
            case QStyle::PE_ArrowUp:
                a.setPoints(4,  3,1,  0,-2,  -3,1,  3,1);
                b.setPoints(2, 2,2,  -2,2);
                break;
            case QStyle::PE_ArrowDown:
                a.setPoints(4,  3,-1,  0,2,  -3,-1,  3,-1);
                b.setPoints(2, 2,-2,  -2,-2);
                break;
            case QStyle::PE_ArrowRight:
                a.setPoints(4,  -1,-3,  2,0,  -1,3,  -1,-3);
                b.setPoints(2, -2,-2,  -2,2);
                break;
            case QStyle::PE_ArrowLeft:
                a.setPoints(4,  1,-3,  -2,0,  1,3,  1,-3);
                b.setPoints(2, 2,2,  2,-2);
                break;
            default:
                return;
        }

    if(a.isNull())
        return;

    p->save();
    a.translate((r.x()+(r.width()>>1)),(r.y()+(r.height()>>1)));
    p->setBrush(col);
    p->setPen(col);
    p->drawPolygon(a);
    if(opts.vArrow && !b.isNull())
    {
        b.translate((r.x()+(r.width()>>1)),(r.y()+(r.height()>>1)));
        p->drawPoints(b);
    }

    p->restore();
}

void QtCurveStyle::drawPrimitive(PrimitiveElement pe, QPainter *p, const QRect &r,
                                 const QColorGroup &cg, SFlags flags, const QStyleOption &data) const
{
    switch(pe)
    {
        case PE_HeaderSection:
        {
            // Is it a taskbar button? Kicker uses PE_HeaderSection for these! :-(
            // If the painter device is a QWidger, assume its not a taskbar button...
            if(APP_KICKER==itsThemedApp && (!p->device() || !dynamic_cast<QWidget *>(p->device())))
            {
                const QColor *use(buttonColors(cg));

                if(flags&Style_Down)
                    flags=((flags|Style_Down)^Style_Down)| Style_Sunken;
                flags|=Style_Enabled;
#if KDE_VERSION >= 0x30200
#if KDE_VERSION >= 0x30400 && KDE_VERSION < 0x30500
                if(HOVER_KICKER==itsHover && itsHoverWidget) //  && itsHoverWidget==p->device())
                    flags|=Style_MouseOver;
#endif
                itsFormMode=itsIsTransKicker;
#endif
                drawLightBevel(p, r, cg, flags|Style_Horizontal, ROUNDED_ALL,
                               getFill(flags, use), use, true, false);
#if KDE_VERSION >= 0x30200
                itsFormMode=false;
#endif
            }
            else
            {
                bool    isFirst(false), isLast(false), isTable(false);
                QHeader *header(p && p->device() ? dynamic_cast<QHeader*>(p->device())
                                                 : NULL);

                if (header)
                {
                    if(header->parent() && ::qt_cast<const QTable *>(header->parent()))
                    {
                        QTable *tbl((QTable *)(header->parent()));

                        isTable=true;
                        if(flags&Style_Horizontal)
                           isFirst=tbl->columnAt(r.x()+header->offset())==0;
                        else
                           isLast=tbl->rowAt(r.y()+header->offset())==(tbl->numRows()-1);
                    }
                    else
                        isFirst = header->mapToIndex(header->sectionAt(r.x()+header->offset())) == 0;
                }
                else if(0==flags) // Header on popup menu?
                {
                    const QColor *use(buttonColors(cg));

                    drawLightBevel(p, r, cg, flags|Style_Horizontal, ROUNDED_ALL,
                                   getFill(flags, use), use);
                    break;
                }

                flags=((flags|Style_Sunken)^Style_Sunken)| Style_Raised;

                if(QTC_NO_SECT!=itsHoverSect && HOVER_HEADER==itsHover && itsHoverWidget)
                {
                    QHeader *hd(::qt_cast<QHeader *>(itsHoverWidget));

                    if(hd && hd->isClickEnabled(itsHoverSect) && r==hd->sectionRect(itsHoverSect))
                        flags|=Style_MouseOver;
                }

                bool sunken(flags &(QStyle::Style_Down | QStyle::Style_On | QStyle::Style_Sunken));

                drawBevelGradient(getFill(flags, itsListviewCols), !sunken, p, r, flags&Style_Horizontal,
                                  sunken ? SHADE_BEVEL_GRAD_SEL_LIGHT : SHADE_BEVEL_GRAD_LIGHT,
                                  sunken ? SHADE_BEVEL_GRAD_SEL_DARK : SHADE_BEVEL_GRAD_DARK,
                                  sunken, opts.lvHeaderAppearance, WIDGET_LISTVIEW_HEADER);

                const QColor *border(borderColors(flags, NULL));

                if(flags&Style_Horizontal)
                {
                    if(border)
                    {
                        p->setPen(border[1]);
                        p->drawLine(r.x(), r.y()+r.height()-2, r.x()+r.width()-1,
                                    r.y()+r.height()-2);
                        p->setPen(border[3]);
                    }
                    else
                    p->setPen(itsListviewCols[5]);
                    p->drawLine(r.x(), r.y()+r.height()-1, r.x()+r.width()-1, r.y()+r.height()-1);

                    if(!isFirst)
                    {
                        p->setPen(itsListviewCols[5]);
                        p->drawLine(r.x(), r.y()+5, r.x(), r.y()+r.height()-6);
                        p->setPen(itsListviewCols[0]);
                        p->drawLine(r.x()+1, r.y()+5, r.x()+1, r.y()+r.height()-6);
                    }
                }
                else
                {
                    if(border)
                    {
                        p->setPen(border[1]);
                        p->drawLine(r.x()+r.width()-2, r.y(), r.x()+r.width()-2, r.y()+r.height()-1);
                        p->setPen(border[3]);
                    }
                    else
                        p->setPen(itsListviewCols[5]);
                        p->drawLine(r.x()+r.width()-1, r.y(), r.x()+r.width()-1, r.y()+r.height()-1);

                    if(!isLast)
                    {
                        p->setPen(itsListviewCols[5]);
                        p->drawLine(r.x()+5, r.y()+r.height()-2, r.x()+r.width()-6,
                                    r.y()+r.height()-2);
                        p->setPen(itsListviewCols[0]);
                        p->drawLine(r.x()+5, r.y()+r.height()-1, r.x()+r.width()-6,
                                    r.y()+r.height()-1);
                    }
                }
            }
            break;
        }
        case PE_HeaderArrow:
            drawArrow(p, r, cg, flags, flags&Style_Up ? PE_ArrowUp : PE_ArrowDown);
            break;
        case PE_ButtonCommand:
        case PE_ButtonBevel:
        case PE_ButtonTool:
        case PE_ButtonDropDown:
        {
            const QColor *use(buttonColors(cg));
            bool         glassMod(PE_ButtonTool==pe && IS_GLASS(opts.appearance) &&
                                  IS_GLASS(opts.toolbarAppearance));

            // If its not sunken, its raised-don't want flat buttons.
            if(!(flags&QStyle::Style_Sunken))
                flags|=QStyle::Style_Raised;

            if(PE_ButtonTool==pe)
            {
                if(flags&QTC_VERTICAL_TB_BUTTON)
                {
                    flags-=QTC_VERTICAL_TB_BUTTON;
                    if(flags&Style_Horizontal)
                        flags-=Style_Horizontal;
                }
            }

            // Dont AA' MDI windows' control buttons...
            itsFormMode=itsFormMode || (!(flags&QTC_CHECK_BUTTON) && !(flags&QTC_STD_TOOLBUTTON) &&
                                 PE_ButtonTool==pe && r.width()<16 && r.height()<16);
            drawLightBevel(p, r, cg, glassMod ? flags : flags|Style_Horizontal,
#if KDE_VERSION >= 0x30200
                           APP_KORN==itsThemedApp && itsIsTransKicker && PE_ButtonTool==pe
                               ? ROUNDED_NONE :
#endif
                           ROUNDED_ALL, getFill(flags, use), use, true, true, WIDGET_STD_BUTTON);

            if(IND_COLORED==opts.defBtnIndicator && (flags&Style_ButtonDefault))
            {
                QRegion outer(r);
                QRect   r2(r);

                r2.addCoords(COLORED_BORDER_SIZE, COLORED_BORDER_SIZE, -COLORED_BORDER_SIZE,
                             -COLORED_BORDER_SIZE);

                QRegion inner(r2);

                p->setClipRegion(outer.eor(inner));

                drawLightBevel(p, r, cg, glassMod ? flags : flags|Style_Horizontal,
                               flags&QTC_CHECK_BUTTON
#if KDE_VERSION >= 0x30200
                                 || (APP_KORN==itsThemedApp && itsIsTransKicker && PE_ButtonTool==pe)
#endif
                                    ? ROUNDED_NONE : ROUNDED_ALL,
                               getFill(flags, itsDefBtnCols), itsDefBtnCols, true, true,
                               WIDGET_DEF_BUTTON);
                p->setClipping(false);
            }
            itsFormMode=false;
            break;
        }

        case PE_ButtonDefault:
            switch(opts.defBtnIndicator)
            {
                case IND_CORNER:
                {
                    const QColor *use(buttonColors(cg));
                    QPointArray  points;
                    bool         sunken(flags&Style_Down || flags&QStyle::Style_Sunken);
                    int          offset(sunken ? 4 : 3);

                    points.setPoints(3, r.x()+offset, r.y()+offset, r.x()+offset+6, r.y()+offset,
                                        r.x()+offset, r.y()+offset+6);

                    p->setBrush(use[sunken ? 0 : 4]);
                    p->setPen(use[sunken ? 0 : 4]);
                    p->drawPolygon(points);
                    break;
                }
                default:
                    break;
            }
            break;
        case PE_CheckMark:
            if(flags&Style_On)
            {
                QPixmap *pix(getPixmap(flags&Style_Enabled
                                           ? flags&Style_Selected
                                               ? cg.highlightedText()
                                               : itsCheckRadioCol
                                           : cg.mid(),
                                       PIX_CHECK, 1.0));

                p->drawPixmap(r.center().x()-(pix->width()/2), r.center().y()-(pix->height()/2),
                              *pix);
            }
            else if (!(flags&Style_Off))    // tri-state
            {
                int x(r.center().x()), y(r.center().y());

                p->setPen(flags&Style_Enabled
                              ? flags&Style_Selected
                                  ? cg.highlightedText()
                                  : cg.text()
                              : cg.mid());
                p->drawLine(x-3, y, x+3, y);
                p->drawLine(x-3, y+1, x+3, y+1);
            }
            break;
        case PE_CheckListController:
        {
            QCheckListItem *item = data.checkListItem();

            if(item)
            {
                QListView *lv(item->listView());
                int       x(r.x()), y(r.y()), w(r.width()), h(r.height()), marg(lv->itemMargin());

                p->setPen(QPen(flags&Style_Enabled ? cg.text()
                                                   : lv->palette().color(QPalette::Disabled,
                                                                         QColorGroup::Text)));

                if (flags&Style_Selected && !lv->rootIsDecorated() &&
                    !((item->parent() && 1==item->parent()->rtti() &&
                     QCheckListItem::Controller==((QCheckListItem*)item->parent())->type())))
                {
                    p->fillRect(0, 0, x+marg+w+4, item->height(), cg.brush(QColorGroup::Highlight));
                    if (item->isEnabled())
                        p->setPen(QPen(cg.highlightedText()));
                }

                p->drawRect(x+marg+2, y+4+2, w-7, h-8);
                p->drawRect(x+marg, y+4, w-7, h-8);
            }
            break;
        }
        case PE_CheckListIndicator:
        {
            QCheckListItem *item(data.checkListItem());

            if(item)
            {
                QListView *lv(item->listView());

                p->setPen(QPen(flags&Style_Enabled ? cg.text()
                                                   : lv->palette().color(QPalette::Disabled,
                                                                         QColorGroup::Text)));
                if (flags&Style_Selected)
                {
                    flags-=Style_Selected;
                    if(!lv->rootIsDecorated() && !((item->parent() && 1==item->parent()->rtti() &&
                       QCheckListItem::Controller==((QCheckListItem*)item->parent())->type())))
                    {
                        p->fillRect(0, 0, r.x()+lv->itemMargin()+r.width()+4, item->height(),
                                    cg.brush(QColorGroup::Highlight));
                        if(item->isEnabled())
                        {
                            p->setPen(QPen(cg.highlightedText()));
                            flags+=Style_Selected;
                        }
                    }
                }

                QRect checkRect(r.x()+1, r.y()+1, QTC_CHECK_SIZE, QTC_CHECK_SIZE);
                p->drawRect(checkRect);
                if(flags&QStyle::Style_On || !(flags&Style_Off))
                    drawPrimitive(PE_CheckMark, p, checkRect, cg, flags);
            }
            break;
        }
        case PE_IndicatorMask:
            if(QTC_ROUNDED)
            {
                p->fillRect(r, color0);
                p->fillRect(r.x()+1, r.y(), r.width()-2, r.height(), color1);
                p->setPen(color1);
                p->drawLine(r.x(), r.y()+1, r.x(), r.y()+r.height()-2);
                p->drawLine(r.x()+r.width()-1, r.y()+1, r.x()+r.width()-1, r.y()+r.height()-2);
            }
            else
                p->fillRect(r, color1);
            break;
        case PE_Indicator:
        {
            bool   on(flags&QStyle::Style_On || !(flags&Style_Off));
            SFlags sflags(!(flags&Style_Off) ? flags|Style_On : flags);

            if(!itsFormMode && sflags&Style_MouseOver && HOVER_NONE==itsHover)
                sflags-=Style_MouseOver;

            drawPrimitive(PE_ButtonTool, p, r, cg, sflags|Style_Horizontal|QTC_CHECK_BUTTON);

            if(on)
                drawPrimitive(PE_CheckMark, p, r, cg, flags);
            break;
        }
        case PE_CheckListExclusiveIndicator:
        {
            QCheckListItem *item(data.checkListItem());

            if(item)
            {
                const QColor &on(flags&Style_Enabled ? itsCheckRadioCol : cg.mid());
                int          x(r.x()), y(r.y()+2);

                p->drawPixmap(x, y, *getPixmap(on, PIX_RADIO_BORDER, 0.8));

                if(flags&QStyle::Style_On)
                    p->drawPixmap(x, y, *getPixmap(on, PIX_RADIO_ON, 1.0));
            }
            break;
        }
        case PE_ExclusiveIndicator:
        case PE_ExclusiveIndicatorMask:
        {
            int         x(r.x()), y(r.y());
            QPointArray clipRegion;

            clipRegion.setPoints(28, x,    y+8,  x,    y+4,  x+1,  y+4,  x+1,  y+3,
                                     x+2,  y+2,  x+3,  y+1,  x+4,  y+1,  x+4,  y,
                                     x+8,  y,    x+8,  y+1,  x+9,  y+1,  x+10, y+2,
                                     x+11, y+3,  x+11, y+4,  x+12, y+4,  x+12, y+8,
                                     x+11, y+8,  x+11, y+9,  x+10, y+10, x+9,  y+11,
                                     x+8,  y+11, x+8,  y+12, x+4,  y+12, x+4,  y+11,
                                     x+3,  y+11, x+2,  y+10, x+1,  y+9,  x+1,  y+8);
            if(PE_ExclusiveIndicatorMask==pe)
            {
                p->fillRect(r, color0);
                p->setPen(Qt::color1);
                p->setBrush(Qt::color1);
                p->drawPolygon(clipRegion);
            }
            else
            {
                SFlags sflags(flags);

                if(!itsFormMode && sflags&Style_MouseOver && HOVER_NONE==itsHover)
                    sflags-=Style_MouseOver;

                const QColor *bc(borderColors(sflags, NULL)),
                             *btn(buttonColors(cg)),
                             *use(bc ? bc : btn);
                const QColor &on(sflags&Style_Enabled
                                     ? sflags&Style_Selected
                                         ? cg.highlightedText()
                                         : itsCheckRadioCol
                                     : cg.mid()),
                             &indBgnd(getFill(sflags, btn));
                bool         set(sflags&QStyle::Style_On);

                p->fillRect(r, opts.crLabelHighlight && sflags&Style_MouseOver
                               ? shade(cg.background(), opts.highlightFactor) : cg.background());

                if(!itsFormMode)
                    p->setClipRegion(QRegion(clipRegion));
                drawBevelGradient(indBgnd, !set, p, QRect(x+1, y+1, r.width()-2, r.height()-2),
                                  true, set ? SHADE_BEVEL_GRAD_SEL_LIGHT : SHADE_BEVEL_GRAD_LIGHT,
                                  set ? SHADE_BEVEL_GRAD_SEL_DARK : SHADE_BEVEL_GRAD_DARK,
                                  set, opts.appearance);

                if(!itsFormMode)
                    p->setClipping(false);

                p->drawPixmap(r.x(), r.y(),
                              *getPixmap(use[opts.coloredMouseOver && sflags&Style_MouseOver ?4:5],
                                         PIX_RADIO_BORDER, 0.8));

                if(set)
                    p->drawPixmap(r.x(), r.y(), *getPixmap(on, PIX_RADIO_ON, 1.0));
                else if(!itsFormMode)
                    p->drawPixmap(r.x(), r.y(), *getPixmap(btn[0], PIX_RADIO_LIGHT));
            }
            break;
        }
        case PE_DockWindowSeparator:
        {
            QRect r2(r);
            int   light=0,
                  dark=3;

            r2.addCoords(-1, -1, 2, 2);
            drawMenuOrToolBarBackground(p, r2, cg, false, flags&Style_Horizontal);

            switch(opts.toolbarSeparators)
            {
                case LINE_NONE:
                    break;
                case LINE_RAISED:
                    light=dark;
                    dark=0;
                case LINE_SUNKEN:
                    if(r.width()<r.height())
                    {
                        int x(r.x()+((r.width()-2) / 2));

                        p->setPen(itsBackgroundCols[dark]);
                        p->drawLine(x, r.y()+6, x, r.y()+r.height()-7);
                        p->setPen(itsBackgroundCols[light]);
                        p->drawLine(x+1, r.y()+6, x+1, r.y()+r.height()-7);
                    }
                    else
                    {
                        int y(r.y()+((r.height()-2) / 2));

                        p->setPen(itsBackgroundCols[3]);
                        p->drawLine(r.x()+6, y, r.x()+r.width()-7, y);
                        p->setPen(itsBackgroundCols[0]);
                        p->drawLine(r.x()+6, y+1, r.x()+r.width()-7, y+1);
                    }
                    break;
                case LINE_DOTS:
                case LINE_LARGE_DOTS:
                    drawDots(p, r, !(flags & Style_Horizontal), 1, 5, itsBackgroundCols, 0, 5,
                             LINE_LARGE_DOTS==opts.toolbarSeparators);
            }
            break;
        }
        case PE_Splitter:
        {
            if(itsHoverWidget && itsHoverWidget == p->device())
                flags|=Style_MouseOver;

            const QColor *use(buttonColors(cg));
            const QColor *border(borderColors(flags, use));

            p->fillRect(r, QColor(flags&Style_MouseOver
                                      ? shade(cg.background(), opts.highlightFactor)
                                      : cg.background()));
            switch(opts.splitters)
            {
                case LINE_LARGE_DOTS:
                    drawDots(p, r, flags&Style_Horizontal, NUM_SPLITTER_DASHES*2, 2, border, 0, 5,
                             true);
                    break;
                default:
                case LINE_DOTS:
                    drawDots(p, r, flags&Style_Horizontal, NUM_SPLITTER_DASHES, 1, border, 0, 5,
                             false);
                    break;
                case LINE_RAISED:
                    drawLines(p, r, flags&Style_Horizontal, NUM_SPLITTER_DASHES, 1, border, 0,
                              false, 3);
                    break;
                case LINE_SUNKEN:
                    drawLines(p, r, flags&Style_Horizontal, NUM_SPLITTER_DASHES, 1, border, 0,
                              true, 3);
                    break;
                case LINE_DASHES:
                    drawLines(p, r, flags&Style_Horizontal, NUM_SPLITTER_DASHES, 1, border, 0,
                              true, 3, 0);
            }
            break;
        }
        case PE_DockWindowResizeHandle:
        {
            if(QTC_ROUNDED)
            {
                const QColor *use(backgroundColors(cg));

                drawBorder(cg.background(), p, r, cg, (SFlags)(flags|Style_Horizontal),
                           ROUNDED_ALL, use, true, BORDER_RAISED, false);
            }
            else
                QCommonStyle::drawPrimitive(pe, p, r, cg, flags, data);
            break;
        }
        case PE_GroupBoxFrame:
        case PE_PanelGroupBox:
            if((APP_OPENOFFICE==itsThemedApp || data.lineWidth()>0 || data.isDefault()) &&
                QTC_ROUNDED)
            {
                const QColor *use(backgroundColors(cg));

                drawBorder(cg.background(), p, r, cg, (SFlags)(flags|Style_Horizontal),
                           ROUNDED_ALL, use, true, BORDER_FLAT);
            }
            else
                QCommonStyle::drawPrimitive(pe, p, r, cg, flags, data);
            break;
        case PE_Panel:
        case PE_WindowFrame:
            if((APP_OPENOFFICE==itsThemedApp || data.lineWidth()>0 || data.isDefault()) &&
               QTC_ROUNDED)
            {
                const QColor *use(backgroundColors(cg));

                itsFormMode=itsIsTransKicker;
                drawBorder(cg.background(), p, r, cg,
                           (SFlags)(flags|Style_Horizontal|Style_Enabled),
                           ROUNDED_ALL, use, APP_KICKER!=itsThemedApp, itsIsTransKicker
                                                                           ? BORDER_FLAT
                                                                           : flags&Style_Sunken
                                                                               ? BORDER_SUNKEN
                                                                               : BORDER_RAISED);
                itsFormMode=false;
            }
            else
                QCommonStyle::drawPrimitive(pe, p, r, cg, flags, data);
            break;
        case PE_PanelPopup:
        case PE_TabBarBase:
        case PE_PanelTabWidget:
        {
            const QColor *use(backgroundColors(cg));
            bool  flat(APPEARANCE_FLAT==opts.appearance);

            if(PE_PanelTabWidget==pe && QTC_ROUNDED)
            {
                const QColor *use(backgroundColors(cg));

                drawBorder(cg.background(), p, r, cg,
                           (SFlags)(flags|Style_Horizontal|Style_Enabled),
                           ROUNDED_ALL, use, true, BORDER_RAISED, false);
            }
            else if(PE_PanelPopup==pe && opts.lighterPopupMenuBgnd)
            {
                p->setPen(use[5]);
                p->setBrush(NoBrush);
                p->drawRect(r);
                p->setPen(itsLighterPopupMenuBgndCol);
                p->drawRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2);
            }
            else if(data.isDefault()|| data.lineWidth()>1)
            {
                p->setPen(use[5]);
                p->setBrush(NoBrush);
                p->drawRect(r);
                qDrawShadePanel(p, r.x()+1, r.y()+1, r.width()-2, r.height()-2,
                                QColorGroup(use[flat ? ORIGINAL_SHADE : 4], use[ORIGINAL_SHADE],
                                use[0], use[flat ? ORIGINAL_SHADE : 4], use[2],
                                cg.text(),
                                use[ORIGINAL_SHADE]), flags & Style_Sunken, data.isDefault()
                                     ? QTC_DEF_FRAME_WIDTH-1 : data.lineWidth()-1);
            }
            else
                qDrawShadePanel(p, r, QColorGroup(use[flat ? ORIGINAL_SHADE : 5],
                                use[ORIGINAL_SHADE], use[0], use[flat ? ORIGINAL_SHADE : 5], use[2],
                                cg.text(), use[ORIGINAL_SHADE]), flags & Style_Sunken,
                                data.isDefault() ? QTC_DEF_FRAME_WIDTH
                                                 : data.lineWidth());
            break;
        }
        case PE_PanelDockWindow:
        case PE_PanelMenuBar:
        {
            // fix for toolbar lag (from Mosfet Liquid) 
            QWidget *w = dynamic_cast<QWidget *>(p->device());

            if(w && PaletteButton==w->backgroundMode())
                w->setBackgroundMode(PaletteBackground);

            if(TB_NONE!=opts.toolbarBorders)
            {
                const QColor *use=PE_PanelMenuBar==pe
                                      ? itsMenubarCols
                                      : backgroundColors(cg.background());

                qDrawShadePanel(p, r, QColorGroup(use[TB_DARK==opts.toolbarBorders ? 5 : 3],
                                                  use[ORIGINAL_SHADE], use[0],
                                                  use[TB_DARK==opts.toolbarBorders ? 5 : 3], use[2],
                                                  cg.text(), use[ORIGINAL_SHADE]),
                                flags & Style_Sunken, 1);
            }
            else
                drawMenuOrToolBarBackground(p, r, cg, PE_PanelMenuBar==pe,
                                            PE_PanelMenuBar==pe || r.width()>r.height());
            break;
        }
        case PE_ScrollBarAddLine:
        case PE_ScrollBarSubLine:
        {
            QRect        br(r);
            bool         down((flags &(QStyle::Style_Down|QStyle::Style_On|QStyle::Style_Sunken)));
            const QColor *use(buttonColors(cg));

            pe=flags&Style_Horizontal
                   ? PE_ScrollBarAddLine==pe ? PE_ArrowRight : PE_ArrowLeft
                   : PE_ScrollBarAddLine==pe ? PE_ArrowDown : PE_ArrowUp;

            int round=PE_ArrowRight==pe ? ROUNDED_RIGHT :
                      PE_ArrowLeft==pe ? ROUNDED_LEFT :
                      PE_ArrowDown==pe ? ROUNDED_BOTTOM :
                      PE_ArrowUp==pe ? ROUNDED_TOP : ROUNDED_NONE;

            switch(opts.scrollbarType)
            {
                default:
                case SCROLLBAR_WINDOWS:
                    break;
                case SCROLLBAR_KDE:
                case SCROLLBAR_PLATINUM:
                    if(PE_ArrowLeft==pe && r.x()>3)
                    {
                        round=ROUNDED_NONE;
                        br.addCoords(0, 0, 1, 0);
                    }
                    else if(PE_ArrowUp==pe && r.y()>3)
                    {
                        round=ROUNDED_NONE;
                        br.addCoords(0, 0, 0, 1);
                    }
                    break;
                case SCROLLBAR_NEXT:
                    if(PE_ArrowRight==pe)
                    {
                        round=ROUNDED_NONE;
                        br.addCoords(-1, 0, 0, 0);
                    }
                    else if(PE_ArrowDown==pe)
                    {
                        round=ROUNDED_NONE;
                        br.addCoords(0, -1, 0, 0);
                    }
                    break;
            }
            drawLightBevel(p, br, cg, down ? flags
                                           : flags|(((flags & Style_Enabled)
                                                 ? Style_Raised
                                                 : Style_Default)),
                           round, getFill(flags, use), use);

            QRect ar(r);

            if(flags&Style_Down)
                ar.addCoords(1, 1, 1, 1);

            drawPrimitive(pe, p, ar, cg, flags);
            break;
        }
        case PE_ScrollBarSlider:
            drawSliderHandle(p, r, cg, flags, true);
            break;
        case PE_FocusRect:
            if(FOCUS_COLORED==opts.focus)
            {
                QPen old=p->pen();
                p->setPen(itsFocusCol);
                p->setBrush(NoBrush);
                p->drawRect(r);
                p->setPen(old);
            }
            else
            {
                p->setPen(Qt::black);
                p->drawWinFocusRect(r);
            }
            break;
        case PE_ArrowUp:
        case PE_ArrowDown:
        case PE_ArrowRight:
        case PE_ArrowLeft:
            drawArrow(p, r, cg, flags, pe);
            break;
        case PE_SpinWidgetUp:
        case PE_SpinWidgetDown:
        {
            QRect        sr(r);
            const QColor *use(buttonColors(cg));
            bool         reverse(QApplication::reverseLayout());

            drawLightBevel(p, sr, cg, flags|Style_Horizontal, PE_SpinWidgetDown==pe
                                                                ? reverse
                                                                    ? ROUNDED_BOTTOMLEFT
                                                                    : ROUNDED_BOTTOMRIGHT
                                                                : reverse
                                                                    ? ROUNDED_TOPLEFT
                                                                    : ROUNDED_TOPRIGHT,
                           getFill(flags, use), use);

            if(opts.vArrow)
            {
                if(PE_SpinWidgetDown==pe)
                    sr.setY(sr.y()-1);
            }
            else
                sr.setY(sr.y()+(PE_SpinWidgetDown==pe ? -2 : 1));

            if(flags&Style_Sunken)
                sr.addCoords(1, 1, 1, 1);

            drawArrow(p, sr, cg, flags, PE_SpinWidgetUp==pe ? PE_ArrowUp : PE_ArrowDown, true);
            break;
        }
        case PE_SpinWidgetPlus:
        case PE_SpinWidgetMinus:
        {
            const QColor *use(buttonColors(cg));
            int          l(QMIN(r.width()-6, r.height()-6));
            QPoint       c(r.x()+(r.width()/2), r.y()+(r.height()/2));
            bool         reverse(QApplication::reverseLayout());

            l/=2;
            if(l%2 != 0)
                --l;

            drawLightBevel(p, r, cg, flags|Style_Horizontal,
                           PE_SpinWidgetMinus==pe
                                ? reverse
                                    ? ROUNDED_BOTTOMLEFT
                                    : ROUNDED_BOTTOMRIGHT
                                : reverse
                                    ? ROUNDED_TOPLEFT
                                    : ROUNDED_TOPRIGHT,
                           getFill(flags, use), use);

            p->setPen(cg.buttonText());

            if(flags&Style_Sunken)
                c+=QPoint(1, 1);

            p->drawLine(c.x()-l, c.y(), c.x()+l, c.y());
            if(PE_SpinWidgetPlus==pe)
                p->drawLine(c.x(), c.y()-l, c.x(), c.y()+l);
            break;
        }
        case PE_PanelLineEdit:
            {
                bool isReadOnly(false),
                     isEnabled(true);
                // panel is highlighted by default if it has focus, but if we have access to the
                // widget itself we can try to avoid highlighting in case it's readOnly or disabled.
                if (p->device() && dynamic_cast<QLineEdit*>(p->device()))
                {
                    QLineEdit *lineEdit(dynamic_cast<QLineEdit*>(p->device()));
                    isReadOnly = lineEdit->isReadOnly();
                    isEnabled = lineEdit->isEnabled();
                }

                // HACK!!  (From Plastik)
                //
                // In this place there is no reliable way to detect if we are in khtml; the
                // only thing we know is that khtml buffers its widgets into a pixmap. So
                // when the paint device is a QPixmap, chances are high that we are in khtml.
                // It's possible that this breaks other things, so let's see how it works...
                if (p->device() && dynamic_cast<QPixmap*>(p->device() ) )
                    itsFormMode=true;

                drawEntryField(p, r, cg, flags, !isReadOnly && isEnabled && (flags&Style_HasFocus),
                               ROUNDED_ALL);
                itsFormMode=false;
                break;
            }
        case PE_StatusBarSection:
            if(opts.drawStatusBarFrames)
                KStyle::drawPrimitive(pe, p, r, cg, flags, data);
            break;
        default:
            KStyle::drawPrimitive(pe, p, r, cg, flags, data);
    }
}

void QtCurveStyle::drawKStylePrimitive(KStylePrimitive kpe, QPainter *p, const QWidget *widget,
                                       const QRect &r, const QColorGroup &cg, SFlags flags,
                                       const QStyleOption &opt) const
{
    ELine handles(kpe!=KPE_ToolBarHandle && LINE_DASHES==opts.handles ? LINE_SUNKEN
                                                                      : opts.handles);

    switch(kpe)
    {
        case KPE_ToolBarHandle:
        {
            QRect r2(r);
            r2.addCoords(-1, -1, 2, 2);
            drawMenuOrToolBarBackground(p, r2, cg, false, flags&Style_Horizontal);
            drawHandleMarkers(p, r, flags, true, handles);
            break;
        }
        case KPE_DockWindowHandle:
                    {
            QRect left(r);

            if(flags&Style_Horizontal)
                left.addCoords(0, 2, 0, -2);
                    else
                    {
                left.addCoords(2, 0, -2, 0);

                if(opts.drawDockWindowTitles)
                {
                    int textSize(0);

                    if(widget && widget->parentWidget())
                    {
                        QString title(widget->parentWidget()->caption());

                        if(!title.isEmpty())
                        {
                           QFont font(QApplication::font(widget));

                            font.setPixelSize(QMIN(10, r.height()));
                            textSize=QFontMetrics(font).width(title)+4;
                            p->setPen(cg.text());
                            p->setFont(font);
                            p->drawText(r, AlignCenter, title);
                        }
                    }

                    if(textSize)
                    {
                        QRect right(left);
                        int   sides((r.width()-textSize)/2);

                        left.addCoords(0, 0, -(sides+textSize), 0);
                        right.addCoords(sides+textSize, 0, 0, 0);

                        drawHandleMarkers(p, right, flags, false, handles);
                    }
                }
            }
            drawHandleMarkers(p, left, flags, false, handles);
            break;
        }
        case KPE_GeneralHandle:
            drawHandleMarkers(p, r, flags, false, handles);
            break;
        case KPE_SliderGroove:
            drawSliderGroove(p, r, cg, flags, widget);
            break;
        case KPE_SliderHandle:
            drawSliderHandle(p, r, cg, flags);
            break;
        case KPE_ListViewExpander:
        {
            int   lvSize(QTC_LV_SIZE(opts.lvExpander));
            QRect ar(r.x()+((r.width()-(lvSize+4))>>1), r.y()+((r.height()-(lvSize+4))>>1), lvSize+4,
                     lvSize+4);

            p->setPen(opts.lvDark ? cg.text() : cg.mid());

            if(LV_LINES_NONE!=opts.lvLines)
            {
                int lo(QTC_ROUNDED ? 2 : 0);

                p->drawLine(ar.x()+lo, ar.y(), (ar.x()+ar.width()-1)-lo, ar.y());
                p->drawLine(ar.x()+lo, ar.y()+ar.height()-1, (ar.x()+ar.width()-1)-lo,
                            ar.y()+ar.height()-1);
                p->drawLine(ar.x(), ar.y()+lo, ar.x(), (ar.y()+ar.height()-1)-lo);
                p->drawLine(ar.x()+ar.width()-1, ar.y()+lo, ar.x()+ar.width()-1,
                            (ar.y()+ar.height()-1)-lo);

                if(QTC_ROUNDED)
                {
                    p->drawPoint(ar.x()+1, ar.y()+1);
                    p->drawPoint(ar.x()+1, ar.y()+ar.height()-2);
                    p->drawPoint(ar.x()+ar.width()-2, ar.y()+1);
                    p->drawPoint(ar.x()+ar.width()-2, ar.y()+ar.height()-2);
                    p->setPen(midColor(/*opts.lvDark ? cg.text() : */cg.mid(), cg.background()));
                    p->drawLine(ar.x(), ar.y()+1, ar.x()+1, ar.y());
                    p->drawLine(ar.x()+ar.width()-2, ar.y(), ar.x()+ar.width()-1, ar.y()+1);
                    p->drawLine(ar.x(), ar.y()+ar.height()-2, ar.x()+1, ar.y()+ar.height()-1);
                    p->drawLine(ar.x()+ar.width()-2, ar.y()+ar.height()-1, ar.x()+ar.width()-1,
                                ar.y()+ar.height()-2);
                }
            }

            if(LV_EXP_ARR==opts.lvExpander)
                drawArrow(p, ar, cg, flags|Style_Enabled, flags&Style_On // Collapsed = On
                                                ?  QApplication::reverseLayout()
                                                     ? PE_ArrowLeft
                                                     : PE_ArrowRight
                                                : PE_ArrowDown);
            else
            {
                int xo((ar.width()-lvSize)>>1),
                    yo((ar.height()-lvSize)>>1);
                int mid(lvSize>>1);

                p->setPen(cg.text());
                p->drawLine(ar.x()+xo+(mid-2), ar.y()+yo+mid, ar.x()+xo+lvSize-(mid-1),
                            ar.y()+yo+mid);
                if (flags & Style_On) // Collapsed = On
                    p->drawLine(ar.x()+xo+mid, ar.y()+yo+(mid-2), ar.x()+xo+mid,
                                ar.y()+yo+lvSize-(mid-1));
            }
            break;
        }
        case KPE_ListViewBranch:
            switch(opts.lvLines)
            {
                case LV_LINES_NONE:
                    break;
                case LV_LINES_DOTTED:
                {
                    QColorGroup cg2(cg);

                    cg2.setColor(QColorGroup::Text, opts.lvDark ? cg.text() : cg.mid());

                    KStyle::drawKStylePrimitive(kpe, p, widget, r, cg2, flags, opt);
                    break;
                }
                case LV_LINES_SOLID:
                    p->setPen(opts.lvDark ? cg.text() : cg.mid());
                    if (flags&Style_Horizontal)
                    {
                        if(r.width()>0)
                            p->drawLine(r.x(), r.y(), r.x()+r.width()-1, r.y());
                    }
                    else
                        if(r.height()>0)
                            p->drawLine(r.x(), r.y(), r.x(), r.y()+r.height()-1);
            }
            break;
        default:
            KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);
    }
}

void QtCurveStyle::drawControl(ControlElement control, QPainter *p, const QWidget *widget,
                               const QRect &r, const QColorGroup &cg, SFlags flags,
                               const QStyleOption &data) const
{
    if(widget==itsHoverWidget)
        flags|=Style_MouseOver;

    switch(control)
    {
        case CE_TabBarTab:
        {
            const QTabBar *tb((const QTabBar *)widget);
            int           tabIndex(tb->indexOf(data.tab()->identifier())),
                          dark(APPEARANCE_FLAT==opts.appearance ? ORIGINAL_SHADE : 4);
            bool          cornerWidget(false),
                          reverse(QApplication::reverseLayout()),
                          firstTab(0==tabIndex),
                          lastTab((tb->count()-1)==tabIndex),
                          isFirstKTabCtlTab(firstTab && widget->parent()
                                        ? 0==qstrcmp("KTabCtl", widget->parent()->className())
                                        : false),
                          active(flags & Style_Selected),
                          itsHover(itsHoverTab && data.tab()==itsHoverTab &&
                                          !(flags&Style_Selected) &&
                                          tb->currentTab()!=tabIndex);
            const QColor  &fill(getTabFill(flags&Style_Selected, itsHover, itsBackgroundCols));

            if(reverse)
            {
                bool oldLast=lastTab;

                lastTab=firstTab;
                firstTab=oldLast;
            }

            if(::qt_cast<const QTabWidget *>(tb->parent()))
            {
                const QTabWidget *tw((const QTabWidget*)tb->parent());

                // is there a corner widget in the (top) left edge?
                if(tw->cornerWidget(Qt::TopLeft))
                    cornerWidget=true;
            }

            bool drawBottomOfActive(APP_KONQUEROR==itsThemedApp && widget &&
                                    widget->parentWidget() &&
                                    0==qstrcmp("KTabBar", widget->className()) &&
                                    0==qstrcmp("KonqFrameTabs",
                                               widget->parentWidget()->className()));

            QRect tr(r),
                  fr(r);
            int   offset(ROUND_FULL==opts.round ? 2 : 0);

            switch(tb->shape())
            {
                case QTabBar::TriangularAbove:
                case QTabBar::RoundedAbove:
                    if(active)
                    {
                        tr.addCoords(0, 0,  0,-1);
                        fr.addCoords(1, 2,-1,-2);

                        drawBevelGradient(fill, true, p, fr, true,
                                          SHADE_TAB_SEL_LIGHT, SHADE_TAB_SEL_DARK, false,
                                          opts.selectedTabAppearance, WIDGET_TAB_TOP);

                        p->setPen(itsBackgroundCols[5]);
                        p->drawLine(tr.left(), !reverse && firstTab && !cornerWidget ? tr.bottom()+1
                                                                         : tr.bottom(),
                                    tr.left(), tr.top()+offset);
                        p->drawLine(tr.left()+offset, tr.top(), tr.right()-offset, tr.top());
                        p->drawLine(tr.right(), tr.top()+1, tr.right(),
                                    lastTab && reverse ? tr.bottom()+1 : tr.bottom());

                        p->setPen(itsBackgroundCols[0]);
                        p->drawLine(tr.left()+1, tr.bottom()+1, tr.left()+1, tr.top()+2);
                        p->drawLine(tr.left()+1, tr.top()+1, tr.right()-1, tr.top()+1);
                        if(!reverse)
                            p->drawLine(tr.right()-1, tr.bottom()+1, tr.right(), tr.bottom()+1);

                        if(cornerWidget)
                            p->drawPoint(tr.left(), tr.bottom()+1);

                        p->setPen(itsBackgroundCols[dark]);
                        p->drawLine(tr.right()-1, tr.top()+1, tr.right()-1,
                                    lastTab && reverse ? tr.bottom()+1 : tr.bottom());

                        if(ROUND_FULL==opts.round)
                        {
                            p->setPen(itsBackgroundCols[5]);
                            p->drawPoint(tr.x()+1, tr.y()+1);
                            p->drawPoint(tr.x()+tr.width()-2, tr.y()+1);
                            p->setPen(itsBackgroundCols[dark]);
                            p->drawLine(tr.x(), tr.y()+1, tr.x()+1, tr.y());
                            p->drawLine(tr.x()+tr.width()-2, tr.y(), tr.x()+tr.width()-1,
                                        tr.y()+1);
                        }

                        if(drawBottomOfActive)
                        {
                            p->setPen(itsBackgroundCols[0]);
                            p->drawLine(r.x(), r.y()+r.height()-1, r.x()+r.width()-1,
                                        r.y()+r.height()-1);
                        }

                        if(opts.highlightTab)
                        {
                            int offset(ROUND_FULL==opts.round ? 1 : 2),
                                topOffset(ROUND_FULL==opts.round ? -1 : 1);

                            p->setPen(itsMenuitemCols[0]);
                            p->drawLine(fr.left()-(offset-1), fr.top()-1, fr.right()+(offset-1),
                                        fr.top()-1);
                            p->drawLine(fr.left()-(offset-1), fr.top(), fr.right()+(offset-1),
                                        fr.top());

                            p->setPen(itsMenuitemCols[3]);
                            p->drawLine(fr.left()-topOffset, fr.top()-2, fr.right()+topOffset,
                                        fr.top()-2);

                            if(ROUND_FULL==opts.round)
                            {
                                p->drawPoint(fr.left()-(topOffset+1), fr.top()-1);
                                p->drawPoint(fr.right()+(topOffset+1), fr.top()-1);
                            }
                            else
                            {
                                p->drawLine(fr.left()-(offset-1), fr.top(), fr.left()-(offset-1),
                                            fr.top()-1);
                                p->drawLine(fr.right()+(offset-1), fr.top(), fr.right()+(offset-1),
                                            fr.top()-1);
                            }
                        }
                    }
                    else
                    {
                        tr.addCoords(0, 2, 0,-1);
                        fr.addCoords(1, 4,-1,-2);

                        drawBevelGradient(fill, true, p, fr, true, SHADE_TAB_SEL_LIGHT,
                                          SHADE_TAB_SEL_DARK, false,
                                          opts.normalTabAppearance, WIDGET_TAB_TOP);

                        p->setPen(itsBackgroundCols[5]);
                        if(firstTab)
                            p->drawLine(tr.left(), !isFirstKTabCtlTab && !cornerWidget
                                            ? tr.bottom()+1 : tr.bottom(),
                                        tr.left(), tr.top()+1);
                        p->drawLine(ROUND_FULL==opts.round && firstTab ? tr.left()+2 : tr.left(),
                                    tr.top(),
                                    ROUND_FULL==opts.round && lastTab ? tr.right()-2 : tr.right(),
                                    tr.top());

                        p->drawLine(tr.right(), tr.top()+1, tr.right(),
                                    lastTab && reverse ? tr.bottom() + 1 : tr.bottom());
                        p->drawLine(tr.left(), tr.bottom(), tr.right(), tr.bottom());
                        p->setPen(itsBackgroundCols[0]);
                        p->drawLine(tr.left()+1, tr.top()+1, tr.right()-1, tr.top()+1);

                        if(cornerWidget)
                            p->drawPoint(tr.left(), tr.bottom()+1);

                        if(IS_FLAT(opts.appearance) || firstTab)
                        {
                            if(!firstTab)
                                p->setPen(itsBackgroundCols[2]);

                            p->drawLine(tr.left()+1, tr.bottom()-1, tr.left()+1, tr.top()+2);
                        }
                        p->setPen(itsBackgroundCols[0]);
                        /* The following (firsTab ? :) helps out withs tora's tabs */
                        p->drawLine(tr.left()+(firstTab && !reverse ? 1 : -1), tr.bottom()+1,
                                    lastTab && reverse ? tr.right()-1 : tr.right(),
                                    tr.bottom()+1);

                        if(ROUND_FULL==opts.round)
                        {
                            p->setPen(itsBackgroundCols[5]);

                            if(firstTab)
                            {
                                p->drawPoint(tr.left()+1, tr.top()+1);
                                p->setPen(midColor(itsBackgroundCols[5], cg.background()));
                                p->drawPoint(tr.left(), tr.top()+1);
                                p->drawPoint(tr.left()+1, tr.top());
                            }
                            else if(lastTab)
                            {
                                p->drawPoint(tr.right()-1, tr.top()+1);
                                p->setPen(midColor(itsBackgroundCols[5], cg.background()));
                                p->drawPoint(tr.right(), tr.top()+1);
                                p->drawPoint(tr.right()-1, tr.top());
                            }
                            if(!reverse && !isFirstKTabCtlTab && firstTab && 0==r.x() &&
                               !cornerWidget)
                            {
                                p->setPen(itsBackgroundCols[dark]);
                                p->drawPoint(r.x()+1, r.y()+r.height()-1);
                                p->setPen(midColor(itsBackgroundCols[3], cg.background()));
                                p->drawPoint(r.x()+1, r.y()+r.height()-2);
                            }
                        }
                        if(itsHover && opts.coloredMouseOver)
                        {
                            const QColor *border(borderColors(Style_MouseOver, NULL));

                            if(border)
                            {
                                int lo(ROUND_NONE!=opts.round && firstTab && !reverse ? 1 : 0),
                                    ro(ROUND_NONE!=opts.round && lastTab && !reverse ? 1 : 0);

                                p->setPen(border[3]);
                                p->drawLine(fr.left()+lo, fr.top()-2, fr.right()-ro, fr.top()-2);
                                p->setPen(border[0]);
                                p->drawLine(fr.left()+lo, fr.top()-1, fr.right()-ro, fr.top()-1);
                            }
                        }
                    }
                    if(ROUND_SLIGHT==opts.round)
                    {
                        p->setPen(midColor(active && opts.highlightTab
                                    ? itsMenuitemCols[3] : itsBackgroundCols[3], cg.background()));
                        if(active || firstTab)
                            p->drawPoint(tr.left(), tr.top());
                        if(active || lastTab)
                            p->drawPoint(tr.right(), tr.top());
                    }
                    break;
                case QTabBar::TriangularBelow:
                case QTabBar::RoundedBelow:
                    if(active)
                    {
                        fr.addCoords(2, 2,-2,-2);

                        drawBevelGradient(fill, false, p, fr, true,
                                          SHADE_BOTTOM_TAB_SEL_DARK, SHADE_BOTTOM_TAB_SEL_LIGHT,
                                          false, opts.selectedTabAppearance, WIDGET_TAB_BOT);

                        p->setPen(itsBackgroundCols[5]);
                        p->drawLine(tr.left(), tr.bottom()-offset, tr.left(),
                                    firstTab && !cornerWidget ? tr.top() : tr.top()+1);
                        p->drawLine(ROUND_FULL==opts.round ? tr.left()+1 : tr.left(), tr.bottom(),
                                    tr.right()-offset, tr.bottom());
                        p->drawLine(tr.right(), tr.top()+1, tr.right(), tr.bottom()-1);
                        p->setPen(itsBackgroundCols[0]);
                        p->drawLine(tr.left()+1, tr.bottom()-2, tr.left()+1,
                                    firstTab && !cornerWidget ? tr.top() : tr.top()-1);

                        p->setPen(itsBackgroundCols[dark]);
                        p->drawLine(tr.left()+1, tr.bottom()-1, tr.right()-1, tr.bottom()-1);
                        p->drawLine(tr.right()-1, tr.bottom()-2, tr.right()-1, tr.top()-1);
                        p->drawPoint(tr.right(), tr.top());
                        if(cornerWidget)
                            p->drawPoint(tr.left(), tr.top());

                        if(ROUND_FULL==opts.round)
                        {
                            p->setPen(itsBackgroundCols[5]);
                            p->drawPoint(tr.x()+1, tr.y()+tr.height()-2);
                            p->drawPoint(tr.x()+tr.width()-2, tr.y()+tr.height()-2);
                            p->setPen(itsBackgroundCols[dark]);
                            p->drawLine(tr.x(), tr.y()+tr.height()-2, tr.x()+1,
                                        tr.y()+tr.height()-1);
                            p->drawLine(tr.x()+tr.width()-2, tr.y()+tr.height()-1,
                                        tr.x()+tr.width()-1, tr.y()+tr.height()-2);
                        }
                        if(opts.highlightTab)
                        {
                            int offset(ROUND_FULL==opts.round ? 1 : 2),
                                topOffset(ROUND_FULL==opts.round ? 0 : 2);

                            p->setPen(itsMenuitemCols[0]);
                            p->drawLine(fr.left()-offset, fr.bottom()+1, fr.right()+offset,
                                        fr.bottom()+1);
                            p->drawLine(fr.left()-offset, fr.bottom(), fr.right()+offset,
                                        fr.bottom());
                            p->setPen(itsMenuitemCols[3]);
                            p->drawLine(fr.left()-topOffset, fr.bottom()+2, fr.right()+topOffset,
                                        fr.bottom()+2);
                            if(ROUND_FULL==opts.round)
                            {
                                p->drawPoint(fr.left()-(topOffset+1), fr.bottom()+1);
                                p->drawPoint(fr.right()+(topOffset+1), fr.bottom()+1);
                            }
                            else
                            {
                                p->drawLine(fr.left()-offset, fr.bottom()+2, fr.left()-offset,
                                            fr.bottom());
                                p->drawLine(fr.right()+offset, fr.bottom()+2, fr.right()+offset,
                                            fr.bottom());
                            }
                        }
                    }
                    else
                    {
                        tr.addCoords(0, 1, 0,-2);
                        fr.addCoords(1, 2,-1,-3);

                        drawBevelGradient(fill, false, p, fr, true,
                                          SHADE_BOTTOM_TAB_SEL_DARK, SHADE_BOTTOM_TAB_SEL_LIGHT,
                                          false, opts.normalTabAppearance, WIDGET_TAB_BOT);

                        p->setPen(itsBackgroundCols[5]);
                        p->drawLine(tr.left(), tr.bottom()-1, tr.left(),
                                    firstTab && !cornerWidget ? tr.top() : tr.top()+1);
                        p->drawLine(ROUND_FULL==opts.round && firstTab ? tr.left()+2 : tr.left(),
                                    tr.bottom(),
                                    ROUND_FULL==opts.round && lastTab ? tr.right()-2 : tr.right(),
                                    tr.bottom());
                        p->drawLine(tr.right(), tr.top()+1, tr.right(), tr.bottom()-1);
                        p->drawLine(tr.right(), tr.top(), tr.left(), tr.top());

                        p->setPen(itsBackgroundCols[dark]);
                        p->drawLine(tr.left(), tr.top()-1, tr.right(), tr.top()-1);

                        if(ROUND_FULL==opts.round)
                        {
                            p->setPen(itsBackgroundCols[5]);
                            if(firstTab)
                            {
                                p->drawPoint(tr.left()+1, tr.bottom()-1);
                                p->setPen(midColor(itsBackgroundCols[5], cg.background()));
                                p->drawPoint(tr.left(), tr.bottom()-1);
                                p->drawPoint(tr.left()+1, tr.bottom());
                            }
                            else if(lastTab)
                            {
                                p->drawPoint(tr.right()-1, tr.bottom()-1);
                                p->setPen(midColor(itsBackgroundCols[5], cg.background()));
                                p->drawPoint(tr.right(), tr.bottom()-1);
                                p->drawPoint(tr.right()-1, tr.bottom());
                            }

                            if(!isFirstKTabCtlTab && firstTab && 0==r.x() && !cornerWidget)
                            {
                                p->setPen(itsBackgroundCols[dark]);
                                p->drawPoint(r.x()+1, r.y());
                                p->setPen(midColor(itsBackgroundCols[3], cg.background()));
                                p->drawPoint(r.x()+1, r.y()+1);
                            }
                        }
                        if(itsHover && opts.coloredMouseOver)
                        {
                            const QColor *border(borderColors(Style_MouseOver, NULL));

                            if(border)
                            {
                                p->setPen(border[3]);
                                p->drawLine(fr.left(), fr.bottom()+1, fr.right(), fr.bottom()+1);
                                p->setPen(border[1]);
                                p->drawLine(fr.left()-(firstTab ? 1 : 0), fr.bottom(),
                                            fr.right()+(lastTab ? 1 : 0), fr.bottom());
                            }
                        }
                    }
                    if(ROUND_SLIGHT==opts.round)
                    {
                        p->setPen(midColor(active && opts.highlightTab
                                     ? itsMenuitemCols[3] : itsBackgroundCols[3], cg.background()));
                        if(active || firstTab)
                            p->drawPoint(tr.left(), tr.bottom());
                        if(active || lastTab)
                            p->drawPoint(tr.right(), tr.bottom());
                    }
                    break;
                default:
                    KStyle::drawControl(control, p, widget, r, cg, flags, data);
            }
            break;
        }
#if QT_VERSION >= 0x030200
        case CE_TabBarLabel:
        {
            if (data.isDefault())
                break;

            const QTabBar *tb((const QTabBar *) widget);
            QTab          *t(data.tab());
            QRect         tr(r);
            int           shift(pixelMetric(QStyle::PM_TabBarTabShiftVertical, tb));

            if (t->identifier() == tb->currentTab())
            {
                if(QTabBar::RoundedAbove==tb->shape() || QTabBar::TriangularAbove==tb->shape())
                    tr.addCoords(0, -shift, 0, -shift);
            }
            else
                if(QTabBar::RoundedBelow==tb->shape() || QTabBar::TriangularBelow==tb->shape())
                    tr.addCoords(0, shift, 0, shift);

            if(APP_MACTOR==itsThemedApp)
            {
                drawControl(CE_TabBarTab, p, widget, t->rect(), cg, flags, data);

                if(t->iconSet())
                {
                    QIconSet::Mode mode((t->isEnabled() && tb->isEnabled())
                                            ? QIconSet::Normal : QIconSet::Disabled);

                    if (mode == QIconSet::Normal && (flags&QStyle::Style_HasFocus))
                        mode = QIconSet::Active;

                    QPixmap pixmap(t->iconSet()->pixmap(QIconSet::Small, mode));
                    int     pixh(pixmap.height()),
                            xoff(0),
                            yoff(0);

                    if(!(flags&QStyle::Style_Selected))
                    {
                        xoff = pixelMetric(QStyle::PM_TabBarTabShiftHorizontal, widget);
                        yoff = pixelMetric(QStyle::PM_TabBarTabShiftVertical, widget);
                    }
                    p->drawPixmap(t->rect().left()+8+xoff, t->rect().center().y()-pixh/2 + yoff,
                                  pixmap);
                }
            }

            drawItem(p, tr, AlignCenter | ShowPrefix, cg, flags & Style_Enabled, 0, t->text());

            if ((flags & Style_HasFocus) && !t->text().isEmpty())
            {
                QRect fr(r);

                if(QTabBar::RoundedAbove==tb->shape() || QTabBar::TriangularAbove==tb->shape())
                    fr.addCoords(0, 1, 0, -1);
                else
                    fr.addCoords(0, 0, 0, -1);

                drawPrimitive(PE_FocusRect, p, fr, cg);
            }
            break;
        }
#endif
        case CE_PushButtonLabel:  // Taken from Highcolor and Plastik...
        {
            int x, y, w, h;

            r.rect(&x, &y, &w, &h);

            const QPushButton *button(static_cast<const QPushButton *>(widget));
            bool              active(button->isOn() || button->isDown()),
                              cornArrow(false);

            // Shift button contents if pushed.
            if (active)
            {
                x += pixelMetric(PM_ButtonShiftHorizontal, widget);
                y += pixelMetric(PM_ButtonShiftVertical, widget);
                flags |= Style_Sunken;
            }

            // Does the button have a popup menu?
            if (button->isMenuButton())
            {
                int dx(pixelMetric(PM_MenuButtonIndicator, widget)),
                    margin(pixelMetric(PM_ButtonMargin, widget));

                if(button->iconSet() && !button->iconSet()->isNull() &&
                    (dx+button->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal, QIconSet::Off
                                                  ).width()) >= w )
                    cornArrow = true; //To little room. Draw the arrow in the corner, don't adjust
                                      //the widget
                else
                {
                    drawPrimitive(PE_ArrowDown, p,
                                  visualRect(QRect((x + w) - (dx + margin), y, dx, h), r), cg,
                                  flags, data);
                    w-=dx;
                }
            }

            // Draw the icon if there is one
            if (button->iconSet() && !button->iconSet()->isNull())
            {
                QIconSet::Mode  mode(QIconSet::Disabled);
                QIconSet::State state(QIconSet::Off);

                if (button->isEnabled())
                    mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
                if (button->isToggleButton() && button->isOn())
                    state = QIconSet::On;

                QPixmap pixmap = button->iconSet()->pixmap(QIconSet::Small, mode, state);

                static const int constSpace(2);

                int xo(0),
                    pw(pixmap.width()),
                    iw(0);

                if (button->text().isEmpty() && !button->pixmap())
                    p->drawPixmap(x + (w>>1) - (pixmap.width()>>1),
                                  y + (h>>1) - (pixmap.height()>>1),
                                  pixmap);
                else
                {
                    iw=button->pixmap() ? button->pixmap()->width()
                                        : widget->fontMetrics().size(Qt::ShowPrefix,
                                                                     button->text()).width();

                    int cw(iw+pw+constSpace);

                    xo=cw<w ? (w-cw)>>1 : constSpace; 
                    p->drawPixmap(x+xo, y + (h>>1) - (pixmap.height()>>1), pixmap);
                    xo+=pw;
                }

                if (cornArrow) //Draw over the icon
                    drawPrimitive(PE_ArrowDown, p, visualRect(QRect(x + w - 6, x + h - 6, 7, 7), r),
                                  cg, flags, data);

                if(xo && iw)
                {
                    x+= xo+constSpace;
                    w=iw;
                }
                else
                {
                    x+= pw+constSpace;
                    w-= pw+constSpace;
                }
            }

            // Make the label indicate if the button is a default button or not
            int          i,
                         j(opts.embolden && button->isDefault() ? 2 : 1);
            const QColor &textCol(opts.coloredSidebarButtons && button->isFlat() &&
                                  button->inherits("KMultiTabBarTab") &&
                                  (button->isOn() || flags&QStyle::Style_On)
                                    ? qApp->palette().active().highlightedText()
                                    : button->colorGroup().buttonText());

            for(i=0; i<j; i++)
                drawItem(p, QRect(x+i, y, w, h), AlignCenter|ShowPrefix, button->colorGroup(),
                         button->isEnabled(),
                         button->pixmap(), button->text(), -1, &textCol);

            // Draw a focus rect if the button has focus
            if (flags & Style_HasFocus)
               drawPrimitive(PE_FocusRect, p, QStyle::visualRect(subRect(SR_PushButtonFocusRect,
                             widget), widget), cg, flags);
            break;
        }
        case CE_PopupMenuItem:
        {
            if(!widget || data.isDefault())
                break;

            const QPopupMenu *popupmenu((const QPopupMenu *)widget);
            QMenuItem        *mi(data.menuItem());
            int              tab(data.tabWidth()),
                             maxpmw(data.maxIconWidth()),
                             x, y, w, h;

            r.rect(&x, &y, &w, &h);

            if((flags & Style_Active)&&(flags & Style_Enabled))
                drawMenuItem(p, r, true, cg, control,
                             opts.roundMenuItems
                                ? ROUNDED_ALL
                                : ROUNDED_NONE,
                             opts.borderMenuItems,
                             opts.lighterPopupMenuBgnd ? itsLighterPopupMenuBgndCol
                                                       : itsBackgroundCols[ORIGINAL_SHADE]);
            else if(widget->erasePixmap() && !widget->erasePixmap()->isNull())
                p->drawPixmap(x, y, *widget->erasePixmap(), x, y, w, h);
            else
                p->fillRect(r, opts.lighterPopupMenuBgnd ? itsLighterPopupMenuBgndCol
                                                         : itsBackgroundCols[ORIGINAL_SHADE]);

            if(!mi)
                break;

            if(mi->isSeparator())
            {
                y=r.y()+((r.height()/2)-1);
                p->setPen(itsBackgroundCols[5]);
                p->drawLine(r.x()+4, y, r.x()+r.width()-5, y);
                p->setPen(itsBackgroundCols[0]);
                p->drawLine(r.x()+4, y+1, r.x()+r.width()-5, y+1);
                break;
            }

            maxpmw=QMAX(maxpmw, 16);

            QRect cr, ir, tr, sr;
            // check column
            cr.setRect(r.left(), r.top(), maxpmw, r.height());
            // submenu indicator column
            sr.setCoords(r.right()-maxpmw, r.top(), r.right(), r.bottom());
            // tab/accelerator column
            tr.setCoords(sr.left()-tab-4, r.top(), sr.left(), r.bottom());
            // item column
            ir.setCoords(cr.right()+4, r.top(), tr.right()-4, r.bottom());

            bool reverse(QApplication::reverseLayout());

            if(reverse)
            {
                cr=visualRect(cr, r);
                sr=visualRect(sr, r);
                tr=visualRect(tr, r);
                ir=visualRect(ir, r);
            }

            if(mi->iconSet())
            {
                // Select the correct icon from the iconset
                QIconSet::Mode mode=flags & Style_Active
                                        ? (mi->isEnabled() ? QIconSet::Active : QIconSet::Disabled)
                                        : (mi->isEnabled() ? QIconSet::Normal : QIconSet::Disabled);
                cr=visualRect(QRect(x, y, maxpmw, h), r);

                // Do we have an icon and are checked at the same time?
                // Then draw a "pressed" background behind the icon
                if(popupmenu->isCheckable() && mi->isChecked())
                {
                    QColor color(mi->isEnabled() ? itsBackgroundCols[2] : cg.background());

                    if(mi->isEnabled() && opts.lighterPopupMenuBgnd)
                        shade(color, &color, POPUPMENU_LIGHT_FACTOR);

                    QBrush brush(color);

                    qDrawShadePanel(p, cr.x()+1, cr.y()+2, cr.width()-2, cr.height()-4,
                                    QColorGroup(itsBackgroundCols[5],
                                    itsBackgroundCols[ORIGINAL_SHADE],
                                    itsBackgroundCols[0], itsBackgroundCols[5],
                                    itsBackgroundCols[2], cg.text(),
                                    itsBackgroundCols[ORIGINAL_SHADE]),
                                    true, 1, &brush);
                }
                // Draw the icon
                QPixmap pixmap(mi->iconSet()->pixmap(QIconSet::Small, mode));
                QRect   pmr(0, 0, pixmap.width(), pixmap.height());

                pmr.moveCenter(cr.center());
                p->drawPixmap(pmr.topLeft(), pixmap);
            }
            else if(popupmenu->isCheckable() && mi->isChecked())
            {
                // Draw a "pressed" background behind the check
                QColor color(mi->isEnabled() ? itsBackgroundCols[2] : cg.background());

                if(mi->isEnabled() && opts.lighterPopupMenuBgnd)
                    shade(color, &color, POPUPMENU_LIGHT_FACTOR);

                QBrush brush(color);

                qDrawShadePanel(p, cr.x()+1, cr.y()+2, cr.width()-2, cr.height()-4,
                                QColorGroup(itsBackgroundCols[5], itsBackgroundCols[ORIGINAL_SHADE],
                                itsBackgroundCols[0], itsBackgroundCols[5], itsBackgroundCols[2],
                                cg.text(), itsBackgroundCols[ORIGINAL_SHADE]), true, 1, &brush);

                drawPrimitive(PE_CheckMark, p, cr, cg,
                              (flags &(Style_Enabled|Style_Active))| Style_On);
            }

            QColor textcolor,
                   embosscolor;

            if(flags & Style_Active)
            {
                if(!(flags & Style_Enabled))
                {
                    textcolor=cg.text();
                    embosscolor=cg.light();
                }
                else
                {
                    textcolor=cg.highlightedText();
                    embosscolor=cg.midlight().light();
                }
            }
            else if(!(flags & Style_Enabled))
            {
                textcolor=cg.text();
                embosscolor=cg.light();
            }
            else
                textcolor=embosscolor=cg.buttonText();
            p->setPen(textcolor);

            if(mi->custom())
            {
                p->save();
                if(!(flags & Style_Enabled))
                {
                    p->setPen(cg.light());
                    mi->custom()->paint(p, cg,(flags & Style_Enabled)?(flags & Style_Active): 0,
                                        flags & Style_Enabled, ir.x()+1, ir.y()+1, ir.width()-1,
                                        ir.height()-1);
                    p->setPen(textcolor);
                }
                mi->custom()->paint(p, cg,(flags & Style_Enabled)?(flags & Style_Active): 0,
                                    flags & Style_Enabled, ir.x(), ir.y(), ir.width(), ir.height());
                p->restore();
            }

            QString text=mi->text();

            if(!text.isNull())
            {
                int t(text.find('\t'));

                // draw accelerator/tab-text
                if(t>=0)
                {
                    int alignFlag(AlignVCenter | ShowPrefix | DontClip | SingleLine);

                    alignFlag |=(reverse ? AlignLeft : AlignRight);

                    if(!(flags & Style_Enabled))
                    {
                        p->setPen(embosscolor);
                        tr.moveBy(1, 1);
                        p->drawText(tr, alignFlag, text.mid(t +1));
                        tr.moveBy(-1,-1);
                        p->setPen(textcolor);
                    }

                    p->drawText(tr, alignFlag, text.mid(t +1));
                }

                int alignFlag(AlignVCenter | ShowPrefix | DontClip | SingleLine);

                alignFlag |=(reverse ? AlignRight : AlignLeft);

                if(!(flags & Style_Enabled))
                {
                    p->setPen(embosscolor);
                    ir.moveBy(1, 1);
                    p->drawText(ir, alignFlag, text, t);
                    ir.moveBy(-1,-1);
                    p->setPen(textcolor);
                }

                p->drawText(ir, alignFlag, text, t);
            } 
            else if(mi->pixmap())
            {
                QPixmap *pixmap(mi->pixmap());

                if(1==pixmap->depth())
                    p->setBackgroundMode(OpaqueMode);
                int diffw(((r.width() - pixmap->width())/2) +
                          ((r.width() - pixmap->width())%2));
                p->drawPixmap(r.x()+diffw, r.y()+1, *pixmap );
                if(1==pixmap->depth())
                    p->setBackgroundMode(TransparentMode);
            }

            if(mi->popup())
                drawArrow(p, sr, cg, flags, reverse ? PE_ArrowLeft : PE_ArrowRight, false, true);
            break;
        }
        case CE_MenuBarItem:
        {
            bool down( (flags&Style_Enabled) && (flags&Style_Active) && (flags&Style_Down) ),
                 active( (flags&Style_Enabled) && (flags&Style_Active) &&
                       ( (flags&Style_Down) || opts.menubarMouseOver ) );

            if(!active || (QTC_ROUNDED && IS_GLASS(opts.menubarAppearance)))
            {
                QMenuBar *mb((QMenuBar*)widget);
                QRect    r2(r);

                r2.setY(mb->rect().y()+1);
                r2.setHeight(mb->rect().height()-2);

                drawMenuOrToolBarBackground(p, r2, cg);
            }

            if(active)
                drawMenuItem(p, r, true, cg, control,
                             opts.roundMenubarItems
                                ? opts.menubarRoundTopOnly && down
                                    ? ROUNDED_TOP
                                    : ROUNDED_ALL
                                : ROUNDED_NONE,
                             opts.borderMenubarItems,
                             itsMenubarCols[ORIGINAL_SHADE]);

            if(data.isDefault())
                break;

            QMenuItem *mi(data.menuItem());

            if(mi->text().isEmpty()) // Draw pixmap...
                drawItem(p, r, AlignCenter|ShowPrefix|DontClip|SingleLine, cg, flags&Style_Enabled,
                     mi->pixmap(), QString::null);
            else
            {
                const QColor *col=active
                                ? opts.customMenuTextColor
                                    ? &opts.customMenuSelTextColor
                                    : &cg.highlightedText()
                                : itsActive
                                    ? opts.customMenuTextColor
                                        ? &opts.customMenuNormTextColor
                                        : SHADE_SELECTED==opts.shadeMenubars ||
                                            (SHADE_CUSTOM==opts.shadeMenubars &&
                                            TOO_DARK(itsMenubarCols[ORIGINAL_SHADE]))
                                            ? &cg.highlightedText()
                                            : &cg.buttonText()
                                    : &cg.buttonText();

                p->setPen(*col);
                p->drawText(r, AlignCenter|ShowPrefix|DontClip|SingleLine, mi->text());
            }

            break;
        }
        case CE_MenuBarEmptyArea:
            drawMenuOrToolBarBackground(p, r, cg);
            break;
        case CE_DockWindowEmptyArea:
            if(widget && widget->inherits("QToolBar"))
            {
                QDockWindow *wind((QDockWindow*)widget);

                drawMenuOrToolBarBackground(p, r, cg, false, Qt::Horizontal==wind->orientation());
            }
            else
                KStyle::drawControl(control, p, widget, r, cg, flags, data);
            break;
        case CE_ProgressBarGroove:
        {
            p->setBrush(flags & Style_Enabled ? cg.base() : cg.background());
            p->drawRect(r);
            if(QTC_ROUNDED)
            {
                const QColor *use(backgroundColors(cg));

                drawBorder(cg.background(), p, r, cg, (SFlags)(flags|Style_Horizontal),
                           ROUNDED_ALL, use, true, BORDER_SUNKEN);
            }
            else
                qDrawShadePanel(p, r, QColorGroup(itsBackgroundCols[5],
                                itsBackgroundCols[ORIGINAL_SHADE],
                                itsBackgroundCols[0], itsBackgroundCols[5],
                                itsBackgroundCols[2], cg.text(), itsBackgroundCols[ORIGINAL_SHADE]),
                                true, 1);
            break;
        }
        case CE_ProgressBarContents:
        {
            const QProgressBar *pb((const QProgressBar*)widget);
            int                steps(pb->totalSteps());

            if(0==steps)//Busy indicator
            {
                static const int barWidth(10);

                int progress(pb->progress() % (2*(r.width()-barWidth)));

                if(progress < 0)
                    progress = 0;
                else if(progress > r.width()-barWidth)
                    progress = (r.width()-barWidth)-(progress-(r.width()-barWidth));

                p->fillRect(r, flags & Style_Enabled ? cg.base() : cg.background());
                drawLightBevel(cg.background(), p, QRect(r.x()+progress, r.y(), barWidth,
                               r.height()), cg, flags, ROUNDED_ALL, itsMenuitemCols[ORIGINAL_SHADE],
                               itsMenuitemCols, true, true);
            }
            else
            {
                QRect cr(subRect(SR_ProgressBarContents, widget));

                if(cr.isValid() && pb->progress()>0)
                {
                    double pg(((double)pb->progress()) / steps);
                    int    width(QMIN(cr.width(), (int)(pg * cr.width())));

                    if(QApplication::reverseLayout())
                        drawProgress(p, QRect(cr.x()+(cr.width()-width), cr.y(), width,
                                     cr.height()), cg, flags,
                                     width==cr.width() ? ROUNDED_NONE : ROUNDED_LEFT, widget);
                    else
                        drawProgress(p, QRect(cr.x(), cr.y(), width, cr.height()), cg, flags,
                                     width==cr.width() ?  ROUNDED_NONE : ROUNDED_RIGHT, widget);
                }
            }
            break;
        }
        case CE_PushButton:
        {
            const QPushButton *button(static_cast<const QPushButton *>(widget));

            if(opts.coloredSidebarButtons && button->isFlat() && button->inherits("KMultiTabBarTab"))
            {
                QRect r2(r);

                flags|=QTC_TOGGLE_BUTTON;
                if(button->isOn())
                    flags|=QStyle::Style_On;

                const QColor *use(flags&QStyle::Style_On ? getSidebarButtons() : buttonColors(cg));

                if((flags&QStyle::Style_On ) || flags&Style_MouseOver)
                {
                    r2.addCoords(-1, -1, 1, 1);
                    drawLightBevel(p, r2, cg, flags|Style_Horizontal, ROUNDED_NONE,
                                   getFill(flags, use), use, false, false, WIDGET_MENU_ITEM);
                }
                else
                    p->fillRect(r2, cg.background());

                if(flags&Style_MouseOver && opts.coloredMouseOver)
                {
                    r2=r;
                    r2.addCoords(1, 1, -1, -1);
                    p->setPen(itsMouseOverCols[flags&QStyle::Style_On ? 0 : 1]);
                    p->drawLine(r.x(), r.y(), r.x()+r.width()-1, r.y());
                    p->drawLine(r.x(), r.y(), r.x(), r.y()+r.height()-1);
                    p->drawLine(r2.x(), r2.y(), r2.x()+r2.width()-1, r2.y());
                    p->drawLine(r2.x(), r2.y(), r2.x(), r2.y()+r2.height()-1);
                    p->setPen(itsMouseOverCols[flags&QStyle::Style_On ? 1 : 2]);
                    p->drawLine(r.x(), r.y()+r.height()-1, r.x()+r.width()-1, r.y()+r.height()-1);
                    p->drawLine(r.x()+r.width()-1, r.y(), r.x()+r.width()-1, r.y()+r.height()-1);
                    p->drawLine(r2.x(), r2.y()+r2.height()-1, r2.x()+r2.width()-1,
                                r2.y()+r2.height()-1);
                    p->drawLine(r2.x()+r2.width()-1, r2.y(), r2.x()+r2.width()-1,
                                r2.y()+r2.height()-1);
                }
            }
            else
            {
                if (QTC_ROUNDED && isFormWidget(widget))
                    itsFormMode = true;

                if(IND_FONT_COLOR==opts.defBtnIndicator && button->isDefault())
                    flags|=Style_ButtonDefault;

                if(button->isToggleButton())
                    flags|=QTC_TOGGLE_BUTTON;

                drawPrimitive(PE_ButtonCommand, p, r, cg, flags);
                if (button->isDefault() && IND_CORNER==opts.defBtnIndicator)
                    drawPrimitive(PE_ButtonDefault, p, r, cg, flags);
                itsFormMode = false;
            }
            break;
        }
        case CE_CheckBox:
            if (QTC_ROUNDED && isFormWidget(widget))
                itsFormMode = true;
            drawPrimitive(PE_Indicator, p, r, cg, flags, data);
            itsFormMode = false;
            break;
        case CE_CheckBoxLabel:
            if(opts.crLabelHighlight)
            {
                const QCheckBox *checkbox((const QCheckBox *)widget);

                if(flags&Style_MouseOver &&
#if QT_VERSION >= 0x030200
                   HOVER_CHECK==itsHover && itsHoverWidget && itsHoverWidget==widget &&
#endif
                   !isFormWidget(widget))
                {
#if QT_VERSION >= 0x030200
                    QRect   cr(checkbox->rect());
                    QRegion r(QRect(cr.x(), cr.y(), visualRect(subRect(SR_CheckBoxFocusRect, widget),
                                                               widget).width()+
                                                               pixelMetric(PM_IndicatorWidth)+4,
                                                               cr.height()));

#else
                    QRegion r(checkbox->rect());
#endif
                    r-=visualRect(subRect(SR_CheckBoxIndicator, widget), widget);
                    p->setClipRegion(r);
                    p->fillRect(checkbox->rect(), shade(cg.background(), opts.highlightFactor));
                    p->setClipping(false);
                }
                int alignment(QApplication::reverseLayout() ? AlignRight : AlignLeft);

                drawItem(p, r, alignment | AlignVCenter | ShowPrefix, cg,
                         flags & Style_Enabled, checkbox->pixmap(),  checkbox->text());

                if(checkbox->hasFocus())
                    drawPrimitive(PE_FocusRect, p, visualRect(subRect(SR_CheckBoxFocusRect, widget),
                                  widget), cg, flags);
            }
            else
                KStyle::drawControl(control, p, widget, r, cg, flags, data);
            break;
        case CE_RadioButton:
            itsFormMode=isFormWidget(widget);
            drawPrimitive(PE_ExclusiveIndicator, p, r, cg, flags, data);
            itsFormMode=false;
            break;
        case CE_RadioButtonLabel:
            if(opts.crLabelHighlight)
            {
                const QRadioButton *radiobutton((const QRadioButton *)widget);

                if(flags&Style_MouseOver &&
#if QT_VERSION >= 0x030200
                   HOVER_RADIO==itsHover && itsHoverWidget && itsHoverWidget==widget &&
#endif
                   !isFormWidget(widget))
                {
#if QT_VERSION >= 0x030200
                    QRect   rb(radiobutton->rect());
                    QRegion r(QRect(rb.x(), rb.y(), 
                                    visualRect(subRect(SR_RadioButtonFocusRect, widget),
                                               widget).width()+
                                               pixelMetric(PM_ExclusiveIndicatorWidth)+4,
                                               rb.height()));
#else
                    QRegion r(radiobutton->rect());
#endif
                    r-=visualRect(subRect(SR_RadioButtonIndicator, widget), widget);
                    p->setClipRegion(r);
                    p->fillRect(radiobutton->rect(), shade(cg.background(), opts.highlightFactor));
                    p->setClipping(false);
                }

                int alignment(QApplication::reverseLayout() ? AlignRight : AlignLeft);

                drawItem(p, r, alignment | AlignVCenter | ShowPrefix, cg, flags & Style_Enabled,
                         radiobutton->pixmap(), radiobutton->text());

                if(radiobutton->hasFocus())
                    drawPrimitive(PE_FocusRect, p, visualRect(subRect(SR_RadioButtonFocusRect,
                                  widget), widget), cg, flags);
                break;
            }
        default:
            KStyle::drawControl(control, p, widget, r, cg, flags, data);
    }
}

void QtCurveStyle::drawControlMask(ControlElement control, QPainter *p, const QWidget *widget,
                                   const QRect &r, const QStyleOption &data) const
{
    switch(control)
    {
        case CE_PushButton:
        case CE_MenuBarItem:
            if(ROUND_FULL==opts.round)
            {
                int offset(ROUND_SLIGHT==opts.round ||
                           r.width()<QTC_MIN_BTN_SIZE || r.height()<QTC_MIN_BTN_SIZE ? 1 : 2);

                p->fillRect(r, color0);
                p->fillRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2, color1);
                p->setPen(color1);
                p->drawLine(r.x()+offset, r.y(), r.x()+r.width()-(offset+1), r.y());
                p->drawLine(r.x()+offset, r.y()+r.height()-1, r.x()+r.width()-(offset+1),
                            r.y()+r.height()-1);
                p->drawLine(r.x(), r.y()+offset, r.x(), r.y()+r.height()-(offset+1));
                p->drawLine(r.x()+r.width()-1, r.y()+offset, r.x()+r.width()-1,
                            r.y()+r.height()-(offset+1));
            }
            else
                p->fillRect(r, color1);
            break;
        default:
            KStyle::drawControlMask(control, p, widget, r, data);
    }
}

void QtCurveStyle::drawComplexControlMask(ComplexControl control, QPainter *p, const QWidget *widget,
                                          const QRect &r, const QStyleOption &data) const
{
    switch (control)
    {
        case CC_SpinWidget:
        case CC_ComboBox:
        case CC_ToolButton:
            drawControlMask(CE_PushButton, p, widget, r, data);
            break;
        default:
            KStyle::drawComplexControlMask(control, p, widget, r, data);
    }
}

QRect QtCurveStyle::subRect(SubRect subrect, const QWidget *widget)const
{
    QRect rect,
          wrect(widget->rect());

    switch(subrect)
    {
        case SR_PushButtonFocusRect:
        {
            int dbw1(pixelMetric(PM_ButtonDefaultIndicator, widget)),
                dbw2(dbw1*2),
                border(3),
                border2=(border*2);

            if(IND_COLORED==opts.defBtnIndicator && FOCUS_COLORED==opts.focus)
            {
                border++;
                border2+=2;
            }
            rect.setRect(wrect.x()+border +dbw1, wrect.y()+border +dbw1,
                         wrect.width()-border2-dbw2,
                         wrect.height()-border2-dbw2);
            break;
        }
        case SR_CheckBoxIndicator:
        {
            int h(pixelMetric(PM_IndicatorHeight));

            rect.setRect((widget->rect().height()-h)>>1, (widget->rect().height()-h)>>1,
                         pixelMetric(PM_IndicatorWidth), h);
            break;
        }
        case SR_RadioButtonIndicator:
        {
            int h(pixelMetric(PM_ExclusiveIndicatorHeight));

            rect.setRect((widget->rect().height()-h)>>1, (widget->rect().height()-h)>>1,
                         pixelMetric(PM_ExclusiveIndicatorWidth), h);
            break;
        }
        case SR_ProgressBarGroove:
            rect=QRect(widget->rect());
            break;
        case SR_ProgressBarContents:
        case SR_ProgressBarLabel:
            rect=QRect(wrect.left()+2, wrect.top()+2, wrect.width()-4, wrect.height()-4);
            break;
        default:
            rect=KStyle::subRect(subrect, widget);
    }

    return rect;
}

void QtCurveStyle::drawComplexControl(ComplexControl control, QPainter *p, const QWidget *widget,
                                      const QRect &r, const QColorGroup &cg, SFlags flags,
                                      SCFlags controls, SCFlags active,
                                      const QStyleOption &data) const
{
    if(widget==itsHoverWidget)
        flags |=Style_MouseOver;

    switch(control)
    {
        case CC_ToolButton:
        {
            const QToolButton *toolbutton((const QToolButton *)widget);
            QRect             button(querySubControlMetrics(control, widget, SC_ToolButton, data)), 
                              menuarea(querySubControlMetrics(control, widget, SC_ToolButtonMenu,
                                       data));
            SFlags            bflags(flags|QTC_STD_TOOLBUTTON),
                              mflags(flags);

            if (APP_KORN==itsThemedApp)
            {
                drawPrimitive(PE_ButtonTool, p, button, cg, bflags, data);
                break;
            }

            const QToolBar *tb(widget->parentWidget()
                                   ? ::qt_cast<const QToolBar *>(widget->parentWidget()) : NULL);
            bool onControlButtons(false),
                 onExtender(!tb &&
                            widget->parentWidget() &&
                            widget->parentWidget()->inherits( "QToolBarExtensionWidget") &&
                            ::qt_cast<QToolBar *>(widget->parentWidget()->parentWidget()));

            if (!tb && !onExtender && widget->parentWidget() &&
                !qstrcmp(widget->parentWidget()->name(), "qt_maxcontrols"))
                onControlButtons = true;

            if(active & SC_ToolButton)
                bflags |=Style_Down;
            if(active & SC_ToolButtonMenu)
                mflags |=Style_Down;

            if(controls&SC_ToolButton)
            {
                // If we're pressed, on, or raised...
#if KDE_VERSION >= 0x30200
                if(bflags &(Style_Down | Style_On | Style_Raised) || onControlButtons)
#else
                if(bflags &(Style_Down | Style_On | Style_Raised | Style_MouseOver) ||
                   onControlButtons)
#endif
                {
                    //Make sure the standalone toolbuttons have a gradient in the right direction
                    if (!tb && !onControlButtons)
                        bflags |= Style_Horizontal;

                    if(tb)
                        if(Qt::Vertical==tb->orientation())
                            bflags|=QTC_VERTICAL_TB_BUTTON;
                        else
                            bflags|=Style_Horizontal;

                    if(toolbutton->isToggleButton())
                        bflags|=QTC_TOGGLE_BUTTON;

                    drawPrimitive(PE_ButtonTool, p, button, cg, bflags, data);
                }

                // Check whether to draw a background pixmap
                else if(APP_MACTOR!=itsThemedApp && toolbutton->parentWidget() &&
                          toolbutton->parentWidget()->backgroundPixmap() &&
                          !toolbutton->parentWidget()->backgroundPixmap()->isNull())
                    p->drawTiledPixmap(r, *(toolbutton->parentWidget()->backgroundPixmap()),
                                       toolbutton->pos());
                else if(widget->parent())
                {
                    QToolBar *tb(NULL);

                    if(::qt_cast<const QToolBar *>(widget->parent()))
                        tb=(QToolBar*)widget->parent();
                    else if(widget->parent()->inherits("QToolBarExtensionWidget"))
                    {
                        QWidget *parent=(QWidget*)widget->parent();

                        tb=(QToolBar*)parent->parent();
                    }

                    if(tb)
                    {
                        QRect tbr(tb->rect());
                        bool  horiz(Qt::Horizontal==tb->orientation());

                        if(!IS_FLAT(opts.toolbarAppearance))
                            if(horiz)
                                tbr.addCoords(0, -1, 0, 0);
                            else
                                tbr.addCoords(-1, 0, 0, 0);

                        drawMenuOrToolBarBackground(p, tbr, cg, false, horiz);
                    }
                }
            }

            if(controls&SC_ToolButtonMenu)
            {
                if(mflags &(Style_Down | Style_On | Style_Raised))
                    drawPrimitive(PE_ButtonDropDown, p, menuarea, cg, mflags, data);
                drawPrimitive(PE_ArrowDown, p, menuarea, cg, mflags, data);
            }

            if(toolbutton->hasFocus() && !toolbutton->focusProxy())
            {
                QRect fr(toolbutton->rect());
                fr.addCoords(3, 3,-3,-3);
                drawPrimitive(PE_FocusRect, p, fr, cg);
            }
            break;
        }
        case CC_ComboBox:
        {
            const QComboBox *combobox((const QComboBox *)widget);
            QRect           frame(QStyle::visualRect(querySubControlMetrics(CC_ComboBox, widget,
                                                                            SC_ComboBoxFrame,
                                                                            data), widget)),
                            arrow(QStyle::visualRect(querySubControlMetrics(CC_ComboBox, widget,
                                                                            SC_ComboBoxArrow,
                                                                            data), widget)),
                            field(QStyle::visualRect(querySubControlMetrics(CC_ComboBox, widget,
                                                                            SC_ComboBoxEditField,
                                                                            data), widget));
            const QColor    *use(buttonColors(cg));
            bool            editable(combobox->editable()),
                            sunken(combobox->listBox() ? combobox->listBox()->isShown() : false),
                            reverse(QApplication::reverseLayout());

            if (QTC_ROUNDED && isFormWidget(widget))
                itsFormMode = true;

            if(editable)
            {
                p->setPen(cg.background());
                p->drawRect(r);
            }

            if(controls&SC_ComboBoxFrame && frame.isValid())
            {
                SFlags fillFlags(flags);

                if(sunken)
                {
                    fillFlags|=Style_Sunken;
                    if(fillFlags&Style_MouseOver)
                        fillFlags-=Style_MouseOver;
                }

                if(opts.coloredMouseOver && fillFlags&Style_MouseOver && editable && !sunken)
                    frame.addCoords(reverse ? 0 : 1, 0, reverse ? -1 : 0, 0);

                drawLightBevel(p, frame, cg, fillFlags|Style_Raised|Style_Horizontal,
                               controls&SC_ComboBoxEditField && field.isValid() && editable
                                   ? (reverse ? ROUNDED_LEFT : ROUNDED_RIGHT) : ROUNDED_ALL,
                               getFill(fillFlags, use), use, true, true, WIDGET_STD_BUTTON);
            }

            if(controls&SC_ComboBoxArrow && arrow.isValid())
            {
                if(sunken)
                    arrow.addCoords(1, 1, 1, 1);
                drawPrimitive(PE_ArrowDown, p, arrow, cg, flags & ~Style_MouseOver);
            }

            if(controls&SC_ComboBoxEditField && field.isValid())
            {
                if(editable)
                {
                    field.addCoords(-1,-1, 0, 1);
                    p->setPen(flags&Style_Enabled ? cg.base() : cg.background());
                    p->drawRect(field);
                    field.addCoords(-2,-2, 2, 2);
                    drawEntryField(p, field, cg, flags, flags&Style_Enabled &&
                                   (flags&Style_HasFocus), reverse ? ROUNDED_RIGHT : ROUNDED_LEFT);
                }
                else 
                {
                    field.addCoords(1, sunken ? 2 : 1, sunken ? 2 : 1, -1);
                    p->setPen(use[5]);
                    p->drawLine(reverse ? field.left()-3 : field.right(), field.top(),
                                reverse ? field.left()-3 : field.right(), field.bottom());
                    if(!sunken)
                    {
                        p->setPen(use[0]);
                        p->drawLine(reverse ? field.left()-2 : field.right()+1, field.top(),
                                    reverse ? field.left()-2 : field.right()+1, field.bottom());
                    }
                }

                if((flags & Style_HasFocus) && !editable)
                {
                    QRect fr(QStyle::visualRect(subRect(SR_ComboBoxFocusRect, widget), widget));

                    if(reverse)
                        fr.addCoords(3, 0, 0, 0);
                    else
                        fr.addCoords(0, 0, -2, 0);

                    drawPrimitive(PE_FocusRect, p, fr, cg, flags | Style_FocusAtBorder,
                                  QStyleOption(cg.highlight()));
                }
            }

            p->setPen(flags & Style_Enabled ? cg.buttonText() : cg.mid());
            itsFormMode = false;
            break;
        }
        case CC_SpinWidget:
        {
            const QSpinWidget *spinwidget((const QSpinWidget *)widget);
            QRect             frame(querySubControlMetrics(CC_SpinWidget, widget, SC_SpinWidgetFrame,
                                    data)),
                              up(spinwidget->upRect()),
                              down(spinwidget->downRect());
            bool              hw(itsHoverWidget && itsHoverWidget==spinwidget),
                              reverse(QApplication::reverseLayout()),
                              doFrame((controls&SC_SpinWidgetFrame) && frame.isValid());

            if(flags&Style_MouseOver)
                flags-=Style_MouseOver;

            if(!reverse && doFrame)
            {
                if (QTC_ROUNDED && isFormWidget(widget))
                    itsFormMode = true;
                frame.setWidth(frame.width()+1);
                drawEntryField(p, frame, cg, flags,
                               spinwidget ? spinwidget->hasFocus() && (flags&Style_Enabled) : false,
                               ROUNDED_LEFT, true);
                itsFormMode=false;
            }

            if((controls&SC_SpinWidgetUp) && up.isValid())
            {
                PrimitiveElement pe(PE_SpinWidgetUp);
                SFlags           upflags(flags);

                if(hw && HOVER_SW_UP==itsHover)
                    upflags|=Style_MouseOver;
                up.setHeight(up.height()+1);
                if(spinwidget->buttonSymbols()==QSpinWidget::PlusMinus)
                    pe=PE_SpinWidgetPlus;
                if(!spinwidget->isUpEnabled())
                    upflags^=Style_Enabled;
                drawPrimitive(pe, p, up, cg,
                             upflags |((active==SC_SpinWidgetUp)
                                 ? Style_On | Style_Sunken : Style_Raised));
            }

            if((controls&SC_SpinWidgetDown) && down.isValid())
            {
                PrimitiveElement pe(PE_SpinWidgetDown);
                SFlags           downflags(flags);

                if(hw && HOVER_SW_DOWN==itsHover)
                    downflags|=Style_MouseOver;
                if(spinwidget->buttonSymbols()==QSpinWidget::PlusMinus)
                    pe=PE_SpinWidgetMinus;
                if(!spinwidget->isDownEnabled())
                    downflags^=Style_Enabled;
                drawPrimitive(pe, p, down, cg,
                              downflags |((active==SC_SpinWidgetDown)
                                  ? Style_On | Style_Sunken : Style_Raised));
            }

            if(reverse && doFrame)
            {
                if (QTC_ROUNDED && isFormWidget(widget))
                    itsFormMode = true;
                frame.setWidth(frame.width()+1);
                drawEntryField(p, frame, cg, flags,
                               spinwidget ? spinwidget->hasFocus() && (flags&Style_Enabled) : false,
                               ROUNDED_RIGHT, true);
                itsFormMode=false;
            }
            break;
        }
        case CC_ScrollBar:
        {
            const QScrollBar *sb((const QScrollBar *)widget);
            bool             hw(itsHoverWidget && itsHoverWidget==sb),
                             useThreeButtonScrollBar(SCROLLBAR_KDE==opts.scrollbarType),
                             horiz(Qt::Horizontal==sb->orientation()),
                             maxed(sb->minValue() == sb->maxValue());
            SFlags           sflags((horiz ? Style_Horizontal : Style_Default) |
                                    (maxed ? Style_Default : Style_Enabled));
            QRect            subline(querySubControlMetrics(control, widget, SC_ScrollBarSubLine,
                                                            data)),
                             addline(querySubControlMetrics(control, widget, SC_ScrollBarAddLine,
                                                            data)),
                             subpage(querySubControlMetrics(control, widget, SC_ScrollBarSubPage,
                                                            data)),
                             addpage(querySubControlMetrics(control, widget, SC_ScrollBarAddPage,
                                                            data)),
                             slider(querySubControlMetrics(control, widget, SC_ScrollBarSlider,
                                                           data)),
                             first(querySubControlMetrics(control, widget, SC_ScrollBarFirst,
                                                          data)),
                             last(querySubControlMetrics(control, widget, SC_ScrollBarLast,
                                                         data)),
                             subline2(addline);

            if (useThreeButtonScrollBar)
                if (horiz)
                    subline2.moveBy(-addline.width(), 0);
                else
                    subline2.moveBy(0, -addline.height());

            // Draw trough...
            const QColor *trough(backgroundColors(cg));
            bool  noButtons(ROUND_NONE!=opts.round && SCROLLBAR_NONE==opts.scrollbarType);
            QRect s2(subpage), a2(addpage);

            if(noButtons)
            {
                // Increase clipping to allow trough to "bleed" into slider corners...
                a2.addCoords(-3, -3, 3, 3);
                s2.addCoords(-3, -3, 3, 3);
            }

            p->setClipRegion(QRegion(s2)+QRegion(addpage));
            drawLightBevel(p, sb->rect(), cg, sflags|Style_Down,
                           SCROLLBAR_NONE==opts.scrollbarType ? ROUNDED_ALL : ROUNDED_NONE,
                           trough[2], trough, true, true, WIDGET_SB_TROUGH);
            p->setClipping(false);

            if((controls&SC_ScrollBarSubLine) && subline.isValid())
            {
                bool enable=(!maxed && sb->value()!=sb->minValue());

                drawPrimitive(PE_ScrollBarSubLine, p, subline, cg, sflags |
                                 //(enable ? Style_Enabled : Style_Default) |
                                 (enable && hw && HOVER_SB_SUB==itsHover
                                       ? Style_MouseOver : Style_Default) |
                                 (enable && (!hw || HOVER_SB_SUB==itsHover || HOVER_NONE==itsHover)
                                   && SC_ScrollBarSubLine==active ? Style_Down : Style_Default));

                if (useThreeButtonScrollBar && subline2.isValid())
                    drawPrimitive(PE_ScrollBarSubLine, p, subline2, cg, sflags |
                                 //(enable ? Style_Enabled : Style_Default) |
                                 (enable && hw && HOVER_SB_SUB2==itsHover
                                     ? Style_MouseOver : Style_Default) |
                                 (enable && (!hw || HOVER_SB_SUB2==itsHover || HOVER_NONE==itsHover)
                                   && SC_ScrollBarSubLine==active ? Style_Down : Style_Default));
            }
            if((controls&SC_ScrollBarAddLine) && addline.isValid())
            {
                bool enable=(!maxed && sb->value()!=sb->maxValue());

                drawPrimitive(PE_ScrollBarAddLine, p, addline, cg, sflags |
                                 //(enable ? Style_Enabled : Style_Default) |
                                 (enable && hw && HOVER_SB_ADD==itsHover
                                     ? Style_MouseOver : Style_Default) |
                                 (enable && SC_ScrollBarAddLine==active
                                     ? Style_Down : Style_Default));
            }

            if((controls&SC_ScrollBarFirst) && first.isValid())
                drawPrimitive(PE_ScrollBarFirst, p, first, cg, sflags |
                                 //(maxed ? Style_Default : Style_Enabled) |
                                 (!maxed && SC_ScrollBarFirst==active ? Style_Down : Style_Default));

            if((controls&SC_ScrollBarLast) && last.isValid())
                drawPrimitive(PE_ScrollBarLast, p, last, cg, sflags |
                                 //(maxed ? Style_Default : Style_Enabled) |
                                 (!maxed && SC_ScrollBarLast==active ? Style_Down : Style_Default));

            if(((controls&SC_ScrollBarSlider) || noButtons) && slider.isValid())
            {
                // If "SC_ScrollBarSlider" wasn't pecified, then we only want to draw the portion
                // of the slider that overlaps with the tough. So, once again set the clipping
                // region...
                if(!(controls&SC_ScrollBarSlider))
                    p->setClipRegion(QRegion(s2)+QRegion(addpage));

                drawPrimitive(PE_ScrollBarSlider, p, slider, cg, sflags |
                                 //(maxed ? Style_Default : Style_Enabled) |
                                 (!maxed && hw && HOVER_SB_SLIDER==itsHover
                                     ? Style_MouseOver : Style_Default) |
                                 (!maxed && SC_ScrollBarSlider==active
                                     ? Style_Down : Style_Default));

                // ### perhaps this should not be able to accept focus if maxedOut?
                if(sb->hasFocus())
                    drawPrimitive(PE_FocusRect, p, QRect(slider.x()+2, slider.y()+2,
                                  slider.width()-5, slider.height()-5), cg, Style_Default);

                if(noButtons)
                {
                    bool notAtMin(!(maxed || sb->value()==sb->minValue())),
                         notAtMax(!(maxed || sb->value()==sb->maxValue()));
                    int  size=ROUND_FULL==opts.round ? 1 : 0;

                    if(notAtMin || notAtMax)
                    {
                        p->setPen(backgroundColors(cg)[5]);

                        if(horiz)
                        {
                            if(notAtMin)
                            {
                                p->drawLine(slider.x(), slider.y(),
                                            slider.x()+size, slider.y());
                                p->drawLine(slider.x(), slider.y()+slider.height()-1,
                                            slider.x()+size, slider.y()+slider.height()-1);
                            }
                            if(notAtMax)
                            {
                                p->drawLine(slider.x()+slider.width()-1, slider.y(),
                                            slider.x()+slider.width()-(1+size), slider.y());
                                p->drawLine(slider.x()+slider.width()-1,
                                            slider.y()+slider.height()-1,
                                            slider.x()+slider.width()-(1+size),
                                            slider.y()+slider.height()-1);
                            }
                        }
                        else
                        {
                            if(notAtMin)
                            {
                                p->drawLine(slider.x(), slider.y(),
                                            slider.x(), slider.y()+size);
                                p->drawLine(slider.x()+slider.width()-1, slider.y(),
                                            slider.x()+slider.width()-1, slider.y()+size);
                            }
                            if(notAtMax)
                            {
                                p->drawLine(slider.x(), slider.y()+slider.height()-1,
                                            slider.x(), slider.y()+slider.height()-(1+size));
                                p->drawLine(slider.x()+slider.width()-1,
                                            slider.y()+slider.height()-1,
                                            slider.x()+slider.width()-1,
                                            slider.y()+slider.height()-(1+size));
                            }
                        }
                    }
                }
                if(!(controls&SC_ScrollBarSlider))
                    p->setClipping(false);
            }
            break;
        }
        default:
            KStyle::drawComplexControl(control, p, widget, r, cg, flags, controls, active, data);
    }
}

QRect QtCurveStyle::querySubControlMetrics(ComplexControl control, const QWidget *widget,
                                           SubControl sc, const QStyleOption &data) const
{
    bool reverse(QApplication::reverseLayout());

    switch(control)
    {
        case CC_SpinWidget:
        {
            if(!widget)
                return QRect();

            int   fw(pixelMetric(PM_SpinBoxFrameWidth, 0));
            QSize bs;

            bs.setHeight(widget->height()>>1);
            if(bs.height()< 8)
                bs.setHeight(8);
            bs.setWidth(15);
            bs=bs.expandedTo(QApplication::globalStrut());

            int extra(bs.height()*2==widget->height()? 0 : 1),
                y(0), x(widget->width()-bs.width()),
                rx(x-fw*2);

            switch(sc)
            {
                case SC_SpinWidgetUp:
                    return QRect(x, y, bs.width(), bs.height());
                case SC_SpinWidgetDown:
                    return QRect(x, y+bs.height(), bs.width(), bs.height()+extra);
                case SC_SpinWidgetButtonField:
                    return QRect(x, y, bs.width(), widget->height()-2*fw);
                case SC_SpinWidgetEditField:
                    return QRect(fw, fw, rx, widget->height()-2*fw);
                case SC_SpinWidgetFrame:
                    return reverse
                              ? QRect(widget->x()+bs.width(), widget->y(),
                                      widget->width()-bs.width()-1, widget->height())
                              : QRect(widget->x(), widget->y(),
                                      widget->width()-bs.width(),widget->height());
                default:
                    break; // Remove compiler warnings...
            }
        }
        case CC_ComboBox:
        {
            QRect r(KStyle::querySubControlMetrics(control, widget, sc, data));

            if(SC_ComboBoxFrame==sc)
            {
                const QComboBox *cb(::qt_cast<const QComboBox *>(widget));

                if(cb && cb->editable())
                    r=QRect((r.x()+r.width()-1)-18, r.y(), 19, r.height());
            }
            else if (reverse && SC_ComboBoxEditField==sc)
                r.addCoords(2, 0, -2, 0);
            return r;
        }
        case CC_ScrollBar:
        {
            // Taken from kstyle.cpp, and modified so as to allow for no scrollbar butttons...
            bool  threeButtonScrollBar(SCROLLBAR_KDE==opts.scrollbarType),
                  platinumScrollBar(SCROLLBAR_PLATINUM==opts.scrollbarType),
                  nextScrollBar(SCROLLBAR_NEXT==opts.scrollbarType),
                  noButtons(SCROLLBAR_NONE==opts.scrollbarType);
            QRect ret;
            const QScrollBar *sb((const QScrollBar*)widget);
            bool  horizontal(sb->orientation() == Qt::Horizontal);
            int   sliderstart(sb->sliderStart()),
                  sbextent(pixelMetric(PM_ScrollBarExtent, widget)),
                  maxlen((horizontal ? sb->width() : sb->height())
                                - (noButtons ? 0 : (sbextent * (threeButtonScrollBar ? 3 : 2)))),
                  sliderlen;

            // calculate slider length
            if (sb->maxValue() != sb->minValue())
            {
                uint range = sb->maxValue() - sb->minValue();
                sliderlen = (sb->pageStep() * maxlen) / (range + sb->pageStep());

                int slidermin = pixelMetric( PM_ScrollBarSliderMin, widget );
                if ( sliderlen < slidermin || range > INT_MAX / 2 )
                    sliderlen = slidermin;
                if ( sliderlen > maxlen )
                    sliderlen = maxlen;
            }
            else
                sliderlen = maxlen;

            // Subcontrols
            switch(sc)
            {
                case SC_ScrollBarSubLine:
                    if(noButtons)
                        return QRect();

                    // top/left button
                    if (platinumScrollBar)
                        if (horizontal)
                            ret.setRect(sb->width() - 2 * sbextent, 0, sbextent, sbextent);
                        else
                            ret.setRect(0, sb->height() - 2 * sbextent, sbextent, sbextent);
                    else
                        ret.setRect(0, 0, sbextent, sbextent);
                    break;
                case SC_ScrollBarAddLine:
                    if(noButtons)
                        return QRect();

                    // bottom/right button
                    if (nextScrollBar)
                        if (horizontal)
                            ret.setRect(sbextent, 0, sbextent, sbextent);
                        else
                            ret.setRect(0, sbextent, sbextent, sbextent);
                    else
                        if (horizontal)
                            ret.setRect(sb->width() - sbextent, 0, sbextent, sbextent);
                        else
                            ret.setRect(0, sb->height() - sbextent, sbextent, sbextent);
                    break;
                case SC_ScrollBarSubPage:
                    // between top/left button and slider
                    if (platinumScrollBar)
                        if (horizontal)
                            ret.setRect(0, 0, sliderstart, sbextent);
                        else
                            ret.setRect(0, 0, sbextent, sliderstart);
                    else if (nextScrollBar)
                        if (horizontal)
                            ret.setRect(sbextent*2, 0, sliderstart-2*sbextent, sbextent);
                        else
                            ret.setRect(0, sbextent*2, sbextent, sliderstart-2*sbextent);
                    else
                        if (horizontal)
                            ret.setRect(noButtons ? 0 : sbextent, 0,
                                        noButtons ? sliderstart
                                                  : (sliderstart - sbextent), sbextent);
                        else
                            ret.setRect(0, noButtons ? 0 : sbextent, sbextent,
                                        noButtons ? sliderstart : (sliderstart - sbextent));
                    break;
                case SC_ScrollBarAddPage:
                {
                    // between bottom/right button and slider
                    int fudge;

                    if (platinumScrollBar)
                        fudge = 0;
                    else if (nextScrollBar)
                        fudge = 2*sbextent;
                    else if(noButtons)
                        fudge = 0;
                    else
                        fudge = sbextent;

                    if (horizontal)
                        ret.setRect(sliderstart + sliderlen, 0,
                                    maxlen - sliderstart - sliderlen + fudge, sbextent);
                    else
                        ret.setRect(0, sliderstart + sliderlen, sbextent,
                                    maxlen - sliderstart - sliderlen + fudge);
                    break;
                }
                case SC_ScrollBarGroove:
                    if(noButtons)
                    {
                        if (horizontal)
                            ret.setRect(0, 0, sb->width(), sb->height());
                        else
                            ret.setRect(0, 0, sb->width(), sb->height());
                    }
                    else
                    {
                        int multi = threeButtonScrollBar ? 3 : 2,
                            fudge;

                        if (platinumScrollBar)
                            fudge = 0;
                        else if (nextScrollBar)
                            fudge = 2*sbextent;
                        else
                            fudge = sbextent;

                        if (horizontal)
                            ret.setRect(fudge, 0, sb->width() - sbextent * multi, sb->height());
                        else
                            ret.setRect(0, fudge, sb->width(), sb->height() - sbextent * multi);
                    }
                    break;
                case SC_ScrollBarSlider:
                    if (horizontal)
                        ret.setRect(sliderstart, 0, sliderlen, sbextent);
                    else
                        ret.setRect(0, sliderstart, sbextent, sliderlen);
                    break;
                default:
                    ret = QCommonStyle::querySubControlMetrics(control, widget, sc, data);
                    break;
            }
            return ret;
        }
        default:
            break; // Remove compiler warnings...
    }

    return KStyle::querySubControlMetrics(control, widget, sc, data);
}

int QtCurveStyle::pixelMetric(PixelMetric metric, const QWidget *widget) const
{
    switch(metric)
    {
        case PM_MenuButtonIndicator:
            return 7;
        case PM_ButtonMargin:
            return 3;
#if QT_VERSION >= 0x030200
        case PM_TabBarTabShiftVertical:
        {
            const QTabBar *tb=::qt_cast<const QTabBar *>(widget);

            return QTabBar::RoundedAbove==tb->shape() || QTabBar::TriangularAbove==tb->shape()
                       ? 1 
                       : -1;
        }
        case PM_TabBarTabShiftHorizontal:
            return 0;
#endif
        case PM_ButtonShiftHorizontal:
        case PM_ButtonShiftVertical:
            return 1;
        case PM_ButtonDefaultIndicator:
            return 0;
        case PM_DefaultFrameWidth:
            return QTC_ROUNDED ? 2
                              : widget &&(::qt_cast<const QTabBar *>(widget) ||
                                          ::qt_cast<const QWidgetStack *>(widget) ||
                                          ::qt_cast<const QLineEdit *>(widget) ||
                                          ::qt_cast<const QComboBox *>(widget) ||
                                          ::qt_cast<const QTextEdit *>(widget) ||
                                          ::qt_cast<const QPopupMenu *>(widget))
                                   ? QTC_BORDERED_FRAME_WIDTH
                                   : QTC_DEF_FRAME_WIDTH;
        case PM_SpinBoxFrameWidth:
            return 2;
        case PM_IndicatorWidth:
        case PM_IndicatorHeight:
            return QTC_CHECK_SIZE;
        case PM_ExclusiveIndicatorWidth:
        case PM_ExclusiveIndicatorHeight:
            return QTC_RADIO_SIZE;
        case PM_TabBarTabOverlap:
            return 1;
        case PM_ProgressBarChunkWidth:
            return 4;
        case PM_DockWindowSeparatorExtent:
            return 4;
        case PM_DockWindowHandleExtent:
            return 10;
        case PM_SplitterWidth:
            return 6;
        case PM_ScrollBarSliderMin:
            return 16;
        case PM_SliderThickness:
            return 18;
        case PM_SliderControlThickness:
            return 16; // This equates to 14, as we draw the handle 2 pix smaller for focus rect...
        case PM_ScrollBarExtent:
            return APP_KPRESENTER==itsThemedApp ? 16 : 15;
        case PM_SliderLength:
            return 24;
        case PM_MaximumDragDistance:
            return -1;
        case PM_TabBarTabVSpace:
            return opts.highlightTab ? 11 : 9;
        default:
            return KStyle::pixelMetric(metric, widget);
    }
}

int QtCurveStyle::kPixelMetric(KStylePixelMetric kpm, const QWidget *widget) const
{
    switch(kpm)
    {
        case KPM_MenuItemSeparatorHeight:
            return 2;
        default:
            return KStyle::kPixelMetric(kpm, widget);
    }
}

QSize QtCurveStyle::sizeFromContents(ContentsType contents, const QWidget *widget,
                                     const QSize &contentsSize, const QStyleOption &data) const
{
    switch(contents)
    {
        case CT_PushButton:
        {
            const QPushButton *button(static_cast<const QPushButton *>(widget));

            if (button && !button->text().isEmpty())
            {
                const int constMinH(27);

                int margin(2*pixelMetric(PM_ButtonMargin, widget)),
                    mbi(button->isMenuButton() ? pixelMetric(PM_MenuButtonIndicator, widget) : 0),
                    w(contentsSize.width() + margin + mbi + 16),
                    h(contentsSize.height() + margin);

                if(button->text()=="...")
                    w+=24;
                else
                {
                    const int constMinW(84);

                    if(opts.embolden)
                        w+=6; // add room for bold font - Assume all buttons can be default!
                    if(w<constMinW)
                        w=constMinW;
                }

                return QSize(w, h < constMinH ? constMinH : h);
            }
            break;
        }
        case CT_ComboBox:
        {
            const int constMinH(24);

            QSize sz(KStyle::sizeFromContents(contents, widget, contentsSize, data));
            int   h(sz.height());

            return QSize(sz.width(), h<constMinH ? constMinH : h);
        }
        case CT_PopupMenuItem:
        {
            if (!widget || data.isDefault())
                break;

            const int        constMinH(opts.thinnerMenuItems ? 25 : 27);

            QMenuItem        *mi(data.menuItem());
            const QPopupMenu *popupmenu(static_cast<const QPopupMenu *>(widget));
            int              maxpmw(data.maxIconWidth()),
                             w(contentsSize.width()), h(contentsSize.height());

            if (mi->custom())
            {
                w = mi->custom()->sizeHint().width();
                h = mi->custom()->sizeHint().height();

                if (!mi->custom()->fullSpan() && h < constMinH)
                    h = constMinH;
            }
            else if(mi->widget())
                ;
            else if (mi->isSeparator())
            {
                w = 10;
                h = opts.thinnerMenuItems ? 6 : 8;
            }
            else
            {
                // check is at least 16x16
                if (h < 16)
                    h = 16;
                if (mi->pixmap())
                    h = QMAX(h, mi->pixmap()->height());
                else if (!mi->text().isNull())
                    h = QMAX(h, popupmenu->fontMetrics().height() + 2);
                if (mi->iconSet()!= 0)
                    h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal).height());
                h+=(opts.thinnerMenuItems ? 2 : 4);
            }

            // check | 4 pixels | item | 8 pixels | accel | 4 pixels | check

            // check is at least 16x16
            maxpmw=QMAX(maxpmw, 16);
            w += (maxpmw * 2) + 8;

            if (! mi->text().isNull() && mi->text().find('\t') >= 0)
                w += 8;

            return QSize(w, h);
        }
        case CT_SpinBox:
        {
            QSize size(KStyle::sizeFromContents(contents, widget, contentsSize, data));

            if(!(size.height()%2))
                size.setHeight(size.height()+1);
            return size;
        }
        case CT_ToolButton:
            if(widget->parent() && ::qt_cast<QToolBar *>(widget->parent()))
                return QSize(contentsSize.width()+8, contentsSize.height()+8);
        default:
            break;  // Remove compiler warnings...
    }

    return KStyle::sizeFromContents(contents, widget, contentsSize, data);
}

int QtCurveStyle::styleHint(StyleHint stylehint, const QWidget *widget, const QStyleOption &option,
                            QStyleHintReturn *returnData) const
{
    switch(stylehint)
    {
        case SH_EtchDisabledText:
        case SH_Slider_SnapToValue:
        case SH_PrintDialog_RightAlignButtons:
        case SH_FontDialog_SelectAssociatedText:
        case SH_MenuBar_AltKeyNavigation:
        case SH_PopupMenu_MouseTracking:
        case SH_PopupMenu_SpaceActivatesItem:
        case SH_ComboBox_ListMouseTracking:
        case SH_ScrollBar_MiddleClickAbsolutePosition:
            return 1;
        case SH_MainWindow_SpaceBelowMenuBar:
            return 0;
        case SH_PopupMenu_AllowActiveAndDisabled:
            return 0;
        case SH_MenuBar_MouseTracking:
            return opts.menubarMouseOver ? 1 : 0;
        default:
            return KStyle::styleHint(stylehint, widget, option, returnData);
    }
}

void QtCurveStyle::drawMenuItem(QPainter *p, const QRect &r, bool horiz, const QColorGroup &cg,
                                ControlElement control, int round, bool border,
                                const QColor &bgnd) const
{
    switch(opts.appearance)
    {
        default:
        {
            int flags(QStyle::Style_Raised);
            QRect r2(r);

            if(!border)
                r2.addCoords(-1, -1, 1, 1);

            if(horiz)
                flags|=Style_Horizontal;

            drawLightBevel(bgnd, p, r2, cg, flags, round, itsMenuitemCols[ORIGINAL_SHADE],
                           itsMenuitemCols, border, !(CE_MenuBarItem==control && QTC_ROUNDED &&
                           IS_GLASS(opts.menubarAppearance)), WIDGET_MENU_ITEM);
            break;
        }
        case APPEARANCE_FLAT:
        case APPEARANCE_RAISED:
            p->fillRect(r, cg.highlight());
    }
}

void QtCurveStyle::drawProgress(QPainter *p, const QRect &r, const QColorGroup &cg, SFlags flags,
                                int round, const QWidget *widget) const
{
    if(r.width()<1)
        return;

    QRegion outer(r);

    if(r.width()>4)
    {
        int animShift=-PROGRESS_CHUNK_WIDTH;

        if (opts.animatedProgress)
        {
            // find the animation Offset for the current Widget
            QWidget                          *nonConstWidget(const_cast<QWidget*>(widget));
            QMapConstIterator<QWidget*, int> it(itsProgAnimWidgets.find(nonConstWidget));

            if (it!=itsProgAnimWidgets.end())
                animShift += it.data();
        }

        if(opts.stripedProgress)
            for(int offset=0; offset<(r.width()+PROGRESS_CHUNK_WIDTH);
                offset+=(PROGRESS_CHUNK_WIDTH*2))
            {
                QRect   r2(r.x()+offset+animShift, r.y(), PROGRESS_CHUNK_WIDTH, r.height());
                QRegion inner(r2);

                outer=outer.eor(inner);
            }
    }

    switch(opts.appearance)
    {
        default:
        {
            flags|=QStyle::Style_Raised|Style_Horizontal;

            drawLightBevel(cg.background(), p, r, cg, flags, round, itsMenuitemCols[ORIGINAL_SHADE],
                           itsMenuitemCols, true, true, WIDGET_MENU_ITEM);

            if(r.width()>4 && opts.stripedProgress)
            {
                p->setClipRegion(outer);
                drawLightBevel(cg.background(), p, r, cg, flags, round, itsMenuitemCols[1],
                               itsMenuitemCols, true, true, WIDGET_MENU_ITEM);
                p->setClipping(false);
            }

            if(QTC_ROUNDED && ROUNDED_ALL!=round)
            {
                p->setPen(midColor(cg.background(), itsMenuitemCols[5]));
                if(!(round&CORNER_TL))
                    p->drawPoint(r.x(), r.y());
                if(!(round&CORNER_BL))
                    p->drawPoint(r.x(), r.y()+r.height()-1);
                if(!(round&CORNER_TR))
                    p->drawPoint(r.x()+r.width()-1, r.y());
                if(!(round&CORNER_BR))
                    p->drawPoint(r.x()+r.width()-1, r.y()+r.height()-1);
            }
            break;
        }
        case APPEARANCE_FLAT:
        case APPEARANCE_RAISED:
            p->fillRect(r, cg.highlight());
            if(r.width()>4 && opts.stripedProgress)
            {
                p->setClipRegion(outer);
                p->fillRect(r, itsMenuitemCols[1]);
                p->setClipping(false);
            }
    }
}

void QtCurveStyle::drawBevelGradient(const QColor &base, bool increase, QPainter *p,
                                     const QRect &origRect, bool horiz, double shadeTop,
                                     double shadeBot, bool sel, EAppearance bevApp, EWidget w) const
{
    if(IS_FLAT(bevApp))
        p->fillRect(origRect, base);
    else
    {
        EAppearance app(APPEARANCE_BEVELLED!=bevApp || WIDGET_BUTTON(w) || WIDGET_LISTVIEW_HEADER==w
                            ? bevApp
                            : APPEARANCE_GRADIENT);

        QRect   r(0, 0, horiz ? QTC_PIXMAP_DIMENSION : origRect.width(),
                        horiz ? origRect.height() : QTC_PIXMAP_DIMENSION);
        QString key(createKey(horiz ? r.height() : r.width(), base.rgb(), horiz, increase,
                              app2App(app, sel), w, shadeTop, shadeBot));
        QPixmap *pix(itsPixmapCache.find(key));

        if(!pix)
        {
            pix=new QPixmap(r.width(), r.height());

            QPainter pixPainter(pix);

            if(!sel && IS_GLASS(app))
            {
                if(WIDGET_TAB_BOT==w)
                {
                    double t(shadeTop);
                    shadeTop=shadeBot;
                    shadeBot=t;
                }

                double shadeTopA(WIDGET_TAB_BOT==w ? 1.0 : shadeTop*SHADE_GLASS_TOP_A(app, w)),
                       shadeTopB(WIDGET_TAB_BOT==w ? 1.0 : shadeTop*SHADE_GLASS_TOP_B(app, w)),
                       shadeBotA(WIDGET_TAB_TOP==w ? 1.0 : shadeBot*SHADE_GLASS_BOT_A(app)),
                       shadeBotB(WIDGET_TAB_TOP==w ? 1.0 : shadeBot*SHADE_GLASS_BOT_B(app));

                QColor topA, topB, botA, botB;
                QRect  r1(r), r2(r), r3(r);

                shade(base, &topA, shadeTopA);
                shade(base, &topB, shadeTopB);
                shade(base, &botA, shadeBotA);
                shade(base, &botB, shadeBotB);

                if(APPEARANCE_SHINY_GLASS==app)
                {
                    int shadeBorder=(int)(((horiz ? r.height() : r.width())*GLASS_BORDER)+0.5);

                    if(horiz)
                    {
                        r1.setHeight(shadeBorder);
                        r3.setHeight(shadeBorder);
                        r2.setHeight(r.height()-(r1.height()+r3.height()));
                        r2.moveTop(r.y()+r1.height());
                        r3.moveTop(r.y()+r1.height()+r2.height());
                    }
                    else
                    {
                        r1.setWidth(shadeBorder);
                        r3.setWidth(shadeBorder);
                        r2.setWidth(r.width()-(r1.width()+r3.width()));
                        r2.moveLeft(r.x()+r1.width());
                        r3.moveLeft(r.x()+r1.width()+r2.width());
                    }
                    drawGradient(topA, topB, true, &pixPainter, r1, horiz);
                    pixPainter.fillRect(r2, WIDGET_TAB_BOT==w ? topB : botA);
                    drawGradient(botA, botB, true, &pixPainter, r3, horiz);
                }
                else
                {
                    if(horiz)
                    {
                        r1.setHeight(r1.height()/2);
                        r2.setY(r2.y()+r1.height());
                    }
                    else
                    {
                        r1.setWidth(r1.width()/2);
                        r2.setX(r2.x()+r1.width());
                    }
                    drawGradient(topA, topB, increase, &pixPainter, r1, horiz);
                    drawGradient(botA, botB, increase, &pixPainter, r2, horiz);
                }
            }
            else if(!sel && APPEARANCE_BEVELLED==app &&
                    ((horiz ? r.height()
                            : r.width()) > (((WIDGET_BUTTON(w) ? 2 : 1)*BEVEL_BORDER(w))+4)))
            {
                if(WIDGET_LISTVIEW_HEADER==w)
                {
                    QColor bot;
                    QRect  r1(r), r2(r);

                    if(horiz)
                    {
                        r2.setHeight(BEVEL_BORDER(w));
                        r1.setHeight(r.height()-r2.height());
                        r2.moveTop(r.y()+r1.height());
                    }
                    else
                    {
                        r2.setWidth(BEVEL_BORDER(w));
                        r1.setWidth(r.width()-r2.width());
                        r2.moveLeft(r.x()+r1.width());
                    }
                    shade(base, &bot, SHADE_BEVEL_BOT(w));
                    pixPainter.fillRect(r1, base);
                    drawGradient(base, bot, true, &pixPainter, r2, horiz);
                }
                else
                {
                    QColor bot, midTop, midBot, top;
                    QRect  r1(r), r2(r), r3(r);

                    if(horiz)
                    {
                        r1.setHeight(BEVEL_BORDER(w));
                        r3.setHeight(BEVEL_BORDER(w));
                        r2.setHeight(r.height()-(r1.height()+r3.height()));
                        r2.moveTop(r.y()+r1.height());
                        r3.moveTop(r.y()+r1.height()+r2.height());
                    }
                    else
                    {
                        r1.setWidth(BEVEL_BORDER(w));
                        r3.setWidth(BEVEL_BORDER(w));
                        r2.setWidth(r.width()-(r1.width()+r3.width()));
                        r2.moveLeft(r.x()+r1.width());
                        r3.moveLeft(r.x()+r1.width()+r2.width());
                    }

                    shade(base, &top, SHADE_BEVEL_TOP);
                    shade(base, &midTop, SHADE_BEVEL_MID_TOP);
                    shade(base, &midBot, SHADE_BEVEL_MID_BOT);
                    shade(base, &bot, SHADE_BEVEL_BOT(w));
                    drawGradient(top, midTop, true, &pixPainter, r1, horiz);
                    drawGradient(midTop, midBot, true, &pixPainter, r2, horiz);
                    drawGradient(midBot, bot, true, &pixPainter, r3, horiz);
                }
            }
            else
            {
                QColor top, bot;

                if(equal(1.0, shadeTop))
                    top=base;
                else
                    shade(base, &top, shadeTop);
                if(equal(1.0, shadeBot))
                    bot=base;
                else
                    shade(base, &bot, shadeBot);

                drawGradient(top, bot, sel || APPEARANCE_INVERTED!=app ? increase : !increase,
                             &pixPainter, r, horiz);
            }
            pixPainter.end();
            itsPixmapCache.insert(key, pix, pix->width()*pix->height()*(pix->depth()/8));
        }
        p->drawTiledPixmap(origRect, *pix);
    }
}

void QtCurveStyle::drawGradient(const QColor &top, const QColor &bot, bool increase,
                                QPainter *p, QRect const &r, bool horiz) const
{
    if(r.width()>0 && r.height()>0)
    {
        if(top==bot)
            p->fillRect(r, top);
        else
        {
            int    i,
                   s=horiz ? r.top() : r.left(),
                   e=horiz ? r.bottom() : r.right();
            double amt=horiz ? r.height() : r.width(),
                   dr=((double)(bot.red()-top.red()))/amt,
                   dg=((double)(bot.green()-top.green()))/amt,
                   db=((double)(bot.blue()-top.blue()))/amt,
                   rc=0, gc=0, bc=0;

            if(increase)
                for(i=s; i<=e; i++)
                {
                    p->setPen(QColor(limit(top.red()+rc), limit(top.green()+gc),
                                     limit(top.blue()+bc)));
                    if(horiz)
                        p->drawLine(r.left(), i, r.right(), i);
                    else
                        p->drawLine(i, r.top(), i, r.bottom());

                    rc+=dr; gc+=dg; bc+=db;
                }
            else
                for(i=e; i>=s; i--)
                {
                    p->setPen(QColor(limit(top.red()+rc), limit(top.green()+gc),
                                     limit(top.blue()+bc)));
                    if(horiz)
                        p->drawLine(r.left(), i, r.right(), i);
                    else
                        p->drawLine(i, r.top(), i, r.bottom());
                    rc+=dr; gc+=dg; bc+=db;
                }
        }
    }
}

void QtCurveStyle::drawSliderHandle(QPainter *p, const QRect &orig, const QColorGroup &cg,
                                    SFlags flags, bool scrollbar) const
{
    ELine        thumbs(scrollbar ? opts.sbSliderThumbs : opts.sliderThumbs);
    int          min(MIN_SLIDER_SIZE(thumbs));
    const QColor *use(sliderColors(cg, scrollbar, flags));
    QRect        r(orig);
    EShade       shade(scrollbar ? opts.shadeSbSliders : opts.shadeSliders);

    if(flags&Style_Down)
        flags-=Style_Down;
    if(r.width()>r.height())
        flags|=Style_Horizontal;
    flags|=Style_Raised;

    if(!scrollbar)
        if(flags&Style_Horizontal)
            r.addCoords(0, 1, 0, -1);
        else
            r.addCoords(1, 0, -1, 0);

    drawLightBevel(p, r, cg, flags, scrollbar && SCROLLBAR_NONE!=opts.scrollbarType
                                        ? ROUNDED_NONE : ROUNDED_ALL,
                   getFill(flags, use), use,
                   true, !scrollbar, scrollbar ? WIDGET_SB_SLIDER : WIDGET_SLIDER);

    const QColor *markers(opts.coloredMouseOver && flags&Style_MouseOver
                              ? SHADE_NONE==shade ? itsMouseOverCols : itsBackgroundCols
                              : use);

    if(LINE_NONE!=thumbs && ((flags & Style_Horizontal && r.width()>=min)|| r.height()>=min))
        switch(thumbs)
        {
            case LINE_SUNKEN:
                drawLines(p, r, !(flags & Style_Horizontal), 4, 3, markers, 0, true, 3);
                break;
            case LINE_RAISED:
                drawLines(p, r, !(flags & Style_Horizontal), 4, 3, markers, 0, false, 3);
                break;
            case LINE_DOTS:
            case LINE_LARGE_DOTS:
            default:
                drawDots(p, r, !(flags & Style_Horizontal), 5, scrollbar ? 2 : 3, markers, 0, 5,
                         LINE_LARGE_DOTS==thumbs);
        }
}

void QtCurveStyle::drawSliderGroove(QPainter *p, const QRect &r, const QColorGroup &cg,
                                    SFlags flags, const QWidget *widget) const
{
    const QSlider *sliderWidget((const QSlider *)widget);
    QRect         groove(r);
    bool          horiz(Qt::Horizontal==sliderWidget->orientation()),
                  reverse(QApplication::reverseLayout());
    const QColor  &usedCol=itsSliderCols ? itsSliderCols[ORIGINAL_SHADE] : itsMenuitemCols[2];

    if(flags & Style_HasFocus)
        drawPrimitive(PE_FocusRect, p, groove, QColorGroup());

    if(horiz)
    {
        int dh=(groove.height()-5)>>1;
        groove.addCoords(0, dh, 0, -dh);
        flags|=Style_Horizontal;
    }
    else
    {
        int dw=(groove.width()-5)>>1;
        groove.addCoords(dw, 0, -dw, 0);
    }

    QRect rest(groove);

    drawLightBevel(p, rest, cg, flags|Style_Down, ROUNDED_ALL, itsBackgroundCols[2],
                   itsBackgroundCols, true, true, WIDGET_SLIDER_TROUGH);

    if(opts.fillSlider &&
#ifdef QTC_SLIDER_SHADING
       itsActive &&
#endif
       sliderWidget->maxValue()!=sliderWidget->minValue())
    {
        QRect used(rest);
        int   pos((int)(((double)(horiz ? groove.width() : groove.height()) /
                                     (sliderWidget->maxValue()-sliderWidget->minValue()))  *
                                 (sliderWidget->value() - sliderWidget->minValue())));

        if(horiz)
            if(reverse)
                used.addCoords(groove.width()-pos, 0, 0, 0);
            else
                used.addCoords(0, 0, -(groove.width()-pos), 0);
        else
            used.addCoords(0, pos, 0, 0);
        if(used.height()>0 && used.width()>0)
            drawLightBevel(p, used, cg, flags|Style_Down, ROUNDED_ALL, usedCol, NULL,
                           true, true, WIDGET_SLIDER_TROUGH);
    }
}

void QtCurveStyle::drawMenuOrToolBarBackground(QPainter *p, const QRect &r, const QColorGroup &cg,
                                               bool menu, bool horiz) const
{
    EAppearance app(menu ? opts.menubarAppearance : opts.toolbarAppearance);
    QColor      color(menu && itsActive ? itsMenubarCols[ORIGINAL_SHADE] : cg.background());
    double      from(0.0), to(0.0);

    switch(app)
    {
        default:
        case APPEARANCE_GRADIENT:
            from=SHADE_MENU_LIGHT;
            to=SHADE_MENU_DARK;
            break;
        case APPEARANCE_FLAT:
        case APPEARANCE_RAISED:
            break;
        case APPEARANCE_SHINY_GLASS:
        case APPEARANCE_DULL_GLASS:
            from=SHADE_BEVEL_GRAD_LIGHT;
            to=SHADE_BEVEL_GRAD_DARK;
    }

    drawBevelGradient(color, true, p, r, horiz, from, to, false, app);
}

void QtCurveStyle::drawHandleMarkers(QPainter *p, const QRect &r, SFlags flags, bool tb,
                                     ELine handles) const
{
    if(r.width()<2 || r.height()<2)
        return;

    if(!(flags&Style_MouseOver) && p->device()==itsHoverWidget)
        flags|=Style_MouseOver;

    const QColor *border(borderColors(flags, itsBackgroundCols));

    switch(handles)
    {
        case LINE_DOTS:
        case LINE_LARGE_DOTS:
            drawDots(p, r, !(flags & Style_Horizontal), 2,
                     APP_KICKER==itsThemedApp ? 1 : tb ? 5 : 3, border,
                     APP_KICKER==itsThemedApp ? 1 : tb ? -2 : 0, 5, LINE_LARGE_DOTS==handles);
            break;
        case LINE_DASHES:
            if(flags&Style_Horizontal)
            {
                QRect r1(r.x()+(tb ? 2 : (r.width()-6)/2), r.y(), 5, r.height());

                drawLines(p, r1, true, (r.height()-8)/3,
                          tb ? 0 : (r.width()-5)/2, border, 0, true, 5);
            }
            else
            {
                QRect r1(r.x(), r.y()+(tb ? 2 : (r.height()-6)/2), r.width(), 5);

                drawLines(p, r1, false, (r.width()-8)/3,
                          tb ? 0 : (r.height()-5)/2, border, 0, true, 5);
            }
            break;
        default:
            drawLines(p, r, !(flags & Style_Horizontal), 2,
                      APP_KICKER==itsThemedApp ? 1 : tb ? 4 : 2, border,
                      APP_KICKER==itsThemedApp ? 1 : tb ? -2 : 0,
                      LINE_SUNKEN==handles, 3);
    }
}

void QtCurveStyle::shadeColors(const QColor &base, QColor *vals) const
{
    QTC_SHADES

    for(int i=0; i<NUM_STD_SHADES; ++i)
        shade(base, &vals[i], QTC_SHADE(opts.contrast, i));
    shade(base, &vals[SHADE_ORIG_HIGHLIGHT], opts.highlightFactor);
    shade(vals[4], &vals[SHADE_4_HIGHLIGHT], opts.highlightFactor);
    shade(vals[2], &vals[SHADE_2_HIGHLIGHT], opts.highlightFactor);
    vals[ORIGINAL_SHADE]=base;
}

const QColor * QtCurveStyle::buttonColors(const QColorGroup &cg) const
{
    if(cg.button()!=itsButtonCols[ORIGINAL_SHADE])
    {
        shadeColors(cg.button(), itsColoredButtonCols);
        return itsColoredButtonCols;
    }

    return itsButtonCols;
}

const QColor * QtCurveStyle::sliderColors(const QColorGroup &cg, bool sb, SFlags flags) const
{
    return (flags&Style_Enabled)
#ifdef QTC_SLIDER_SHADING
                 && itsActive
#endif
                ? sb
                    ? SHADE_NONE!=opts.shadeSbSliders && cg.button()==itsButtonCols[ORIGINAL_SHADE]
                        ? itsSbSliderCols
                        : buttonColors(cg)
                    : SHADE_NONE!=opts.shadeSliders && cg.button()==itsButtonCols[ORIGINAL_SHADE]
                        ? itsSliderCols
                        : buttonColors(cg)
                : itsBackgroundCols;
}

const QColor * QtCurveStyle::backgroundColors(const QColor &c) const
{
    if(c!=itsBackgroundCols[ORIGINAL_SHADE])
    {
        shadeColors(c, itsColoredBackgroundCols);
        return itsColoredBackgroundCols;
    }

    return itsBackgroundCols;
}

const QColor * QtCurveStyle::borderColors(SFlags flags, const QColor *use) const
{
    return itsMouseOverCols && opts.coloredMouseOver && flags&QStyle::Style_MouseOver
               ? itsMouseOverCols : use;
}

const QColor * QtCurveStyle::getSidebarButtons() const
{
    if(!itsSidebarButtonsCols)
    {
        if(SHADE_SELECTED==opts.shadeSliders)
            itsSidebarButtonsCols=itsSliderCols;
        else if(SHADE_SELECTED==opts.shadeSbSliders)
            itsSidebarButtonsCols=itsSbSliderCols;
        else if(IND_COLORED==opts.defBtnIndicator)
            itsSidebarButtonsCols=itsDefBtnCols;
        else
        {
            itsSidebarButtonsCols=new QColor [TOTAL_SHADES+1];
            shadeColors(midColor(itsMenuitemCols[ORIGINAL_SHADE], itsButtonCols[ORIGINAL_SHADE]),
                        itsSidebarButtonsCols);
        }
    }

    return itsSidebarButtonsCols;
}

void QtCurveStyle::setMenuColors(const QColorGroup &cg)
{
    switch(opts.shadeMenubars)
    {
        case SHADE_NONE:
            memcpy(itsMenubarCols, itsBackgroundCols, sizeof(QColor)*(TOTAL_SHADES+1));
            break;
        case SHADE_SELECTED:
            shadeColors(shade(itsMenuitemCols[ORIGINAL_SHADE], MENUBAR_GLASS_SELECTED_DARK_FACTOR),
                        itsMenubarCols);
            break;
        case SHADE_CUSTOM:
            shadeColors(opts.customMenubarsColor, itsMenubarCols);
            break;
        case SHADE_DARKEN:
            shadeColors(shade(cg.background(), MENUBAR_DARK_FACTOR), itsMenubarCols);
    }
}

bool QtCurveStyle::redrawHoverWidget(const QPoint &pos)
{
    if(!itsHoverWidget || !itsHoverWidget->isShown() || !itsHoverWidget->isVisible())
        return false;

#if QT_VERSION >= 0x030200
    //
    // Qt>=3.2 sets the sensitive part of a check/radio to the image + label -> anything else
    // is not sensitive. But, the widget can ocupy a larger area - and this whole area will
    // react to mouse over. This needs to be counteracted so that it looks as if only the
    // sensitive area mouse-overs...
    QRadioButton *rb(::qt_cast<QRadioButton *>(itsHoverWidget));

    if(rb)
    {
        QRect rect(0, 0,
                   visualRect(subRect(SR_RadioButtonFocusRect, rb), rb).width()+
                   pixelMetric(PM_ExclusiveIndicatorWidth)+4, itsHoverWidget->height());

        itsHover=rect.contains(pos) ? HOVER_RADIO : HOVER_NONE;
        return (HOVER_NONE!=itsHover && !rect.contains(itsOldPos)) ||
               (HOVER_NONE==itsHover && rect.contains(itsOldPos));
    }
    else
    {
        QCheckBox *cb(::qt_cast<QCheckBox *>(itsHoverWidget));

        if(cb)
        {
            QRect rect(0, 0,
                       visualRect(subRect(SR_CheckBoxFocusRect, cb), cb).width()+
                       pixelMetric(PM_IndicatorWidth)+4, itsHoverWidget->height());

            itsHover=rect.contains(pos) ? HOVER_CHECK : HOVER_NONE;
            return (HOVER_NONE!=itsHover && !rect.contains(itsOldPos)) || (HOVER_NONE==itsHover &&
                                                                     rect.contains(itsOldPos));
        }
        else
        {
#endif
            QScrollBar *sb(::qt_cast<QScrollBar *>(itsHoverWidget));

            if(sb)  // So, are we over add button, sub button, slider, or none?
            {
                bool  useThreeButtonScrollBar(SCROLLBAR_KDE==opts.scrollbarType);
                QRect subline(querySubControlMetrics(CC_ScrollBar, itsHoverWidget,
                                                     SC_ScrollBarSubLine)),
                      addline(querySubControlMetrics(CC_ScrollBar, itsHoverWidget,
                                                     SC_ScrollBarAddLine)),
                      slider(querySubControlMetrics(CC_ScrollBar, itsHoverWidget,
                                                    SC_ScrollBarSlider)),
                      subline2(addline);

                if (useThreeButtonScrollBar)
                    if (Qt::Horizontal==sb->orientation())
                    subline2.moveBy(-addline.width(), 0);
                else
                    subline2.moveBy(0, -addline.height());

                if(slider.contains(pos))
                    itsHover=HOVER_SB_SLIDER;
                else if(subline.contains(pos))
                    itsHover=HOVER_SB_SUB;
                else if(addline.contains(pos))
                    itsHover=HOVER_SB_ADD;
                else if(subline2.contains(pos))
                    itsHover=HOVER_SB_SUB2;
                else
                    itsHover=HOVER_NONE;

                return (HOVER_SB_SLIDER==itsHover && !slider.contains(itsOldPos)) ||
                       (HOVER_SB_SLIDER!=itsHover && slider.contains(itsOldPos)) ||
                       (HOVER_SB_SUB==itsHover && !subline.contains(itsOldPos)) ||
                       (HOVER_SB_SUB!=itsHover && subline.contains(itsOldPos)) ||

                       (useThreeButtonScrollBar &&
                            (HOVER_SB_SUB2==itsHover && !subline2.contains(itsOldPos)) ||
                            (HOVER_SB_SUB2!=itsHover && subline2.contains(itsOldPos)))  ||

                       (HOVER_SB_ADD==itsHover && !addline.contains(itsOldPos)) ||
                       (HOVER_SB_ADD!=itsHover && addline.contains(itsOldPos));
            }
            else
            {
#if KDE_VERSION >= 0x30400 && KDE_VERSION < 0x30500
                QToolButton *tb(::qt_cast<QToolButton *>(itsHoverWidget));

                if(tb)
                {
                    itsHover=APP_KICKER==itsThemedApp ? HOVER_KICKER : HOVER_NONE;
                    return HOVER_KICKER==itsHover;
                }
                else
                {
#endif
                    QHeader *hd(::qt_cast<QHeader *>(itsHoverWidget));

                    if(hd)
                    {
                        // Hmm... this ones tricky, as there's only 1 widget - but it has different
                        // sections...
                        // and the ones that aren't clickable should not highlight on mouse over!

                        QRect rect(0, 0, itsHoverWidget->width(), itsHoverWidget->height());
                        int  s(0);
                        bool redraw(false);

                        itsHover=rect.contains(pos) ? HOVER_HEADER : HOVER_NONE;
                        itsHoverSect=QTC_NO_SECT;

                        for(s=0; s<hd->count() && (QTC_NO_SECT==itsHoverSect || !redraw); ++s)
                        {
                            QRect r(hd->sectionRect(s));
                            bool  hasNew(r.contains(pos));

                            if(hasNew)
                                itsHoverSect=s;

                            if(!redraw)
                            {
                                bool hasOld(r.contains(itsOldPos));

                                if((hasNew && !hasOld) || (!hasNew && hasOld))
                                    redraw=true;
                            }
                        }
                        return redraw;
                    }
                    else
                    {
                        QSpinWidget *sw(::qt_cast<QSpinWidget *>(itsHoverWidget));

                        if(sw)  // So, are we over up or down?
                        {
                            QRect up(querySubControlMetrics(CC_SpinWidget, itsHoverWidget,
                                                            SC_SpinWidgetUp)),
                                  down(querySubControlMetrics(CC_SpinWidget, itsHoverWidget,
                                                              SC_SpinWidgetDown));

                            if(up.contains(pos))
                                itsHover=HOVER_SW_UP;
                            else if(down.contains(pos))
                                itsHover=HOVER_SW_DOWN;
                            else
                                itsHover=HOVER_NONE;

                            return (HOVER_SW_UP==itsHover && !up.contains(itsOldPos)) ||
                                   (HOVER_SW_UP!=itsHover && up.contains(itsOldPos)) ||
                                   (HOVER_SW_DOWN==itsHover && !down.contains(itsOldPos)) ||
                                   (HOVER_SW_DOWN!=itsHover && down.contains(itsOldPos));
                        }
                        else
                        {
                            QTabBar *tabbar(::qt_cast<QTabBar *>(itsHoverWidget));

                            if(tabbar)
                            {
                                bool redraw(false);
                                QTab *tab(tabbar->selectTab(pos));
                                int  tabIndex(tab ? tabbar->indexOf(tab->identifier()) : -1),
                                     selectedTab(tabbar->currentTab());

                                redraw=tab!=itsHoverTab && tabIndex!=selectedTab;
                                itsHoverTab=tab;

                                return redraw;
                            }
                            else
                                return itsOldPos==QPoint(-1, -1);
                        }
#if KDE_VERSION >= 0x30400 && KDE_VERSION < 0x30500
                    }
#endif
#if KDE_VERSION >= 0x30400
                }
#endif
            }
#if QT_VERSION >= 0x030200
        }
    }
#endif

    return false;
}

const QColor & QtCurveStyle::getFill(SFlags flags, const QColor *use) const
{
    return !(flags&QStyle::Style_Enabled)
               ? use[1]
               : flags&QStyle::Style_Down
                   ? use[4]
                   : flags&QStyle::Style_MouseOver
                         ? flags&(QStyle::Style_On | QStyle::Style_Sunken)
                               ? use[SHADE_4_HIGHLIGHT]
                               : use[SHADE_ORIG_HIGHLIGHT]
                         : flags&(QStyle::Style_On | QStyle::Style_Sunken)
                               ? use[4]
                               : use[ORIGINAL_SHADE];
}

const QColor & QtCurveStyle::getTabFill(bool current, bool highlight, const QColor *use) const
{
    return current
            ? use[ORIGINAL_SHADE]
            : highlight
                ? use[SHADE_2_HIGHLIGHT]
                : use[2];
}

QPixmap * QtCurveStyle::getPixelPixmap(const QColor col) const
{
    QRgb    rgb(col.rgb());
    QString key(createKey(rgb));

    QPixmap *pix=itsPixmapCache.find(key);

    if(!pix)
    {
        static const int constAlpha=110;

        QImage img(1, 1, 32);

        img.setAlphaBuffer(true);
        img.setPixel(0, 0, qRgba(qRed(rgb), qGreen(rgb), qBlue(rgb), constAlpha));
        pix=new QPixmap(img);
        itsPixmapCache.insert(key, pix, pix->depth()/8);
    }

    return pix;
}

QPixmap * QtCurveStyle::getPixmap(const QColor col, EPixmap p, double shade) const
{
    QRgb    rgb(col.rgb());
    QString key(createKey(rgb, p));
    QPixmap *pix=itsPixmapCache.find(key);

    if(!pix)
    {
        pix=new QPixmap();

        QImage img;

        switch(p)
        {
            case PIX_RADIO_BORDER:
                img.loadFromData(qembed_findData("radio_frame.png"));
                break;
            case PIX_RADIO_LIGHT:
                img.loadFromData(qembed_findData("radio_light.png"));
                break;
            case PIX_RADIO_ON:
                img.loadFromData(qembed_findData("radio_on.png"));
                break;
            case PIX_CHECK:
                img.loadFromData(qembed_findData("check_on.png"));
        }

        if (img.depth()<32)
            img=img.convertDepth(32);

        adjustPix(img.bits(), 4, img.width(), img.height(), img.bytesPerLine(), col.red(),
                  col.green(), col.blue(), shade);
        pix->convertFromImage(img);
        itsPixmapCache.insert(key, pix, pix->depth()/8);
    }

    return pix;
}

void QtCurveStyle::setSbType()
{
    switch(opts.scrollbarType)
    {
        case SCROLLBAR_KDE:
            this->setScrollBarType(KStyle::ThreeButtonScrollBar);
            break;
        default:
        case SCROLLBAR_WINDOWS:
            this->setScrollBarType(KStyle::WindowsStyleScrollBar);
            break;
        case SCROLLBAR_PLATINUM:
            this->setScrollBarType(KStyle::PlatinumStyleScrollBar);
            break;
        case SCROLLBAR_NEXT:
            this->setScrollBarType(KStyle::NextStyleScrollBar);
            break;
    }
}

void QtCurveStyle::modifyInactivePal(QWidget *widget, bool check) const
{
    if(!opts.inactiveHighlight)
    {
        QPalette pal(widget->palette());

        if(!check || pal.color(QPalette::Inactive, QColorGroup::Highlight)
                        !=itsInactiveHighlightCol)
        {
            pal.setColor(QPalette::Inactive, QColorGroup::Highlight,
                         itsInactiveHighlightCol);
            pal.setColor(QPalette::Inactive, QColorGroup::HighlightedText,
                         pal.color(QPalette::Active, QColorGroup::Text));
            widget->setPalette(pal);
        }
    }
}

void QtCurveStyle::resetInactivePal(QWidget *widget) const
{
    if(!opts.inactiveHighlight)
    {
        QPalette pal(widget->palette());

        if(pal.color(QPalette::Inactive, QColorGroup::Highlight)==itsInactiveHighlightCol)
        {
            pal.setColor(QPalette::Inactive, QColorGroup::Highlight,
                         pal.color(QPalette::Active, QColorGroup::Highlight));
            pal.setColor(QPalette::Inactive, QColorGroup::HighlightedText,
                         pal.color(QPalette::Active, QColorGroup::HighlightedText));
        }
    }
}

void QtCurveStyle::updateProgressPos()
{
    // Taken from lipstik!
    QMap<QWidget*, int>::iterator it(itsProgAnimWidgets.begin()),
                                  end(itsProgAnimWidgets.end());
    bool                          visible(false);
    for (; it!=end; ++it)
    {
        if (!::qt_cast<QProgressBar*>(it.key()))
            continue;

        QProgressBar *pb(::qt_cast<QProgressBar*>(it.key()));

        if (it.key() -> isEnabled() && pb -> progress()!=pb->totalSteps())
        {
            // update animation Offset of the current Widget
            it.data() = (it.data() + (QApplication::reverseLayout() ? -1 : 1))
                        % (PROGRESS_CHUNK_WIDTH*2);
            it.key()->update();
        }
        if (it.key()->isVisible())
            visible = true;
    }
    if (!visible)
        itsAnimationTimer->stop();
}

void QtCurveStyle::progressBarDestroyed(QObject *bar)
{
    itsProgAnimWidgets.remove(static_cast<QWidget*>(bar));
}

void QtCurveStyle::sliderThumbMoved(int)
{
    QSlider *slider(::qt_cast<QSlider*>(sender()));

    if(slider)
        slider->repaint(false);
}

#include "qtcurve.moc"

